package wafly.framework.storyTK;

import java.util.ArrayList;
import wafly.framework.storyTK.basic.Config;
import wafly.framework.storyTK.basic.DrawableHelper;
import wafly.framework.storyTK.basic.StoreMgr;
import wafly.framework.storyTK.basic.Util;
import android.annotation.SuppressLint;
import android.content.res.AssetFileDescriptor;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.PixelFormat;
import android.graphics.PorterDuff.Mode;
import android.graphics.Rect;
import android.media.MediaPlayer;
import android.net.Uri;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.widget.AbsoluteLayout;

/**
 * With this class you can control your application.
 * @author Wafly
 */
@SuppressLint("ViewConstructor")
@SuppressWarnings("deprecation")
public final class Arena extends SurfaceView
{
	private static Arena instance;
	private static long startTimeMillis4Second;
	private static int currentFPS=0, sleepFrame=0;
	private static String msg = "[null]";
	private static Canvas mainCanvas=null;
	private static DrawableHelper threadDrawable = new DrawableHelper(1,1), completedDrawable = new DrawableHelper(1,1);
	private static Scene scene = null;
	private static STK iContext = null;
	private static MediaPlayer musicPlayer = null;
	private static boolean threadRunning = false, showLock = false;
	private static SurfaceHolder holder = null;
	static ArrayList<String> refuseList = new ArrayList<String>();
	static boolean moving=false;
	private static String musicPath = null;
//	private static int frameLocation_X=0, frameLocation_Y=0;
	static boolean isInit=false;
	static Rect drawSourceRect = null, drawDestRect = null;
	static MediaPlayer videoPlayer = null;
	
	/**
	 * 获取舞台实例，有且仅有一个；此对象属于SurfaceView类型
	 */
	public static Arena getInstance() { return instance; }
	
	/**
	 * It returns Arena's Context which being converted to Activity type, and it always is STK
	 * @return Arena's Activity
	 */
	public static STK getContainer()
	{
		return iContext;
	}
	
	/**
	 * Set Arena to refuse the events via touch screen or not.
	 * @param refuse TRUE is to refuse to touch screen, FALSE is to enable the touching
	 */
	public static void refuseInput(String tag, boolean isRefuse)
	{
		if(isRefuse&&!refuseList.contains(tag))
			refuseList.add(tag);
		else if(!isRefuse&&refuseList.contains(tag))
			refuseList.remove(tag);
	}
	
	/**
	 * 播放背景音乐
	 * @param path 音乐文件路径
	 * 当音乐文件在assets文件夹中时，需要使用相对路径；
	 * 当音乐文件在SD卡中时，可以使用绝对路径；
	 * 但如果已经设置了游戏根目录，则可以使用相对路径或绝对路径，区别为是否以“/”开头
	 */
	public static void playMusic(String path)
	{
		musicPath = Util.verifyFilePath(path);
		if(musicPath==null)
		{
			Util.logError("Fail to locate music file.", null);
			return;
		}
		
		try {
			musicPlayer.stop();
			musicPlayer.release();
		} catch (Exception e) { }
		musicPlayer = null;
		if(musicPath==null && musicPlayer!=null)
			return;
		new Thread(){
			public void run() {
				if(musicPath.startsWith("/"))
				{
					musicPlayer = MediaPlayer.create(iContext, Uri.parse(musicPath));
					musicPlayer.setLooping(true);
					musicPlayer.start();
				}
				else
				{
					musicPlayer = new MediaPlayer();
					AssetFileDescriptor fd = Util.openInnerDescriptor(musicPath);
					try {
						musicPlayer.setDataSource(fd.getFileDescriptor(), fd.getStartOffset(),fd.getLength());
						musicPlayer.prepare();
						fd.close();
					} catch (Exception e) {
						Util.logError("Fail to pley music ["+musicPath+"].", e);
					}
					musicPlayer.setLooping(true);
					musicPlayer.start();
				}
			}
		}.start();
	}
	
	/**
	 * 停止播放当前音乐
	 */
	public static void stopMusic()
	{
		if(musicPlayer != null)
		{
			try {
				musicPlayer.stop();
				musicPlayer.release();
				musicPlayer=null;
			} catch (Exception e) { }
		}
	}

	/**
	 * 播放视频文件，此方法会比较迅速地返回（不会等待视频播放完毕，但是需要一些初始化时间）;
	 * 一旦开始播放，任何画面操作（除了强制停止视频）可以继续进行，但不会反应到显示中。
	 * @param moviePath 视频文件路径
	 * 当视频文件在assets文件夹中时，需要使用相对路径；
	 * 当视频文件在SD卡中时，可以使用绝对路径；
	 * 但如果已经设置了游戏根目录，则可以使用相对路径或绝对路径，区别为是否以“/”开头
	 */
	public static void playMovie(String moviePath)
	{
		if(!Util.isVideoEnable())
		{
			Util.logError("Video function being disabled.", null);
			return;
		}
		moviePath = Util.verifyFilePath(moviePath);
		if(moviePath==null)
		{
			moving=false;
			return;
		}
		iContext.videoViewHandler.post(addVideoViewRunnable);
		try {
			if(moviePath.startsWith("/"))
			{
				videoPlayer.setDataSource(moviePath);
			}
			else
			{
				AssetFileDescriptor fd = Util.openInnerDescriptor(moviePath);
				videoPlayer.setDataSource(fd.getFileDescriptor(), fd.getStartOffset(),fd.getLength());
				fd.close();
			}
//			videoPlayer.setDisplay(STK.videoViewHolder);  
//			videoPlayer.setAudioStreamType(AudioManager.STREAM_MUSIC);
//			videoPlayer.setOnPreparedListener(new OnPreparedListener() { public void onPrepared(MediaPlayer mp) {
//            		Util.logInfo("Video start.");
//            		videoPlayer.start();
//				}});
			moving=true;
			videoPlayer.prepare();
			final int duration = videoPlayer.getDuration();
			Util.logInfo("Video duration: "+duration);
			new Thread(){ public void run() {
				while(videoPlayer.getCurrentPosition()<duration&&moving);
        		Util.logInfo("Video completed.");
        		moving=false;
        		videoPlayer.stop();
//        		videoPlayer.release();
        		iContext.videoViewHandler.post(removeVideoViewRunnable);
			}}.start();
//			new Thread(){public void run(){
//				synchronized (this) {
//					try {
//						Thread.sleep(duration);
//					} catch (Exception e) { }
//					moving=false;
//		    		iContext.videoViewHandler.post(removeVideoViewRunnable);
//				}
//			}}.start();
		} catch (Exception e) {
			Util.logError("Video error code "+e.getStackTrace()[0].getLineNumber()+", "+e.getMessage(), e);
    		moving=false;
			videoPlayer.stop();
//			videoPlayer.release();
    		iContext.videoViewHandler.post(removeVideoViewRunnable);
		}
	}
	
	private static Runnable addVideoViewRunnable=new Runnable(){
        public void run() {
//    		iContext.layout.addView(iContext.videoView); 
    		iContext.videoView.setVisibility(View.VISIBLE);
        }
    };
	private static Runnable removeVideoViewRunnable=new Runnable(){
        public void run() {
//    		iContext.layout.removeView(iContext.videoView); 
    		iContext.videoView.setVisibility(View.INVISIBLE);
        }
    };
	
	public static boolean isMoviePlaying()
	{
		return moving;
	}
	
	/**
	 * 销毁当前的scene并切换到新的scene
	 */
	public static void setScene(Scene newScene)
	{
		if(newScene==null)
		{
			Util.logError("New scene could not be null.", null);
			return;
		}
		if(scene!=null)
			scene.destroy();
		scene = newScene;
		scene.onLoadSceneCompleted();
	}
	
	/**
	 * 获取当前scene
	 * @return current scene
	 */
	public static Scene getScene() { return scene; }
	
	/**
	 * set the message which shown on screen, it will enable only the debug is ecabled in Config.
	 * @param message shown content
	 */
	public static void setMessage(String message) { msg = message==null?"[null]":message; }
	
	/**
	 * get the message which shown on screen, it will enable only the debug is ecabled in Config.
	 * @return
	 */
	public static String getMessage() { return msg; }
	
	/**
	 * 获取当前已经按下的按键
	 * @return
	 */
	public static ArrayList<Integer> getPressedKeys()
	{
		return iContext.keysPressed;
	}
	
	/**
	 * 取消一个已经按下的按键
	 * @param keyCode
	 */
	public static void removePressedKey(int keyCode)
	{
		iContext.removePressedKey(keyCode);
	}
	
	/**
	 * 使Arena对象的运作暂停一段时间
	 * @param frameCount 暂停的帧数，数值须大于0
	 */
	public static void stop(int frameCount)
	{
		sleepFrame = frameCount>0?frameCount:1;
		while(sleepFrame>0);
	}
	
	/**
	 * Start the Arena performing, but please mind the Arena only cna start once.
	 */
	static void start()
	{
		if(!threadRunning)
		{
			Util.logError("Arena already shutdown.", null);
			return;
		}

		final long startTime=System.currentTimeMillis();
		
		new Thread() { public void run() 
		{
				long prepareFrameStartTime=0;
				long prepareFrameDuration=0, drawFrameDuration=0;
				int fontHeight = Util.getGlobalFontSize();
				for(int i=0; threadRunning; i++)
				{
					prepareFrameStartTime = System.currentTimeMillis();
					
					threadDrawable.drawColor(Util.getBackgroundColor());
					scene.createFrame(threadDrawable);
					if (Util.isDebug()) {
						if (prepareFrameStartTime - startTimeMillis4Second > 1000) {
							currentFPS = i;
							i = 0;
							startTimeMillis4Second = System.currentTimeMillis();
							prepareFrameDuration = System.currentTimeMillis() - prepareFrameStartTime;
						}
						threadDrawable.setForeColor(Util.getInverseColorValue(Util.getBackgroundColor()));
						threadDrawable.drawString("Screen Size: " + instance.getWidth() + "x" + instance.getHeight(), 0, 0);
						threadDrawable.drawString("Game Size: " + Util.getGameWidth() + "x" + Util.getGameHeight(), 0, (int) (fontHeight));
						threadDrawable.drawString("Refresh: " + currentFPS + " FPS", 0, (int) (fontHeight * 2));
						threadDrawable.drawString("Game Time: " + (System.currentTimeMillis() - startTime) + " ms", 0, (int) (fontHeight * 3));
						threadDrawable.drawString("Back Local: " + scene.getBackgroundLocation_X() + "," + scene.getBackgroundLocation_Y(), 0, (int) (fontHeight * 4));
						threadDrawable.drawString("Prepare Duration: " + prepareFrameDuration + " ms", 0, (int) (fontHeight * 5));
						threadDrawable.drawString("Draw Duration: " + drawFrameDuration + " ms", 0, (int) (fontHeight * 6));
						threadDrawable.drawString("Message: " + msg, 0, (int) (fontHeight * 7));
					} else {
						i = 1;
					}
					showLock = true;
					completedDrawable.drawImage(threadDrawable,STK.touchRelaX,STK.touchRelaY);
					showLock = false;
					if(Util.isDebug() && prepareFrameStartTime <= startTimeMillis4Second)
						drawFrameDuration = System.currentTimeMillis() - prepareFrameStartTime - prepareFrameDuration;
					synchronized (this)
					{
						long frameLong = 1000 / Util.getMaxFPS() - System.currentTimeMillis() + prepareFrameStartTime;
						if(frameLong<1) frameLong=1;
						try
						{
							if(sleepFrame>0)
							{
								Thread.sleep(sleepFrame*1000/Util.getMaxFPS());
								sleepFrame=0;
							}
							else
							{
								Thread.sleep(frameLong);
							}
							while(moving)
								Thread.sleep(1000/Util.getMaxFPS());
						}
						catch (Exception e)
						{
							Util.logError("Error in thread.", e);
						}
					}
				}
				scene.destroy();
				scene=null;
				Util.logInfo("Arena stopped.");
		}}.start();

		new Thread()
		{
			public void run()
			{
				Paint mainPaint = new Paint();
				while (threadRunning)
				{
					if (!showLock)
					{
						mainCanvas=holder.lockCanvas();
						if(mainCanvas!=null)
						{
							if(moving){
								mainCanvas.drawColor(Color.TRANSPARENT,Mode.DST_IN);
							}else{
//								mainCanvas.drawBitmap(completedDrawable.getSource(), 0, 0, mainPaint);
								if(Config.isFixedDraw()){
									mainCanvas.drawBitmap(completedDrawable.getSource(),
											new Rect(0, 0, completedDrawable.getWidth(), completedDrawable.getHeight()),
											new Rect(0, 0, instance.getWidth(), instance.getHeight()), mainPaint);
								}else{
									mainCanvas.drawBitmap(completedDrawable.getSource(), 0, 0, mainPaint);
								}
							}
						}
						if(mainCanvas!=null)
							holder.unlockCanvasAndPost(mainCanvas);
					}
				}
			}
		}.start();
		
		Util.logInfo("Arena started.");
	}
	
	/**
	 * 完全停止Arena的运行
	 */
	public static void shutdown()
	{
		if (iContext.onExit())
		{
			moving = false;
			threadRunning = false;
			
			if(scene!=null){
				try {
					scene.closeAllPanel();
					scene.destroy();
				} catch (Exception e) { }
			}
			if(threadDrawable!=null)
				try {
					threadDrawable.destroy();
				} catch (Exception e) { }
			if(completedDrawable!=null)
				try {
					completedDrawable.destroy();
				} catch (Exception e) { }
			iContext.finish();
		}
	}

	/**
	 * 保存当前游戏状态
	 */
	public static void save(StoreMgr storeMgr)
	{
		if(scene==null)
			return;
		refuseInput("Arena.save", true);
		Util.logInfo("Start to save");
			storeMgr.setValue("ARENA.MUSIC", musicPath);
			storeMgr.setValue("ARENA.SCENE", scene.getClass().getName());
			scene.storeValue(storeMgr);
			Util.logInfo("Save completed");
		refuseInput("Arena.save", false);
	}

	/**
	 * 读取已保存的游戏状态
	 */
	public static void load(StoreMgr storeMgr)
	{
		refuseInput("Arena.load", true);
		Util.logInfo("Start to load");
		playMusic(storeMgr.getValue("ARENA.MUSIC"));
		try
		{
			Scene scene = (Scene)Class.forName(storeMgr.getValue("ARENA.SCENE")).newInstance();
			scene.restoreValue(storeMgr);
			setScene(scene);
		} catch (Exception e)
		{
			Util.logError("Fail to load game status.", e);
		}
		refuseInput("Arena.load", false);
	}
	
	Arena(STK context)
	{
		super(context);
		instance = this;
		holder = this.getHolder();
		holder.setFormat(PixelFormat.TRANSLUCENT);
		holder.addCallback(new SurfaceHolder.Callback()
		{
			public void surfaceChanged(SurfaceHolder holder, int format, int width, int height)
			{
				if(isInit)
					return;
				int x=0, y=0, distWidth=getWidth(), distHeight=getHeight();
				switch (Util.getDisplayMode())
				{
				case ORIGINAL:
					x=(getWidth() - Util.getGameWidth()) / 2;
					distWidth=Util.getGameWidth();
					y=(getHeight() - Util.getGameHeight()) / 2;
					distHeight=Util.getGameHeight();
					break;
				case ZOOM:
					if ((float) distWidth / (float) distHeight > (float) Util.getGameWidth() / (float) Util.getGameHeight()){
						distWidth = (int) (distHeight * (float) Util.getGameWidth() / (float) Util.getGameHeight());
						x = (getWidth() - distWidth) / 2;
					} else {
						distHeight = (int) (distWidth * (float) Util.getGameHeight() / (float) Util.getGameWidth());
						y = (getHeight() - distHeight) / 2;
					}
//					Config.setScale((float)distWidth/(float)Util.getGameWidth(), (float)distHeight/(float)Util.getGameHeight());
					
					break;
				case STRETCH:
//					Config.setScale((float)distWidth/(float)Util.getGameWidth(), (float)distHeight/(float)Util.getGameHeight());
					
					break;
				}
				Config.dimScreenSize(distWidth, distHeight);
				drawDestRect = new Rect(x,y,distWidth, distHeight);
				STK.touchRelaX=x;
				STK.touchRelaY=y;
				setLayoutParams(new AbsoluteLayout.LayoutParams(distWidth, distHeight, x, y));
				Matrix matrix=new Matrix();
//				matrix.postScale(Util.getWidthScale(), Util.getHeightScale());
				Config.setMatrix(matrix);
				
//				threadDrawable = new DrawableHelper(getWidth(), getHeight());
//				completedDrawable = new DrawableHelper(getWidth(), getHeight());
				if(Config.isFixedDraw()){
					threadDrawable = new DrawableHelper(Util.getMaxSupportScreenWidth(), Util.getMaxSupportScreenHeight());
					completedDrawable = new DrawableHelper(Util.getMaxSupportScreenWidth(), Util.getMaxSupportScreenHeight());
				}else{
					threadDrawable = new DrawableHelper(getWidth(), getHeight());
					completedDrawable = new DrawableHelper(getWidth(), getHeight());
				}
				
				Util.logInfo("Arena ["+x+","+y+","+distWidth+","+distHeight+"] created with ["+Util.getDisplayMode()+"] mode");
				isInit=true;
			}
			public void surfaceDestroyed(SurfaceHolder holder) { }
			public void surfaceCreated(SurfaceHolder holder) { }
		});
		iContext = context;
		threadRunning=true;
		
	}

	static void drawGameScreen(DrawableHelper draw)
	{
		draw.drawImage(completedDrawable, 0, 0);
	}
	
	
}
