package wafly.framework.storyTK;

import android.annotation.SuppressLint;
import java.util.ArrayList;

import wafly.framework.storyTK.basic.DrawableHelper;
import wafly.framework.storyTK.basic.KeyCode;
import wafly.framework.storyTK.basic.Util;
import wafly.framework.storyTK.tmpl.Control;
import wafly.framework.storyTK.tmpl.OperationalControl;

/**
 * to show a group of control(include the background) to the scene
 * @author Wafly
 */
public class Panel extends DrawableHelper
{
	private ArrayList<Control> controls = new ArrayList<Control>();
	private ArrayList<OperationalControl> operationals = new ArrayList<OperationalControl>();
	private int alpha=255;
	private boolean destoryed=false;
	private int drawLocal_X = 0, drawLocal_Y = 0;
	
//	/**
//	 * should use this paint to draw your panel
//	 */
//	protected Paint paint = new Paint();

    /**
     * 通过面板背景图片初始化一个面板<br>
     * to init a panel via image, then the panel's size will be the same as the image
     */
    public Panel(String backgroundFilePath)
    {
      super(backgroundFilePath);
      drawLocal_X = (Util.getGameWidth() - getWidth()) / 2;
      drawLocal_Y = (Util.getGameHeight() - getHeight()) / 2;
    }
	
	/**
	 * to init a panel via specified size and specified color.
	 * @param width panel's width
	 * @param height panel's height
	 * @param color panel's background color
	 */
	public Panel(int width, int height, int color)
	{
    	super(width, height);
    	drawColor(color);
        drawLocal_X = (Arena.getInstance().getWidth() - getWidth()) / 2;
        drawLocal_Y = (Arena.getInstance().getHeight() - getHeight()) / 2;
	}

	@Override
	public void zoomTo(int newWidth, int newHeight) {
		super.zoomTo(newWidth, newHeight);
	    drawLocal_X = (Arena.getInstance().getWidth() - getWidth()) / 2;
	    drawLocal_Y = (Arena.getInstance().getHeight() - getHeight()) / 2;
	    for(Control control : controls)
	    	control.setRelative(drawLocal_X, drawLocal_Y);
	}

	@Override
	public void cutTo(int x, int y, int width, int height) {
		super.cutTo(x, y, width, height);
	    drawLocal_X = (Arena.getInstance().getWidth() - getWidth()) / 2;
	    drawLocal_Y = (Arena.getInstance().getHeight() - getHeight()) / 2;
	    for(Control control : controls)
	    	control.setRelative(drawLocal_X, drawLocal_Y);
	}

	/**
	 * add one control to this panel
	 * @param control
	 */
	public void addControl(Control control)
	{
		if(control==null)
			return;
		control.setRelative(drawLocal_X, drawLocal_Y);
		controls.add(control);
		if(control instanceof OperationalControl)
			operationals.add((OperationalControl)control);
	}
	
	/**
	 * to remove the specified exist control from current panel, but first you should has this control's instance 
	 * @param control
	 */
	public void removeControl(Control control)
	{
		controls.remove(control);
		if(operationals.contains(control))
			operationals.remove((OperationalControl)control);
	}
	
	/**
	 * 销毁此Panel所包含的资源，包括其中所有控件
	 */
	public void destroy()
	{
		super.destroy();
		destoryed=true;
		for(int i=0; i<controls.size(); i++)
			controls.get(i).destroy();
		controls.clear();
		operationals.clear();
		Util.logInfo("Panel being destroyed.");
	}
	
    /**
     * 按键按下时发生事件
     * @param keyCode
     */
	public boolean onKeyDown(int keyCode)
	{
		if(keyCode==KeyCode.CANCEL)
		{
			close();
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * 长按某个按键时发生事件
	 * @param keyCode
	 */
	public boolean onKeyLongPress(int keyCode)
	{
		return false;
	}
	
	/**
	 * 某个按键弹起时发生事件
	 * @param keyCode
	 */
	public void onKeyUp(int keyCode) {}
	
	/**
	 * 开启显示此面板
	 */
	public void show()
	{
		Arena.getScene().setPanel(this);
	}
	
	/**
	 * 关闭并销毁此面板
	 */
	public void close()
	{
		Panel currPanel = Arena.getScene().getPanel();
		if(currPanel!=null && currPanel==this) {
			Arena.getScene().setPanel(null);
		} else {
			Arena.getScene().removePanel(this);
			this.destroy();
		}
	}

    /**
     * 在画布上画出面板内容
     *
     * @param canvas
     *            画布,面板所有内容所绘的地方
     */
    @SuppressLint("WrongCall")
	public void onDraw(DrawableHelper draw)
    {
        if (destoryed)
            return;
        int tAlpha=draw.getDrawAlpha();
        if (alpha != tAlpha)
            draw.setDrawAlpha(alpha);
        draw.drawImage(this, drawLocal_X, drawLocal_Y);
//        Util.logInfo("drraw panel x:"+drawLocal_X+",y:"+drawLocal_Y);
        if (alpha != tAlpha)
            draw.setDrawAlpha(tAlpha);
        try {
			for (int i = 0; i < controls.size(); i++)
			    controls.get(i).onDraw(draw);
		} catch (Exception e) { }
    }
	
	/**
	 * to get all the controls in this panel
	 * @return controls list
	 */
	public ArrayList<Control> getControls() { return controls; }
	
	/**
	 * to get all the controls from this panel which operational
	 * @return operational control list
	 */
	public ArrayList<OperationalControl> getOperationalControls() { return this.operationals; }
	
	private OperationalControl focusedControl = null;
	public void setFocusControl(OperationalControl control){
		if(focusedControl!=null)
			focusedControl.setFocus(false);
		focusedControl = control;
		focusedControl.setFocus(true);
	}
	
	/**
	 * 设置panel的透明度，同时包括其上的Control也一同设置。
	 * @param alpha transparency
	 */
	public void setAlpha(int alpha)
	{
        if (alpha < 0)
            alpha = 0;
        if (alpha > 255)
            alpha = 255;
		this.alpha=alpha;
	}
	
	/**
	 * to get the current alpha of the panel
	 * @return transparency
	 */
	public int getAlpha() { return alpha; }
    
    /**
     * 面板显示完成时发生事件
     */
    public void onShown() {}
    
    /**
     * 面板完全关闭后发生事件
     */
    public void onClosed() {}
	
}
