package wafly.framework.storyTK;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.TreeMap;

import wafly.framework.storyTK.basic.DrawableHelper;
import wafly.framework.storyTK.basic.StoreMgr;
import wafly.framework.storyTK.basic.Util;
import wafly.framework.storyTK.control.FrameAnimation;
import wafly.framework.storyTK.tmpl.Component;
import wafly.framework.storyTK.tmpl.Control;
import wafly.framework.storyTK.tmpl.Ievent;
import wafly.framework.storyTK.tmpl.OperationalControl;
import android.annotation.SuppressLint;
import android.graphics.Canvas;
import android.media.AudioManager;
import android.media.SoundPool;

/**
 * To display your story
 * @author Wafly
 */
public class Scene
{
	private transient ArrayList<Panel> panels = new ArrayList<Panel>();
	private transient ArrayList<Control> controls = new ArrayList<Control>();
	private transient ArrayList<OperationalControl> operationals = new ArrayList<OperationalControl>();
	private TreeMap<Integer, ArrayList<Component>> components = new TreeMap<Integer, ArrayList<Component>>();
	private int paintAlpha=255, newPanelAlpha=0;
	private int backgroundLocation_X=0, backgroundLocation_Y=0, newBackgroundLocation_X=0, newBackgroundLocation_Y=0;
	protected DrawableHelper backgroundDrawable = new DrawableHelper(Arena.getInstance().getWidth(), Arena.getInstance().getHeight());
	private DrawableHelper newBackgroundDrawable = null;
	private ArrayList<DrawableHelper> back4PanelDrawables = new ArrayList<DrawableHelper>();
	private SoundPool sounds = null;
	private boolean destoryed=false;
	/**控制除了背景外的所有元素的隐藏或显示*/
	private boolean noShow=false;
	private BackgroundSwitchType backgroundSwitchType = BackgroundSwitchType.NORMAL;
	
	public Scene() { }
	/**
	 * 继承此函数可以添加自定义的画面内容<br>
	 * override it to add custom content to the screen.
	 */
	protected void onDraw(DrawableHelper draw) {};
	/**
	 * 当场景加载完毕后发生事件
	 */
	protected void onLoadSceneCompleted() {};

	/**
	 * 切换当前背景.这是一个阻塞方法,执行这个方法会阻塞当前线程，等待动态效果，直到新背景切换完毕，才退出方法
	 * @param newBackgroundPath 新背景
	 * @param x 新背景放置的位置(X轴)
	 * @param y 新背景放置的位置(Y轴)
	 * @param maskLayerPath 用于遮罩层的图片, 传入null则不使用遮罩层
	 */
	public synchronized void setBackground(
			DrawableHelper newBackgroundDraw, int x, int y, DrawableHelper maskLayerDraw)
	{
		if(Arena.getScene()!=this)
		{
			Util.logError("The scene is not the current running scene, perform scene too early?", null);
			return;
		}
		noShow = true;
		newPanelAlpha = 0;
		paintAlpha = 255;
		try
		{
			switch (backgroundSwitchType)
			{
				case NORMAL:
					if (maskLayerDraw == null)
					{
						while (paintAlpha > 0)
						{
							paintAlpha = paintAlpha < Util.getSwitchSpeed() ? 0 : paintAlpha - Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}

						backgroundDrawable = newBackgroundDraw;
						backgroundLocation_X = x;
						backgroundLocation_Y = y;

						while (paintAlpha < 255)
						{
							paintAlpha = paintAlpha > (255 - Util.getSwitchSpeed()) ? 255 : paintAlpha + Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}
					}
					else
					{
						newBackgroundDrawable = maskLayerDraw;
						while (newPanelAlpha < 255)
						{
							newPanelAlpha = newPanelAlpha > (255 - Util.getSwitchSpeed()) ? 255 : newPanelAlpha + Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}
						while (paintAlpha > 0)
						{
							paintAlpha = paintAlpha < Util.getSwitchSpeed() ? 0 : paintAlpha - Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}

						backgroundDrawable = newBackgroundDraw;
						backgroundLocation_X = x;
						backgroundLocation_Y = y;

						while (paintAlpha < 255 || newPanelAlpha < 255)
						{
							paintAlpha = paintAlpha > (255 - Util.getSwitchSpeed()) ? 255 : paintAlpha + Util.getSwitchSpeed();
							newPanelAlpha = newPanelAlpha > (255 - Util.getSwitchSpeed()) ? 255 : newPanelAlpha + Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}
						while (newPanelAlpha > 0)
						{
							newPanelAlpha = newPanelAlpha < Util.getSwitchSpeed() ? 0 : newPanelAlpha - Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}
					}

					break;
				case DIRECT:
					if (newBackgroundDraw == null)
					{
						while (paintAlpha > 0)
						{
							paintAlpha = paintAlpha < Util.getSwitchSpeed() ? 0 : paintAlpha - Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}
//						backgroundDrawable.drawColor(Util.getBackgroundColor());
						backgroundDrawable=null;
					}
					else
					{
						newBackgroundDrawable = newBackgroundDraw.getPart(x, y, Arena.getInstance().getWidth(), Arena.getInstance().getHeight());
						while (newPanelAlpha < 255)
						{
							newPanelAlpha = newPanelAlpha > (255 - Util.getSwitchSpeed()) ? 255 : newPanelAlpha + Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}
					}
					backgroundDrawable = newBackgroundDraw;
					paintAlpha = 255;

					break;
				case TOP_TOP:
					if (maskLayerDraw == null)
					{
						newBackgroundLocation_Y = -Arena.getInstance().getHeight();
						newPanelAlpha = 255;
						if (newBackgroundDraw == null)
						{
							newBackgroundDrawable = new DrawableHelper(Arena.getInstance().getWidth(), Arena.getInstance().getHeight());
							newBackgroundDrawable.drawColor(Util.getBackgroundColor());
						}
						else
						{
							newBackgroundDrawable = newBackgroundDraw.getPart(x, y, Arena.getInstance().getWidth(), Arena.getInstance().getHeight());
						}
						while (newBackgroundLocation_Y < 0)
						{
							newBackgroundLocation_Y = (newBackgroundLocation_Y > -Util.getSwitchSpeed()) ? 0 : newBackgroundLocation_Y + Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}
						backgroundDrawable = newBackgroundDraw;
					}
					else
					{
						newBackgroundLocation_Y = -(Arena.getInstance().getHeight() * 2);
						newPanelAlpha = 255;
						backgroundDrawable = maskLayerDraw;
						while (newBackgroundLocation_Y < 0)
						{
							newBackgroundLocation_Y = (newBackgroundLocation_Y > -Util.getSwitchSpeed()) ? 0 : newBackgroundLocation_Y + Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}

						backgroundDrawable = newBackgroundDraw;

						while (newBackgroundLocation_Y > -(Arena.getInstance().getHeight() * 2))
						{
							newBackgroundLocation_Y = newBackgroundLocation_Y - Util.getSwitchSpeed() < -(Arena.getInstance().getHeight() * 2) ? -(Arena.getInstance().getHeight() * 2) : newBackgroundLocation_Y - Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}
					}
					break;
				case BOTTOM_BOTTOM:
					newBackgroundLocation_Y = Arena.getInstance().getHeight();
					newPanelAlpha = 255;
					if (maskLayerDraw == null)
					{
						if (newBackgroundDraw == null)
						{
							newBackgroundDrawable = new DrawableHelper(Arena.getInstance().getWidth(), Arena.getInstance().getHeight());
							newBackgroundDrawable.drawColor(Util.getBackgroundColor());
						}
						else
						{
							newBackgroundDrawable = newBackgroundDraw.getPart(x, y, Arena.getInstance().getWidth(), Arena.getInstance().getHeight());
						}
						while (newBackgroundLocation_Y > 0)
						{
							newBackgroundLocation_Y = (newBackgroundLocation_Y < Util.getSwitchSpeed()) ? 0 : newBackgroundLocation_Y - Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}
						backgroundDrawable = newBackgroundDraw;
					}
					else
					{
						backgroundDrawable = maskLayerDraw;
						while (newBackgroundLocation_Y > -Arena.getInstance().getHeight())
						{
							newBackgroundLocation_Y = (newBackgroundLocation_Y + Arena.getInstance().getHeight() < Util.getSwitchSpeed()) ? -Arena.getInstance().getHeight() : newBackgroundLocation_Y - Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}

						backgroundDrawable = newBackgroundDraw;

						while (newBackgroundLocation_Y < Arena.getInstance().getHeight())
						{
							newBackgroundLocation_Y = newBackgroundLocation_Y + Util.getSwitchSpeed() > Arena.getInstance().getHeight() ? Arena.getInstance().getHeight() : newBackgroundLocation_Y + Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}
					}
					break;
				case LEFT_LEFT:
					if (maskLayerDraw == null)
					{
						newBackgroundLocation_X = -Arena.getInstance().getWidth();
						newPanelAlpha = 255;
						if (newBackgroundDraw == null)
						{
							newBackgroundDrawable = new DrawableHelper(Arena.getInstance().getWidth(), Arena.getInstance().getHeight());
							newBackgroundDrawable.drawColor(Util.getBackgroundColor());
						}
						else
						{
							newBackgroundDrawable = newBackgroundDraw.getPart(x, y, Arena.getInstance().getWidth(), Arena.getInstance().getHeight());
						}
						while (newBackgroundLocation_X < 0)
						{
							newBackgroundLocation_X = (newBackgroundLocation_X > -Util.getSwitchSpeed()) ? 0 : newBackgroundLocation_X + Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}
						backgroundDrawable = newBackgroundDraw;
					}
					else
					{
						newBackgroundLocation_X = -(Arena.getInstance().getWidth() * 2);
						newPanelAlpha = 255;
						backgroundDrawable = maskLayerDraw;
						while (newBackgroundLocation_X < 0)
						{
							newBackgroundLocation_X = (newBackgroundLocation_X > -Util.getSwitchSpeed()) ? 0 : newBackgroundLocation_X + Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}

						backgroundDrawable = newBackgroundDraw;

						while (newBackgroundLocation_X > -(Arena.getInstance().getWidth() * 2))
						{
							newBackgroundLocation_X = newBackgroundLocation_X - Util.getSwitchSpeed() < -(Arena.getInstance().getWidth() * 2) ? -(Arena.getInstance().getWidth() * 2) : newBackgroundLocation_X - Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}
					}
					break;
				case RIGHT_RIGHT:
					newBackgroundLocation_X = Arena.getInstance().getWidth();
					newPanelAlpha = 255;
					if (maskLayerDraw == null)
					{
						if (newBackgroundDraw == null)
						{
							newBackgroundDrawable = new DrawableHelper(Arena.getInstance().getWidth(), Arena.getInstance().getHeight());
							newBackgroundDrawable.drawColor(Util.getBackgroundColor());
						}
						else
						{
							newBackgroundDrawable = newBackgroundDraw.getPart(x, y, Arena.getInstance().getWidth(), Arena.getInstance().getHeight());
						}
						while (newBackgroundLocation_X > 0)
						{
							newBackgroundLocation_X = (newBackgroundLocation_X < Util.getSwitchSpeed()) ? 0 : newBackgroundLocation_X - Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}
						backgroundDrawable = newBackgroundDraw;
					}
					else
					{
						backgroundDrawable = maskLayerDraw;
						while (newBackgroundLocation_X > -Arena.getInstance().getWidth())
						{
							newBackgroundLocation_X = (newBackgroundLocation_X + Arena.getInstance().getWidth() < Util.getSwitchSpeed()) ? -Arena.getInstance().getWidth() : newBackgroundLocation_X - Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}

						backgroundDrawable = newBackgroundDraw;

						while (newBackgroundLocation_X < Arena.getInstance().getWidth())
						{
							newBackgroundLocation_X = newBackgroundLocation_X + Util.getSwitchSpeed() > Arena.getInstance().getWidth() ? Arena.getInstance().getWidth() : newBackgroundLocation_X + Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}
					}
					break;
				case TOP_BOTTOM:
					if (maskLayerDraw == null)
					{
						newBackgroundLocation_Y = -Arena.getInstance().getHeight();
						newPanelAlpha = 255;
						if (newBackgroundDraw == null)
						{
							newBackgroundDrawable = new DrawableHelper(Arena.getInstance().getWidth(), Arena.getInstance().getHeight());
							newBackgroundDrawable.drawColor(Util.getBackgroundColor());
						}
						else
						{
							newBackgroundDrawable = newBackgroundDraw.getPart(x, y, Arena.getInstance().getWidth(), Arena.getInstance().getHeight());
						}
						while (newBackgroundLocation_Y < 0)
						{
							newBackgroundLocation_Y = (newBackgroundLocation_Y > -Util.getSwitchSpeed()) ? 0 : newBackgroundLocation_Y + Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}
						backgroundDrawable = newBackgroundDraw;
					}
					else
					{
						newBackgroundLocation_Y = -(Arena.getInstance().getHeight() * 2);
						newPanelAlpha = 255;
						backgroundDrawable = maskLayerDraw;
						while (newBackgroundLocation_Y < 0)
						{
							newBackgroundLocation_Y = (newBackgroundLocation_Y > -Util.getSwitchSpeed()) ? 0 : newBackgroundLocation_Y + Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}

						backgroundDrawable = newBackgroundDraw;

						while (newBackgroundLocation_Y < Arena.getInstance().getHeight())
						{
							newBackgroundLocation_Y = newBackgroundLocation_Y + Util.getSwitchSpeed() > Arena.getInstance().getHeight() ? Arena.getInstance().getHeight() : newBackgroundLocation_Y + Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}
					}
					break;
				case BOTTOM_TOP:
					newBackgroundLocation_Y = Arena.getInstance().getHeight();
					newPanelAlpha = 255;
					if (maskLayerDraw == null)
					{
						if (newBackgroundDraw == null)
						{
							newBackgroundDrawable = new DrawableHelper(Arena.getInstance().getWidth(), Arena.getInstance().getHeight());
							newBackgroundDrawable.drawColor(Util.getBackgroundColor());
						}
						else
						{
							newBackgroundDrawable = newBackgroundDraw.getPart(x, y, Arena.getInstance().getWidth(), Arena.getInstance().getHeight());
						}
						while (newBackgroundLocation_Y > 0)
						{
							newBackgroundLocation_Y = (newBackgroundLocation_Y < Util.getSwitchSpeed()) ? 0 : newBackgroundLocation_Y - Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}
						backgroundDrawable = newBackgroundDraw;
					}
					else
					{
						backgroundDrawable = maskLayerDraw;
						while (newBackgroundLocation_Y > -Arena.getInstance().getHeight())
						{
							newBackgroundLocation_Y = (newBackgroundLocation_Y + Arena.getInstance().getHeight() < Util.getSwitchSpeed()) ? -Arena.getInstance().getHeight() : newBackgroundLocation_Y - Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}

						backgroundDrawable = newBackgroundDraw;

						while (newBackgroundLocation_Y > -(Arena.getInstance().getHeight() * 2))
						{
							newBackgroundLocation_Y = newBackgroundLocation_Y - Util.getSwitchSpeed() < -(Arena.getInstance().getHeight() * 2) ? -(Arena.getInstance().getHeight() * 2) : newBackgroundLocation_Y - Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}
					}
					break;
				case LEFT_RIGHT:
					if (maskLayerDraw == null)
					{
						newBackgroundLocation_X = -Arena.getInstance().getWidth();
						newPanelAlpha = 255;
						if (newBackgroundDraw == null)
						{
							newBackgroundDrawable = new DrawableHelper(Arena.getInstance().getWidth(), Arena.getInstance().getHeight());
							newBackgroundDrawable.drawColor(Util.getBackgroundColor());
						}
						else
						{
							newBackgroundDrawable = newBackgroundDraw.getPart(x, y, Arena.getInstance().getWidth(), Arena.getInstance().getHeight());
						}
						while (newBackgroundLocation_X < 0)
						{
							newBackgroundLocation_X = (newBackgroundLocation_X > -Util.getSwitchSpeed()) ? 0 : newBackgroundLocation_X + Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}
						backgroundDrawable = newBackgroundDraw;
					}
					else
					{
						newBackgroundLocation_X = -(Arena.getInstance().getWidth() * 2);
						newPanelAlpha = 255;
						backgroundDrawable = maskLayerDraw;
						while (newBackgroundLocation_X < 0)
						{
							newBackgroundLocation_X = (newBackgroundLocation_X > -Util.getSwitchSpeed()) ? 0 : newBackgroundLocation_X + Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}

						backgroundDrawable = newBackgroundDraw;

						while (newBackgroundLocation_X < Arena.getInstance().getWidth())
						{
							newBackgroundLocation_X = newBackgroundLocation_X + Util.getSwitchSpeed() > Arena.getInstance().getWidth() ? Arena.getInstance().getWidth() : newBackgroundLocation_X + Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}
					}
					break;
				case RIGHT_LEFT:
					newBackgroundLocation_X = Arena.getInstance().getWidth();
					newPanelAlpha = 255;
					if (maskLayerDraw == null)
					{
						if (newBackgroundDraw == null)
						{
							newBackgroundDrawable = new DrawableHelper(Arena.getInstance().getWidth(), Arena.getInstance().getHeight());
							newBackgroundDrawable.drawColor(Util.getBackgroundColor());
						}
						else
						{
							newBackgroundDrawable = newBackgroundDraw.getPart(x, y, Arena.getInstance().getWidth(), Arena.getInstance().getHeight());
						}
						while (newBackgroundLocation_X > 0)
						{
							newBackgroundLocation_X = (newBackgroundLocation_X < Util.getSwitchSpeed()) ? 0 : newBackgroundLocation_X - Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}
						backgroundDrawable = newBackgroundDraw;
					}
					else
					{
						backgroundDrawable = maskLayerDraw;
						while (newBackgroundLocation_X > -Arena.getInstance().getWidth())
						{
							newBackgroundLocation_X = (newBackgroundLocation_X + Arena.getInstance().getWidth() < Util.getSwitchSpeed()) ? -Arena.getInstance().getWidth() : newBackgroundLocation_X - Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}

						backgroundDrawable = newBackgroundDraw;

						while (newBackgroundLocation_X > -(Arena.getInstance().getWidth() * 2))
						{
							newBackgroundLocation_X = newBackgroundLocation_X - Util.getSwitchSpeed() < -(Arena.getInstance().getWidth() * 2) ? -(Arena.getInstance().getWidth() * 2) : newBackgroundLocation_X - Util.getSwitchSpeed();
							wait(1000 / Util.getMaxFPS());
						}
					}
					break;
			}
		} catch (Exception e)
		{
			e.printStackTrace();
			Util.logError("Error while switch background.", e);
			backgroundDrawable.drawColor(Util.getBackgroundColor());
		} finally
		{
			newPanelAlpha = 0;
			if (newBackgroundDrawable != null)
			{
				newBackgroundDrawable.destroy();
				newBackgroundDrawable = null;
			}
			noShow = false;
		}
	}
	
	/**
	 * 如果需要实现自定义scene，则需要重写这个方法，并在canvas中绘制自定义内容。
	 */
	protected void onDraw(Canvas canvas){};

	/**
	 * 以指定速度移动背景到指定位置, 这是一个阻塞方法, 会阻塞当前线程直到移动结束.
	 * @param x 新的背景位置(X轴)
	 * @param y 新的背景位置(Y轴)
	 * @param speed 移动速度(只是一个大概的数值,没有太大参考价值)
	 */
	public synchronized void moveBackground(int x, int y, int speed)
	{
		while (backgroundLocation_X != x || backgroundLocation_X != y)
		{
			if (backgroundLocation_X < x)
			{
				backgroundLocation_X += speed;
				if (backgroundLocation_X > x)
					backgroundLocation_X = x;
			}
			else if (backgroundLocation_X > x)
			{
				backgroundLocation_X -= speed;
				if (backgroundLocation_X < x)
					backgroundLocation_X = x;
			}
			if (backgroundLocation_X < y)
			{
				backgroundLocation_X += speed;
				if (backgroundLocation_X > y)
					backgroundLocation_X = y;
			}
			else if (backgroundLocation_X > y)
			{
				backgroundLocation_X -= speed;
				if (backgroundLocation_X < y)
					backgroundLocation_X = y;
			}
			try {
				wait(1000 / Util.getMaxFPS());
			} catch (Exception e) {}
		}
	}
	
	public DrawableHelper getBackground(){ return backgroundDrawable; }

	/**
	 * 直接将背景移动到指定位置
	 * @param x 新的显示位置(X轴)
	 * @param y 新的显示位置(Y轴)
	 */
	public void moveBackground(int x, int y)
	{
		backgroundLocation_X = x;
		backgroundLocation_Y = y;
	}

	/**
	 * 获取当前背景位置(X轴)
	 */
	public int getBackgroundLocation_X()
	{
		return backgroundLocation_X;
	}

	/**
	 * 获取当前背景位置(Y轴)
	 */
	public int getBackgroundLocation_Y()
	{
		return backgroundLocation_Y;
	}
	
	/**
	 * The way for old background switch to new background
	 * @author Wafly
	 */
	public enum BackgroundSwitchType { NORMAL,
		/**
		 * would switch to new background directry, without any transition.
		 */
		DIRECT,
		/**
		 * top in and top out, need prepare mask layer whose size is <strong>gameWidth*(gameHeight*2)</strong>
		 *  and should with the top half which could cover the whole screen. 
		 */
		TOP_TOP,
		/**
		 * bottom in and bottom out, need prepare mask layer whose size is <strong>gameWidth*(gameHeight*2)</strong>
		 *  and should with the bottom half which could cover the whole screen.
		 */
		BOTTOM_BOTTOM,
		/**
		 * left in and left out, need prepare mask layer whose size is <strong>gameHeight*(gameWidth*2)</strong>
		 *  and should with the left half which could cover the whole screen.
		 */
		LEFT_LEFT,
		/**
		 * right in and right out, need prepare mask layer whose size is <strong>gameHeight*(gameWidth*2)</strong>
		 *  and should with the right half which could cover the whole screen.
		 */
		RIGHT_RIGHT,
		/**
		 * top in and bottom out, need prepare mask layer whose size is <strong>gameWidth*(gameHeight*3)</strong>
		 *  and should with the middle part which could cover the whole screen. 
		 */
		TOP_BOTTOM,
		/**
		 * bottom in and top out, need prepare mask layer whose size is <strong>gameWidth*(gameHeight*3)</strong>
		 *  and should with the middle part which could cover the whole screen.
		 */
		BOTTOM_TOP,
		/**
		 * left in and right out, need prepare mask layer whose size is <strong>gameHeight*(gameWidth*3)</strong>
		 *  and should with the middle part which could cover the whole screen.
		 */
		LEFT_RIGHT,
		/**
		 * right in and left out, need prepare mask layer whose size is <strong>gameHeight*(gameWidth*3)</strong>
		 *  and should with the middle part which could cover the whole screen.
		 */
		RIGHT_LEFT };
	/**
	 * Set the way for the background switch from old to new
	 * @param type the way for the background switch from old to new
	 */
	public void setBackgroundSwitchType(BackgroundSwitchType type) { backgroundSwitchType=type; }
	
	/**
	 * get the controls on current screen(if has panel on screen, the controls would be panel's) which could be touched
	 * @return Operational Controls
	 */
	public ArrayList<OperationalControl> getOperationalControls()
	{
		if(panels.size()>0)
			return panels.get(0).getOperationalControls();
		else
			return operationals;
	}
	
	/**
	 * Add control to current screen(not panel)
	 * @param control control
	 */
	public void addControl(final Control control)
	{
		if(control==null)
			return;
		if(control instanceof FrameAnimation){
			((FrameAnimation)control).reset();
		}
		controls.add(control);
		if(control instanceof OperationalControl)
			operationals.add((OperationalControl)control);
	}
	
	/**
	 * 移除一个控件，但不会销毁
	 * @return 是否正确移除
	 * */
	public boolean removeControl(Control control)
	{
		boolean result = false;
		result = controls.remove(control);
		if(operationals.contains(control))
		{
			((OperationalControl) control).setStatus(OperationalControl.Status.NORMAL);
			result = operationals.remove((OperationalControl)control);
			if(!result)
				Util.logError("The control is OperationalControl but fail to remove from operationals list.", null);
		}
		return result;
	}
	
	/**
	 * 移除当前所有控件
	 */
	public void removeAllControl()
	{
		controls.clear();
		for(OperationalControl control:operationals)
			control.setStatus(OperationalControl.Status.NORMAL);
		operationals.clear();
	}

	/**
	 * 添加一个组件
	 */
	public void addComponent(Component component){
		int layer = component.getLayerIndex();
		if(layer<0 || layer>9)
			return;
		if (components.containsKey(layer))
		{
			components.get(layer).add(component);
		}
		else
		{
			ArrayList<Component> subComponents = new ArrayList<Component>();
			subComponents.add(component);
			components.put(layer, subComponents);
		}
	}

	/**
	 * 获取指定图层内所有组件
	 * @param layer
	 * @return
	 */
	public ArrayList<Component> getComponents(int layer)
	{
		return components.get(layer);
	}

	/**
	 * 移除指定组件，但不会销毁；若在多个图层添加了多次，则所有图层的此Component实例均会被移除。
	 * @param component
	 * @return 是否有组件被移除
	 */
	public boolean removeComponent(Component component)
	{
		boolean result = false;
		for(int i=0; i<10; i++){
			ArrayList<Component> subComponents = components.get(i);
			if(subComponents==null)
				continue;
			if(subComponents.contains(component)){
				subComponents.remove(component);
				result=true;
			}
//			if(!result)
//				result = subComponents.remove(component);
//			else
//				subComponents.remove(component);
		}
		return result;
	}
	
	/**
	 * 执行一个事件
	 * @param className 事件类的全名，此类已实现了wafly.framework.storyTK.tmpl.Ievent接口
	 * @param waiting 是否等待此事件执行完毕，还是立刻返回
	 * @param args 事件方法参数
	 */
	public void executeEvent(final String className, boolean waiting, final String... args)
	{
		try {
			final Ievent eventClass = (Ievent) Class.forName(className).newInstance();
			if (waiting) {
				eventClass.takeAffect(args);
			} else {
				new Thread(){public void run(){
					try{
						eventClass.takeAffect(args);
					} catch (Exception e) {
						Util.logError("Fail to perform event [" + className + "]", e);
					}
				}}.start();
			}
		} catch (Exception e) {
			Util.logError("Fail to perform event [" + className + "]", e);
		}
	}
	
	/**
	 * 某键按下时发生事件
	 * @param keyCode
	 * @return 返回true表示此按键进制被继续处理，否则继续其它处理
	 */
	public boolean onKeyDown(int keyCode) { return false; }
	
	/**
	 * 长按某按键时发生事件
	 * @param keyCode
	 */
	public boolean onKeyLongPress(int keyCode) { return false; }
	
	/**
	 * 按键弹起时发生事件
	 * @param keyCode
	 */
	public void onKeyUp(int keyCode) {}

	/**
	 * 在当前Scene中添加或删除一个Panel，此方法会阻塞直至此Panel动态地显示或关闭完成
	 * @param panel 面板。如果传入null则关闭最上一个面板，同时也会销毁这个面板
	 */
	public synchronized void setPanel(Panel panel)
	{
		Panel currentPanel = getPanel();
		if (panel != null)
		{
			DrawableHelper back4PanelDrawable = new DrawableHelper(Arena.getInstance().getWidth(), Arena.getInstance().getHeight());
			Arena.drawGameScreen(back4PanelDrawable);
			back4PanelDrawables.add(back4PanelDrawable);
			newPanelAlpha = 0;
			panels.add(0, panel);
			try
			{
				ArrayList<Control> controls = panel.getControls();
				while (newPanelAlpha < 255)
				{
					newPanelAlpha = newPanelAlpha > (255 - Util.getSwitchSpeed()) ? 255 : newPanelAlpha + Util.getSwitchSpeed();
					panel.setAlpha(newPanelAlpha);
					for (Control control : controls)
						control.setAlpha(newPanelAlpha);
					wait(1000 / Util.getMaxFPS());
				}
			} catch (Exception e)
			{
				Util.logError("Error in performing panel.", e);
				newPanelAlpha = 255;
				panel.setAlpha(newPanelAlpha);
				for (Control control : controls)
					control.setAlpha(newPanelAlpha);
			}
			panel.onShown();
		}
		else if (currentPanel!=null)
		{
			ArrayList<Control> controls = currentPanel.getControls();
			newPanelAlpha = 255;
			try
			{
				while (newPanelAlpha > 0)
				{
					newPanelAlpha = newPanelAlpha < Util.getSwitchSpeed() ? 0 : newPanelAlpha - Util.getSwitchSpeed();
					currentPanel.setAlpha(newPanelAlpha);
					for (Control control : controls)
						control.setAlpha(newPanelAlpha);
					wait(1000 / Util.getMaxFPS());
				}
			} catch (Exception e)
			{
				Util.logError("Error in closing panel.", e);
				newPanelAlpha = 0;
				currentPanel.setAlpha(newPanelAlpha);
				for (Control control : controls)
					control.setAlpha(newPanelAlpha);
			}
			currentPanel.onClosed();
			currentPanel.destroy();
			if(panels.size()>0)
				panels.remove(0);
			if(back4PanelDrawables.size()>0){
				back4PanelDrawables.get(back4PanelDrawables.size()-1).destroy();
				back4PanelDrawables.remove(back4PanelDrawables.size()-1);
			}
		}
	}
	
	/**
	 * 获取最上方显示的面板
	 * @return 如果没有面板则返回null
	 */
	public Panel getPanel()
	{
		if(panels.size()>0)
			return panels.get(0);
		else
			return null;
	}
	
	/**
	 * 从当前Panel队列中移除指定的panel（但不会销毁）
	 * @param panel
	 * @return 如果当前队列包含了指定的panel，则移除并返回被移除的panel，否则返回null。
	 */
	public Panel removePanel(Panel panel){
		if(panels.contains(panel)){
			panels.remove(panel);
			return panel;
		}
		return null;
	}
	
	/**
	 * 关闭当前场景中存活的所有的panel
	 */
	public void closeAllPanel()
	{
		if(panels.size()==0) return;
		while(panels.size()>1)
		{
			panels.get(panels.size()-1).destroy();
			panels.remove(panels.size()-1);
		}
		setPanel(null);
	}
	
	/**
	 * 捕获当前画面并存放到指定的DrawableHelper中，如果当前画面有panel存在，则捕获被panel覆盖下的画面。
	 * @param draw 用于存放当前画面的DrawableHelper
	 */
	public void drawGamePicture(DrawableHelper draw)
	{
		if(back4PanelDrawables.size()>0)
			draw.drawImage(back4PanelDrawables.get(0),0,0);
		else
			Arena.drawGameScreen(draw);
	}
	
	/**
	 * 获取当前场景中全部存活的panel的数量
	 * @return panels count
	 */
	public int getPanelCount() { return panels.size(); }
	
	/**
	 * 播放一小段音频,此函数重复调用时需要等待一段间隔
	 * @param soundPath 音频文件路径
	 */
	public void playSound(final String soundPath)
	{
		final String sp=Util.verifyFilePath(soundPath);
		if(sp == null)
		{
			Util.logError("Sound file ["+soundPath+"] not found.", null);
			return;
		}
		new Thread(){ public void run() {
			if(sounds==null)
				sounds = new SoundPool(5, AudioManager.STREAM_MUSIC, 0);
			int id=0;
			if(sp.startsWith("/"))
				id=sounds.load(sp, 1);
			else
				id=sounds.load(Util.openInnerDescriptor(sp), 1);
				
//			synchronized (this) {
//				try {
//					Thread.sleep(1000);
//				} catch (Exception e) {}
//			}
			if(sounds.play(id, 1, 1, 1, 0, 1)==0)
				Util.logError("Fail to play sound ["+sp+"].(Format not support?)", null);
		}}.start();
	}

	/**
	 * 销毁当前Scene的所有资源，包括正在播放的音乐，同时也销毁其上的所有控件，但不会销毁任何Panel
	 */
	public void destroy()
	{
		backgroundSwitchType = BackgroundSwitchType.DIRECT;
		setBackground(null, 0, 0, null);
		noShow = true;
		destoryed = true;
		for (int i = 0; i < controls.size(); i++)
			controls.get(i).destroy();
		controls.clear();
		operationals.clear();
		for (Iterator<Integer> it = components.keySet().iterator(); it.hasNext();)
		{
			ArrayList<Component> subComponents = components.get(it.next());
			for (int i = 0; i < subComponents.size(); i++)
			{
				subComponents.get(i).destroy();
			}
			subComponents.clear();
		}
		components.clear();
		if (backgroundDrawable != null)
		{
			backgroundDrawable.destroy();
			backgroundDrawable = null;
		}
		for(DrawableHelper draw:back4PanelDrawables)
			draw.destroy();
		back4PanelDrawables.clear();
		if (newBackgroundDrawable != null)
		{
			newBackgroundDrawable.destroy();
			newBackgroundDrawable = null;
		}
		if(sounds!=null)
		{
			sounds.release();
			sounds=null;
		}
		Util.logInfo("Scene being destroyed.");
	}
	
	/**
	 * 隐藏或显示除了背景外的所有元素
	 */
	public void hideAll(boolean isHide) { noShow=!isHide; }

	@SuppressLint("WrongCall")
	void createFrame(DrawableHelper draw)
	{
		if (destoryed)
			return;
		if (panels.size() == 0)
		{
			if (paintAlpha != 255)
				draw.setDrawAlpha(paintAlpha);
			draw.drawImage(backgroundDrawable, backgroundLocation_X, backgroundLocation_Y);
			if (paintAlpha != 255)
				draw.setDrawAlpha(255);

			if (!noShow)
			{
				onDraw(draw);
				for (int i = 0; i < controls.size(); i++) {
					Control c = controls.get(i);
					if(c.getAlpha()>0)
						c.onDraw(draw);
				}
				for(int i=0; i<10; i++){
					ArrayList<Component> subComponents = components.get(i);
					if(subComponents==null)
						continue;
					for (int j = 0; j < subComponents.size(); j++) {
						subComponents.get(j).onDraw(draw);
					}
				}
			}

			if (newPanelAlpha != 0)
				draw.setDrawAlpha(newPanelAlpha);
			draw.drawImage(newBackgroundDrawable, newBackgroundLocation_X, newBackgroundLocation_Y);
			if (newPanelAlpha != 0)
				draw.setDrawAlpha(255);
		} else {
			if(back4PanelDrawables.size()>0)
				draw.drawImage(back4PanelDrawables.get(back4PanelDrawables.size()-1),0,0);
			if (panels.size() > 0 && !noShow)
				panels.get(0).onDraw(draw);
		}

	}

	/**
	 * 存储当前场景的数据到存储器中
	 * @param storeMgr 指定存储到的存储器
	 */
	public void storeValue(StoreMgr storeMgr)
	{
		storeMgr.setValue("SCENE.BACKGROUND.LOCATION.X", backgroundLocation_X);
		storeMgr.setValue("SCENE.BACKGROUND.LOCATION.Y", backgroundLocation_Y);
		storeMgr.setValue("SCENE.NEWBACKGROUND.LOCATION.X", newBackgroundLocation_X);
		storeMgr.setValue("SCENE.NEWBACKGROUND.LOCATION.Y", newBackgroundLocation_Y);
		storeMgr.setValue("SCENE.NOSHOW", noShow);
		storeMgr.setValue("SCENE.BACKGROUND.SWITCHTYPE", backgroundSwitchType);
		if(backgroundDrawable!=null)
			backgroundDrawable.storeValue("SCENE.BACKGROUND", storeMgr);
		for(int i=0; i<controls.size(); i++)
		{
			Control control = controls.get(i);
			storeMgr.setValue("SCENE.CONTROL."+i+".ID", control.getID());
			storeMgr.setValue("SCENE.CONTROL."+control.getID()+".CLASS", control.getClass().getName());
			control.storeValue(storeMgr);
		}
		for (Iterator<Integer> it = components.keySet().iterator(); it.hasNext();)
		{
			ArrayList<Component> subComponents = components.get(it.next());
			for (int i = 0; i < subComponents.size(); i++)
			{
				Component e = subComponents.get(i);
				storeMgr.setValue("COMPONENT."+i+".ID", e.getID());
				storeMgr.setValue("COMPONENT."+e.getID()+".CLASS", e.getClass().getName());
				e.storeValue(storeMgr);
			}
		}
	}
	
	/**
	 * 从指定存储器中恢复场景数据
	 * @param storeMgr
	 */
	public void restoreValue(StoreMgr storeMgr)
	{
		backgroundLocation_X=Integer.parseInt(storeMgr.getValue("SCENE.BACKGROUND.LOCATION.X"));
		backgroundLocation_Y=Integer.parseInt(storeMgr.getValue("SCENE.BACKGROUND.LOCATION.Y"));
		newBackgroundLocation_X=Integer.parseInt(storeMgr.getValue("SCENE.NEWBACKGROUND.LOCATION.X"));
		newBackgroundLocation_Y=Integer.parseInt(storeMgr.getValue("SCENE.NEWBACKGROUND.LOCATION.Y"));
		noShow=Boolean.parseBoolean(storeMgr.getValue("SCENE.NOSHOW"));
		backgroundSwitchType=BackgroundSwitchType.valueOf(storeMgr.getValue("SCENE.BACKGROUND.SWITCHTYPE"));
		backgroundDrawable=DrawableHelper.restoreValue("SCENE.BACKGROUND", storeMgr);
		for(int i=0;;i++)
		{
			try
			{
				long ID = Long.parseLong(storeMgr.getValue("SCENE.CONTROL."+i+".ID"));
				addControl((Control) Class.forName(storeMgr.getValue("SCENE.CONTROL."+ID+".CLASS"))
						.getConstructor(new Class[]{Long.class,StoreMgr.class}).newInstance(ID, storeMgr));
			} catch (Exception e)
			{
				break;
			}
		}
		for(int i=0;;i++)
		{
			try
			{
				long ID = Long.parseLong(storeMgr.getValue("COMPONENT."+i+".ID"));
				Class.forName(storeMgr.getValue("COMPONENT."+ID+".CLASS")).getConstructor(new Class[]{Long.class,StoreMgr.class}).newInstance(ID, storeMgr);
			} catch (Exception e)
			{
				break;
			}
		}
	}

	public void onUserTouchScreen(int x, int y){ }
	public void onUserReleaseScreen(int x, int y){ }
	public void onUserMoveScreen(int x, int y){ }
	
}
