package wafly.framework.storyTK.basic;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import android.annotation.SuppressLint;
import android.graphics.Bitmap;
import android.graphics.Matrix;
import android.graphics.Typeface;
import android.os.Environment;
import android.view.View;
import android.widget.AbsoluteLayout;

/**
 * To set the game's config
 * 
 * @author Wafly
 */
@SuppressWarnings("deprecation")
public class Config
{

	/********************************************************************
	 * 
	 * Game Size
	 * 
	 * ******************************************************************/

	static int	screenWidth	= 480, screenHeight = 320;

	/**
	 * To set the game size, default is 480x320
	 * 
	 * @param width
	 *            game's width
	 * @param height
	 *            game's height
	 */
	public static void setGameSize(int width, int height)
	{
		if (width < 1) width = 1;
		if (height < 1) height = 1;
		screenWidth = width;
		screenHeight = height;
		if(sptSrcWidth==0 && sptSrcHeight==0){
			sptSrcWidth=screenWidth;
			sptSrcHeight=screenHeight;
		}
	}
	
	static int sptSrcWidth=0, sptSrcHeight=0;
	/**
	 * 实际加载的图片尺寸最大范围，如果不设置此方法，则默认为图片原始大小
	 * @param width
	 * @param height
	 */
	public static void setMaxScreenSize(int width, int height){
		if (width < 1) width = 1;
		if (height < 1) height = 1;
		sptSrcWidth = width;
		sptSrcHeight = height;
	}

	/********************************************************************
	 * 
	 * Max fps
	 * 
	 * ******************************************************************/

	static int	maxFPS	= 30;

	/**
	 * To set the max fps(1-1000), default 60
	 * 
	 * @param fps
	 */
	public static void setMaxFPS(int fsp)
	{
		if (fsp < 1) fsp = 1;
		if (fsp > 1000) fsp = 1000;
		maxFPS = fsp;
	}

	/********************************************************************
	 * 
	 * Enable debug
	 * 
	 * ******************************************************************/

	static boolean	enableDebug	= false;

	/**
	 * to show the debug message, default unavailable
	 */
	public static void enableDebug()
	{
		enableDebug = true;
	}

	/********************************************************************
	 * 
	 * disable video
	 * 
	 * ******************************************************************/

	static boolean	disableVideo	= false;

	/**
	 * to show the debug message, default unavailable
	 */
	public static void disableVideo()
	{
		disableVideo = true;
	}

	/********************************************************************
	 * 
	 * Screen direction
	 * 
	 * ******************************************************************/

	static int	screenDirection	= 0;

	/**
	 * to set the screen direction as PORTRAIT mode, default screen direction is
	 * LANDSCAPE mode
	 */
	public static void setVerticalScreen()
	{
		screenDirection = 1;
	}

	/********************************************************************
	 * 
	 * Background Color
	 * 
	 * ******************************************************************/

	static int	backgroundColor	= 0xff000000;

	/**
	 * to set the beckaround color of the game, default black.
	 * 
	 * @param colorValue
	 *            background color value
	 */
	public static void setBackgroundColor(int colorValue)
	{
		backgroundColor = colorValue;
	}

	/********************************************************************
	 * 
	 * Color Level
	 * 
	 * ******************************************************************/

	static Bitmap.Config	colorLevel	= Bitmap.Config.ARGB_4444;

	/**
	 * to set the color level, it will help you improve your game's perfomance,
	 * default Bitmap.Config.ARGB_8888
	 * 
	 * @param level
	 *            the value of <strong>Bitmap.Config</ctrong>
	 */
	public static void setColorLevel(Bitmap.Config level)
	{
		colorLevel = level;
	}

	/********************************************************************
	 * 
	 * Display Mode
	 * 
	 * ******************************************************************/

	static DisplayMode	displayMode	= DisplayMode.STRETCH;

	/**
	 * display mode,<br>
	 * NORMAL: draw the frame to screen directly, without any change;<br>
	 * CENTER: draw the frame to screen directly, and align at the center of the
	 * screen and background;<br>
	 * ZOOM: zoom the frame by original ratio to fit screen;<br>
	 * STRETCH: zoom the frame to fit the screen and no matter the original
	 * radio
	 */
	public static enum DisplayMode
	{
		/**
		 * 当游戏画面小于屏幕尺寸时居中显示,游戏画面大于屏幕尺寸时顶点对齐
		 */
		ORIGINAL,
		/**
		 * 按游戏画面尺寸等比例缩放,以适应屏幕大小
		 */
		ZOOM,
		/**
		 * 拉伸画面,满屏幕显示
		 */
		STRETCH
	};

	/**
	 * 设置游戏画面的显示模式,默认为通常模式(NORMAL)
	 */
	public static void setDisplayMode(DisplayMode mode) {
		displayMode = mode;
	}

	/********************************************************************
	 * 
	 * Antialias
	 * 
	 * ******************************************************************/

	static boolean	antialiasing	= false;
	
	/**
	 * 启用抗锯齿, 默认不启用
	 */
	public static void enableAntialiasing()
	{
		antialiasing = true;
	}

	/********************************************************************
	 * 
	 * Game root path
	 * 
	 * ******************************************************************/

	static String	iRootPath	= Environment.getExternalStorageDirectory().getPath();

	/**
	 * set the game's home path, use only for the files are in the SD card, then
	 * you can use relative path to get the file. default root path
	 * 
	 * @param rootPath
	 *            game's home path
	 */
	@SuppressLint("SdCardPath")
	public static void setRootPath(String rootPath)
	{
		iRootPath = rootPath == null ? Environment.getExternalStorageDirectory().getPath() : rootPath.trim();
		if(!iRootPath.endsWith("/"))
			iRootPath+="/";
		if(new File(iRootPath).exists())
			return;
		String[] sdpaths = new String[]{
			"/sdcard","/mnt/sdcard","/mnt/extSdCard","/storage/sdcard","/sdcard/external_sd","/mnt/sdcard/external_sd","/emmc","/emmc/sdcard",
			"/mnt/external_sdcard","/mnt/sdcard/external_sdcard","/sdcard/external_sdcard"
		};
		for(int i=0; i<sdpaths.length; i++){
			if(new File(sdpaths[i]+iRootPath).exists()){
				iRootPath=sdpaths[i]+iRootPath;
				break;
			}
		}
	}

	/********************************************************************
	 * 
	 * Switch Speed
	 * 
	 * ******************************************************************/

	static int	switchSpeed	= 25;

	/**
	 * to set the speed of switch background and panel (0-255), default 10
	 * 
	 * @param speed
	 *            speed
	 */
	public static void setSwitchSpeed(int speed)
	{
		if (speed < 0)
			switchSpeed = 0;
		else if (speed > 255)
			switchSpeed = 255;
		else
			switchSpeed = speed;
	}
	
	/********************************************************************
	 * 
	 * sys control
	 * 
	 * ******************************************************************/
	
	static ArrayList<View> sysControl=new ArrayList<View>();
	
	/**
	 * 添加系统组件, 同时设置其位置及大小<br>
	 * add views
	 */
	public static void addSysControl(View component, int x, int y, int width, int height)
	{
		component.setLayoutParams(new AbsoluteLayout.LayoutParams(width, height, x, y));
		sysControl.add(component);
	}

	/********************************************************************
	 * 
	 * System Font
	 * 
	 *******************************************************************/
	
	static Typeface sysFont = Typeface.create("sans", Typeface.NORMAL);
	static int sysFontSize = 12;
	static int drawOffset = 12;
	
	/**
	 * 指定字体文件设置游戏字体, 默认使用15号大小的系统字体
	 * @param fontFilePath 字体文件路径
	 * @param fontSize 字体大小
	 */
	public static void setGlobalFont(String fontFilePath, int fontSize)
	{
		if(fontSize<1)
			fontSize=1;
		String filePath = Util.verifyFilePath(fontFilePath);
		if(filePath==null)
			return;
		sysFontSize=fontSize;
		drawOffset=fontSize;
		sysFont = Typeface.createFromFile(fontFilePath);
	}
	
	/**
	 * 指定游戏字体大小,默认15
	 * @param fontSize 字体大小
	 */
	public static void setGlobalFontSize(int fontSize)
	{
		if(fontSize<1)
			fontSize=1;
		sysFontSize=fontSize;
		drawOffset=fontSize;
	}

	/********************************************************************
	 * 
	 * Matrix
	 * 
	 *******************************************************************/
	
	static Matrix matrix=new Matrix();
	
	public static void setMatrix(Matrix m)
	{
		matrix = m;
	}

	/********************************************************************
	 * 
	 * post Scale
	 * 
	 *******************************************************************/
	
	/**
	 * 虽然存储了屏幕尺寸，但游戏逻辑中不应该使用的，真正应该只面对游戏尺寸
	 */
	static int realWidth=0, realHeight=0;
	static boolean isFitDraw = false;
	public static boolean isFixedDraw(){ return isFitDraw; }
	static float postScale_X=1, postScale_Y=1;
	static float gameScale_X=1, gameScale_Y=1;

	/**
	 * 自定义屏幕尺寸，用于绕过系统参数，一般不建议手动修改
	 */
	public static void dimScreenSize(int width, int height){
		realWidth = width;
		realHeight = height;
		if(realWidth>sptSrcWidth || realHeight>sptSrcHeight){
			isFitDraw=true;
			postScale_X=realWidth/(float)sptSrcWidth;
			postScale_Y=realHeight/(float)sptSrcHeight;
			gameScale_X=sptSrcWidth/(float)screenWidth;
			gameScale_Y=sptSrcHeight/(float)screenHeight;
		}
	}

	/********************************************************************
	 * 
	 * Other Param
	 * 
	 *******************************************************************/

	@SuppressLint("SimpleDateFormat")
	static SimpleDateFormat dataFormat = new SimpleDateFormat("[HH:mm:ss.SSS] ");

}
