package wafly.framework.storyTK.basic;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import wafly.framework.storyTK.Arena;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.RectF;

public class DrawableHelper
{
	private Bitmap image = null;
	private Canvas canvas = null;
	private Paint paint = new Paint();
	private int width = 1, height = 1;
//	private int[] pixels;
	private int alpha = 255, foreColor = 0xff000000;
	private boolean destroy = false;
	private String filePath=null;
	private int fileIndex=-1;
//	private int strOffset=0;
	
	protected DrawableHelper(){}

	/**
	 * 通过指定尺寸初始化一副全透明没内容图片
	 * @param width
	 * @param height
	 */
	public DrawableHelper(int width, int height)
	{
		this.width = width < 1 ? 1 : width;
		this.height = height < 1 ? 1 : height;
		image = Bitmap.createBitmap(this.width, this.height, Util.getColorLevel());
//		image.getPixels(pixels, 0, this.width, 0, 0, this.width, this.height);
		canvas = new Canvas(this.image);
		paint.setColor(Util.getBackgroundColor());
		paint.setAntiAlias(Util.isAntialiasing());
		paint.setTypeface(Config.sysFont);
		paint.setTextSize(Config.sysFontSize);
	}

	/**
	 * 通过图片文件路径初始化一副图片
	 * @param imgFilePath
	 */
	public DrawableHelper(String imgFilePath)
	{
		filePath = Util.verifyFilePath(imgFilePath);
		if(filePath==null)
		{
			Util.logError("Fail to load image ["+filePath+"].", null);
			image = Bitmap.createBitmap(1, 1, Util.getColorLevel());
		}
		else
		{
			if(filePath.startsWith("/"))
			{
				Bitmap oriImg = BitmapFactory.decodeFile(filePath);
				if(oriImg!=null){
					int realWidth=0, realHeight=0;
					if(Config.isFitDraw){
						realWidth=(int) (Config.gameScale_X*oriImg.getWidth());
						realHeight=(int) (Config.gameScale_Y*oriImg.getHeight());
					}else{
						realWidth=(int)((Config.realWidth*(float)oriImg.getWidth())/Util.getGameWidth());
						realHeight=(int)((Config.realHeight*(float)oriImg.getHeight())/Util.getGameHeight());
					}
					Bitmap oriImage1 = Bitmap.createScaledBitmap(oriImg, realWidth, realHeight, true);
					image = oriImage1.copy(Util.getColorLevel(), true);
					oriImage1.recycle();
					oriImg.recycle();
				}
			}
			else
			{
				try {
					InputStream is = Arena.getContainer().getAssets().open(filePath);
					Bitmap oriImage = BitmapFactory.decodeStream(is);
					is.close();
					int realWidth=0, realHeight=0;
					if(Config.isFitDraw){
						realWidth=(int) (Config.gameScale_X*oriImage.getWidth());
						realHeight=(int) (Config.gameScale_Y*oriImage.getHeight());
					}else{
						realWidth=(int)((Config.realWidth*(float)oriImage.getWidth())/Util.getGameWidth());
						realHeight=(int)((Config.realHeight*(float)oriImage.getHeight())/Util.getGameHeight());
					}
					Bitmap oriImage1 = Bitmap.createScaledBitmap(oriImage, realWidth, realHeight, true);
					image = oriImage1.copy(Util.getColorLevel(), true);
					oriImage1.recycle();
					oriImage.recycle();
				} catch (Exception e) {
					Util.logError("Fail to zoom image ["+filePath+"].", e);
					if(image!=null)
						image.recycle();
					image=null;
				}
			}
			if(image==null)
			{
				Util.logError("Fail to load image ["+filePath+"].", null);
				image = Bitmap.createBitmap(1, 1, Util.getColorLevel());
			}
		}
		canvas = new Canvas(this.image);
		paint.setColor(Util.getBackgroundColor());
		paint.setAntiAlias(Util.isAntialiasing());
		paint.setTypeface(Config.sysFont);
		paint.setTextSize(Config.sysFontSize);
		width=image.getWidth();
		height=image.getHeight();
	}

	/**
	 * 通过一个图片包文件路径及其内所在索引初始化一副图片
	 * @param packingFilePath
	 * @param fileIndex
	 */
//	public DrawableHelper(String packingFilePath, int fileIndex)
//	{
//		packingFilePath = Util.verifyFilePath(packingFilePath);
//		try
//		{
//			if(packingFilePath==null)
//				throw new Exception("Packing file not found.");
//			this.filePath=packingFilePath;
//			this.fileIndex=fileIndex;
//			File f = new File(packingFilePath);
//			FileInputStream fis = new FileInputStream(f);
//			int count = fis.read();
//			count = count * 0x100 + fis.read();
//			if (fileIndex >= count || fis.read() != 0) // image=0, audio=1, video=2, text=3
//			{
//				try{
//					fis.close();
//				}catch(Exception e){}
//				throw new Exception("Specified drama not found.");
//			}
//			fis.skip(fileIndex * 4);
//
//			int i1 = fis.read(), i2 = fis.read(), i3 = fis.read(), i4 = fis.read();
//			int fIndex = i1 * 0x1000000 + i2 * 0x10000 + i3 * 0x100 + i4;
//			int fLength = 0;
//
//			if (fileIndex == count - 1)
//			{
//				fLength = (int) f.length() - fIndex;
//			}
//			else
//			{
//				int ii1 = fis.read(), ii2 = fis.read(), ii3 = fis.read(), ii4 = fis.read();
//				fLength = ii1 * 0x1000000 + ii2 * 0x10000 + ii3 * 0x100 + ii4 - fIndex;
//				fis.skip((count - fileIndex - 2) * 4);
//			}
//			fis.skip(fIndex - count * 4 - 3);
//
//			byte[] bytes = new byte[fLength];
//			fis.read(bytes);
//			fis.close();
//
//			for (int i = 0; i < bytes.length; i++)
//				bytes[i] = (byte) (bytes[i] - 98);
//
//			InputStream is = new ByteArrayInputStream(bytes, 0, bytes.length);
//			image = BitmapFactory.decodeStream(is);
//			is.close();
//		} catch (Exception e)
//		{
//			Util.logError("Error for loading image.", e);
//			this.width = 1;
//			this.height = 1;
//			image = Bitmap.createBitmap(1, 1, Util.getColorLevel());
//		}
////		image.getPixels(pixels, 0, this.width, 0, 0, this.width, this.height);
//		canvas = new Canvas(this.image);
//		paint.setColor(Util.getBackgroundColor());
//		paint.setAntiAlias(Util.isAntialiasing());
//		paint.setTypeface(Config.sysFont);
//		paint.setTextSize(Config.sysFontSize);
//		width=image.getWidth();
//		height=image.getHeight();
//	}
	
	public static DrawableHelper[] getDrawableArray(String imgFilePath, int spriteCount){
		String filePath = Util.verifyFilePath(imgFilePath);
		if(filePath==null) {
			Util.logError("Fail to load image ["+filePath+"].", null);
			return null;
		}
		Bitmap bigImg=null;
		if(filePath.startsWith("/")) {
			bigImg = BitmapFactory.decodeFile(filePath);
		} else {
			try {
				InputStream is = Arena.getContainer().getAssets().open(filePath);
				bigImg = BitmapFactory.decodeStream(is);
				is.close();
			} catch (Exception e) { }
		}
		if(bigImg==null) {
			Util.logError("Fail to load image ["+filePath+"].", null);
			return null;
		}
		int width = bigImg.getWidth() / spriteCount;
		int height = bigImg.getHeight();
		DrawableHelper[] drawArray = new DrawableHelper[spriteCount];
		for(int i=0; i<spriteCount; i++){
			drawArray[i] = new DrawableHelper();
			drawArray[i].image = Bitmap.createBitmap(width, height, Util.getColorLevel());
			drawArray[i].canvas = new Canvas(drawArray[i].image);
			drawArray[i].paint = new Paint();
			drawArray[i].canvas.drawBitmap(bigImg, -(i*width), 0, drawArray[i].paint);
			int realWidth=0, realHeight=0;
			if(Config.isFitDraw){
				realWidth=(int) (Config.gameScale_X*width);
				realHeight=(int) (Config.gameScale_Y*height);
			}else{
				realWidth=(int)((Config.realWidth*(float)width)/Util.getGameWidth());
				realHeight=(int)((Config.realHeight*(float)height)/Util.getGameHeight());
			}
			Bitmap tmpMap = drawArray[i].image;
			drawArray[i].image = Bitmap.createScaledBitmap(drawArray[i].image, realWidth, realHeight, true);
			tmpMap.recycle();
			drawArray[i].paint.setColor(Util.getBackgroundColor());
			drawArray[i].paint.setAntiAlias(Util.isAntialiasing());
			drawArray[i].paint.setTypeface(Config.sysFont);
			drawArray[i].paint.setTextSize(Config.sysFontSize);
			drawArray[i].width=width;
			drawArray[i].height=height;
		}
		bigImg.recycle();
		return drawArray;
	
	}
	
	public String getFilepath(){ return filePath; }

	/**
	 * 获取图片
	 * @return
	 */
	public Bitmap getSource() { return image; }

	public int getWidth() { return width; }

	public int getHeight() { return height; }

	public boolean isDestroted() { return destroy; }

	public DrawableHelper clone()
	{
		DrawableHelper draw = new DrawableHelper(width, height);
		draw.image = this.image.copy(Util.getColorLevel(), true);
//		draw.image.getPixels(draw.pixels, 0, this.width, 0, 0, draw.width, draw.height);
		draw.canvas = new Canvas(draw.image);
		draw.paint.setColor(Util.getBackgroundColor());
		draw.paint.setAntiAlias(Util.isAntialiasing());
		draw.paint.setTypeface(Config.sysFont);
		draw.paint.setTextSize(Config.sysFontSize);
		return draw;
	}

	/**
	 * 缩放图片大小
	 * @param newWidth
	 * @param newHeight
	 */
	public void zoomTo(int newWidth, int newHeight)
	{
		width = newWidth < 1 ? 1 : newWidth;
		height = newHeight < 1 ? 1 : newHeight;
		Bitmap tmpMap=image.copy(Config.colorLevel, true);
		image = Bitmap.createScaledBitmap(tmpMap, width, height, true);
		if(tmpMap!=null)
			tmpMap.recycle();
//		image.getPixels(pixels, 0, this.width, 0, 0, this.width, this.height);
		canvas = new Canvas(this.image);
	}

	/**
	 * 将图片转换为黑白格式
	 */
	public void convertToGray()
	{
//		for (int i = 0; i < pixels.length; i++)
//		{
//			int[] rgbs = getRGBs(pixels[i]);
//			int value = (int) (0.299 * rgbs[0] + 0.587 * rgbs[1] + 0.114 * rgbs[2]);
//			pixels[i] = (255 << 24) + (value << 16) + (value << 8) + value;
//		}
	}

	/**
	 * 将图片转换为反色
	 */
	public void convertToXor()
	{
//		for (int i = 0; i < pixels.length; i++)
//		{
//			int[] rgbs = getRGBs(pixels[i]);
//			int r = rgbs[0] ^ 0xff;
//			int g = rgbs[1] ^ 0xff;
//			int b = rgbs[2] ^ 0xff;
//			pixels[i] = (255 << 24) + (r << 16) + (g << 8) + b;
//		}
	}

	/**
	 * 将图片保存到磁盘文件
	 * @param filePath
	 * @return
	 */
	public boolean saveToFile(String filePath)
	{
		File f = new File(filePath);
		FileOutputStream fOut = null;
		try
		{
			f.createNewFile();
			fOut = new FileOutputStream(f);
			image.compress(Bitmap.CompressFormat.JPEG, 100, fOut);
			fOut.flush();
			fOut.close();
			return true;
		} catch (Exception e)
		{
			Util.logError("Fail to save image.", e);
			return false;
		}
	}

	/**
	 * 获取图片的一部分
	 * @param x
	 * @param y
	 * @param width
	 * @param height
	 * @return
	 */
	public DrawableHelper getPart(int x, int y, int width, int height)
	{
		DrawableHelper draw = new DrawableHelper(width, height);
		new Canvas(draw.image).drawBitmap(this.image, -x, -y, new Paint());
//		draw.image.getPixels(draw.pixels, 0, this.width, 0, 0, draw.width, draw.height);
		draw.canvas = new Canvas(draw.image);
		draw.paint.setColor(Util.getBackgroundColor());
		draw.paint.setAntiAlias(Util.isAntialiasing());
		draw.paint.setTypeface(Config.sysFont);
		draw.paint.setTextSize(Config.sysFontSize);
		return draw;
	}

	/**
	 * 将图片从指定位置剪切为新的大小
	 * @param x
	 * @param y
	 * @param width
	 * @param height
	 */
	public void cutTo(int x, int y, int width, int height)
	{
		this.width=width;
		this.height=height;
		if (width < 1)
			this.width = 1;
		if (height < 1)
			this.height = 1;
		if (width + x > this.width)
			this.width = this.width - x;
		if (height + x > this.height)
			this.height = this.height - x;

		Bitmap newImg = Bitmap.createBitmap(this.width, this.height, Util.getColorLevel());
		new Canvas(newImg).drawBitmap(this.image, -x, -y, new Paint());
		this.image.recycle();
		this.image=newImg;
//		image.getPixels(pixels, 0, this.width, 0, 0, this.width, this.height);
		canvas = new Canvas(this.image);
	}

	/**
	 * 在其上绘制图片
	 * @param drawable
	 * @param x
	 * @param y
	 */
	public void drawImage(DrawableHelper drawable, int x, int y)
	{
		if (drawable == null || drawable.isDestroted())
			return;
//		if(Config.isFitDraw){
//			canvas.drawBitmap(drawable.image,
//					new Rect(0, 0, drawable.getWidth(), drawable.getHeight()),
//					new Rect(x, y, (int)(drawable.getWidth()*Config.postScale_X)+x, (int)(drawable.getHeight()*Config.postScale_Y)+y),
//					paint);
//		}else{
			canvas.drawBitmap(drawable.image, x, y, paint);
//		}
	}

	/**
	 * 在其上绘制直线
	 * @param startX
	 * @param startY
	 * @param endX
	 * @param endY
	 */
	public void drawLine(int startX, int startY, int endX, int endY)
	{
		canvas.drawLine(startX, startY, endX, endY, paint);
	}

	/**
	 * 在其上绘制椭圆
	 * @param x
	 * @param y
	 * @param width
	 * @param height
	 * @param isFill
	 */
	public void drawOval(int x, int y, int width, int height, boolean isFill)
	{
		if (isFill)
		{
	        paint.setStyle(Paint.Style.FILL);
			canvas.drawOval(new RectF(x, y, x+width, y+height), paint);
		}
		else
		{
	        paint.setStyle(Paint.Style.STROKE);
			canvas.drawOval(new RectF(x, y, x+width, y+height), paint);
		}
	}

	/**
	 * 在其上绘制矩形
	 * @param x
	 * @param y
	 * @param width
	 * @param height
	 * @param isFill
	 */
	public void drawRect(int x, int y, int width, int height, boolean isFill)
	{
		if (isFill)
		{
	        paint.setStyle(Paint.Style.FILL);
	        canvas.drawRect(x, y, x+width, y+height, paint);
		}
		else
		{
	        paint.setStyle(Paint.Style.STROKE);
	        canvas.drawRect(x, y, x+width, y+height, paint);
		}
	}

	/**
	 * 在其上绘制圆角矩形
	 * @param x
	 * @param y
	 * @param width
	 * @param height
	 * @param radius
	 * @param isFill
	 */
	public void drawRoundRect(int x, int y, int width, int height, int radius, boolean isFill)
	{
		if (isFill)
		{
	        paint.setStyle(Paint.Style.FILL);
	        canvas.drawRoundRect(new RectF(x, y, x+width, y+height), radius, radius, paint);
		}
		else
		{
	        paint.setStyle(Paint.Style.STROKE);
	        canvas.drawRoundRect(new RectF(x, y, x+width, y+height), radius, radius, paint);
		}
	}

	/**
	 * 在其上绘制文字
	 * @param string
	 * @param x
	 * @param y
	 */
	public void drawString(String string, int x, int y)
	{
		canvas.drawText(string, x, y+Config.drawOffset, paint);
	}

	/**
	 * 将图片刷成指定颜色
	 * @param color
	 */
	public void drawColor(int color)
	{
//		int[] argb = Util.getARGB(color);
//		if (argb[3] == 255)
//		{
//			for (int i = 0; i < pixels.length; i++)
//				pixels[i] = 0xff000000+color;
//		}
//		else
//		{
			canvas.drawColor(color);
//		}
	}

	/**
	 * 设置前景色，设置后在其上绘制的矢量图形（直线，矩形…）和文字均使用这个颜色值
	 * @param color
	 */
	public void setForeColor(int color)
	{
		this.foreColor = color;
		paint.setColor(color);
	}

	/**
	 * @see setForeColor(int color)
	 * @return
	 */
	public int getForeColor()
	{
		return foreColor;
	}

	/**
	 * 设置绘制透明度，这个透明度不是自身的透明度，而是接下来将要绘制到其上的其他（图片，矩形…）的透明度
	 * @param alpha
	 */
	public void setDrawAlpha(int alpha)
	{
		if (alpha < 0)
			alpha = 0;
		if (alpha > 255)
			alpha = 255;
		this.alpha = alpha;
		paint.setAlpha(alpha);
	}

	/**
	 * @see setDrawAlpha(int alpha)
	 * @return
	 */
	public int getDrawAlpha()
	{
		return alpha;
	}

	public void destroy()
	{
		canvas=null;
		paint=null;
		if(image!=null)
		{
			image.recycle();
			image=null;
		}
		destroy = true;
	}

	/*********************************************************************************************************************************************************/
	/*********************************************************************************************************************************************************/
	/*********************************************************************************************************************************************************/

//	private int[] getRGBs(final int pixel)
//	{
//		int[] rgbs = new int[3];
//		rgbs[0] = (pixel >> 16) & 0xff;
//		rgbs[1] = (pixel >> 8) & 0xff;
//		rgbs[2] = (pixel) & 0xff;
//		return rgbs;
//	}

//	public int[] getPixels()
//	{
//		return pixels;
//	}
	
	public void storeValue(String storeID, StoreMgr storeMgr)
	{
		storeMgr.setValue(storeID+".DRAWABLE.WIDTH", width);
		storeMgr.setValue(storeID+".DRAWABLE.HEIGHT", height);
		storeMgr.setValue(storeID+".DRAWABLE.ALPHA", alpha);
		storeMgr.setValue(storeID+".DRAWABLE.FORECOLOR", foreColor);
		storeMgr.setValue(storeID+".DRAWABLE.FILEPATH", filePath);
		storeMgr.setValue(storeID+".DRAWABLE.FILEIndex", fileIndex);
	}
	
	public static DrawableHelper restoreValue(String storeID, StoreMgr storeMgr)
	{
		DrawableHelper newDrawInstance = null;
		String filePath = storeMgr.getValue(storeID+".DRAWABLE.FILEPATH");
		if(filePath!=null)
		{
//			int index = Integer.parseInt(storeMgr.getValue(storeID+".DRAWABLE.FILEIndex"));
//			if(index>-1)
//				newDrawInstance=new DrawableHelper(filePath, index);
//			else
				newDrawInstance=new DrawableHelper(filePath);
		}
		else
		{
			try {
			newDrawInstance=new DrawableHelper(Integer.parseInt(storeMgr.getValue(storeID+".DRAWABLE.WIDTH")),
					Integer.parseInt(storeMgr.getValue(storeID+".DRAWABLE.HEIGHT")));
			} catch(Exception e) {
				return null;
			}
		}
		newDrawInstance.setDrawAlpha(Integer.parseInt(storeMgr.getValue(storeID+".DRAWABLE.ALPHA")));
		newDrawInstance.setForeColor(Integer.parseInt(storeMgr.getValue(storeID+".DRAWABLE.FORECOLOR")));
		return newDrawInstance;
	}
	
}
