package wafly.framework.storyTK.basic;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import wafly.framework.storyTK.Arena;
import wafly.framework.storyTK.basic.Config.DisplayMode;
import android.content.res.AssetFileDescriptor;
import android.graphics.Bitmap;
import android.graphics.Paint;
import android.graphics.Rect;
import android.graphics.Typeface;
import android.util.Log;
import android.view.View;

/**
 * The comment methods
 * @author Wafly
 */
public class Util
{
	/**
	 * 将红,绿,蓝三种颜色的值和透明度整合成一个Color值<br>
	 * To convert the color value from 3 int(red, green, blue) to 1 int
	 * 
	 * @param red
	 *            红颜色的值,取值范围0-255<br>
	 *            red color value (0-255)
	 * @param green
	 *            绿颜色的值,取值范围0-255<br>
	 *            green color value (0-255)
	 * @param blue
	 *            蓝颜色的值,取值范围0-255<br>
	 *            blue color value (0-255)
	 * @param alpha
	 *            透明度,取值范围(0-255)<br>
	 *            transparents value(0-255)
	 */
	public static int getColorValue(int red, int green, int blue, int alpha)
	{
		if (red < 0 || green < 0 || blue < 0)
		{
			Util.logError("Value could not less than 0.", null);
			return -1;
		}
		else if (red > 255 || green > 255 || blue > 255)
		{
			Util.logError("Value could not larger than 255.", null);
			return -1;
		}
		return alpha * 0x1000000 + red * 0x10000 + green * 0x100 + blue;
	}

	/**
	 * 获取指定颜色的反色(透明度不变)<br>
	 * To get the inverse color from original color
	 * 
	 * @param color
	 *            色彩值, 取值范围0x00000000-0xffffffff(16进制)<br>
	 *            color value (0x00000000-0xffffffff)
	 */
	public static int getInverseColorValue(int color)
	{
		String colorStr = Integer.toHexString(color);
		if (colorStr.length() <= 6)
		{
			return 0xffffff - Integer.parseInt(colorStr.toString(), 16);
		}
		else
		{
			return Integer.parseInt(colorStr.substring(0, colorStr.length() - 6), 16) * 0x1000000 + 0xffffff - Integer.parseInt(colorStr.substring(colorStr.length() - 6), 16);
		}
	}

	/**
	 * 获取某一色彩的三原色的值及透明度<br>
	 * To convert the color value from 1 int to 4 ints
	 * 
	 * @param colorValue
	 *            色彩值, 取值范围0x00000000-0xffffffff(16进制)<br>
	 *            color value (0x00000000-0xffffffff)
	 * @return 长度为3的int数组,顺次为红,绿,蓝颜色的值及透明度<br>
	 *         4 length int[], int[0]=red color, int[1]=green color, int[2]=blue color, int[3]=alpha
	 */
	public static int[] getARGB(int color)
	{
		int[] colors = new int[4];
		String colorStr = Integer.toHexString(color);
		if (colorStr.length() > 6)
		{
			colors[3] = Integer.parseInt(colorStr.substring(0, colorStr.length() - 6), 16);
			colorStr = colorStr.substring(colorStr.length() - 6);
		}
		if (colorStr.length() > 4)
		{
			colors[0] = Integer.parseInt(colorStr.substring(0, colorStr.length() - 4), 16);
			colorStr = colorStr.substring(colorStr.length() - 4);
		}
		if (colorStr.length() > 2)
		{
			colors[1] = Integer.parseInt(colorStr.substring(0, colorStr.length() - 2), 16);
			colorStr = colorStr.substring(colorStr.length() - 2);
		}
		colors[2] = Integer.parseInt(colorStr, 16);
		return colors;
	}

	/**
	 * 获取字符串宽度<br>
	 * to get the pix length of the string
	 */
	public static int calStringPixLength(String string)
	{
		if (string == null)
		{
			Util.logError("Fail to get string width because string or paint is null.", null);
			return 0;
		}
		Rect bounds = new Rect();
		Paint paint = new Paint();
		paint.setTypeface(Config.sysFont);
		paint.setTextSize(Config.sysFontSize);
		paint.getTextBounds(string, 0, string.length(), bounds);
		return bounds.width();
	}
	
	/**
	 * to get the font height with paint
	 * @param paint
	 * @return the font height
	 */
	public static int getFontHeight(Paint paint) { return (int)(paint.getFontMetrics().bottom - paint.getFontMetrics().top); }
	
	/**
	 * to log down the error message. If debug mode, this message will show out to the screen
	 * @param errorMessage message for shown
	 */
	public static void logError(String errorMessage, Throwable e)
	{
		if(!Config.enableDebug)
			return;
		Log.e("StoryToolKit", Config.dataFormat.format(new Date()) + errorMessage);
//		System.err.println(Config.dataFormat.format(new Date()) + errorMessage);
		if(Arena.getInstance()!=null)
			Arena.setMessage(errorMessage);
		if (e != null)
			e.printStackTrace();
	}
	
	/**
	 * to show your message to the console
	 * @param infoMessage message for shown
	 */
	public static void logInfo(String infoMessage)
	{
		if(!Config.enableDebug)
			return;
		Log.i("StoryToolKit", Config.dataFormat.format(new Date()) + infoMessage);
//		System.out.println(Config.dataFormat.format(new Date()) + infoMessage);
	}
	
	/**
	 * get real screen position X
	 * @param x the x position of the game
	 * @return the x position of the screen
	 */
	public static int getLeftOnScreen(int x) { return x*Config.realWidth/Config.screenWidth; }
	
	/**
	 * get real screen position Y
	 * @param y the y position of the game
	 * @return the y position of the screen
	 */
	public static int getTopOnScreen(int y) { return y*Config.realHeight/Config.screenHeight; }
//	
//	/**
//	 * get game position X
//	 * @param x the x position of the screen
//	 * @return the x position of the game
//	 */
//	public static int getLeftOnGame(int x) { return (int)((x)/Config.postScale_X); }
//	
//	/**
//	 * get game position Y
//	 * @param y the y position of the screen
//	 * @return the y position of the game
//	 */
//	public static int getTopOnGame(int y) { return (int)((y)/Config.postScale_Y); }
//	
//	/**
//	 * get X postScale from game to screen
//	 * @return X postScale from game to screen
//	 */
//	public static float getWidthScale() { return Config.postScale_X; }
//	
//	/**
//	 * get Y postScale from game to screen
//	 * @return Y postScale from game to screen
//	 */
//	public static float getHeightScale() { return Config.postScale_Y; }
	
	public static int getMaxSupportScreenWidth(){ return Config.sptSrcWidth; }
	
	public static int getMaxSupportScreenHeight(){ return Config.sptSrcHeight; }
	
	/**
	 * To open a file which in the assets folder and get its AssetFileDescriptor, need to closd it after used
	 * @param filePath the relative path of the file
	 * @return AssetFileDescriptor
	 */
	public static AssetFileDescriptor openInnerDescriptor(String filePath)
	{
		try {
			return Arena.getContainer().getAssets().openFd(filePath);
		} catch (Exception e) {
			logError("Fail to open file ["+filePath+"].", e);
			return null;
		}
	}
	
	/**
	 * to open a file and get its InputStream, need to close after used
	 * @param filePath file's path. 
	 * When your file is in assets folder, please use relative path; 
	 * if your file is in SD card, please use absolute path; 
	 * but if you setted the root path of the application to Config, please use relative path.
	 * @return
	 */
	public static InputStream openFile(String filePath)
	{
		filePath = verifyFilePath(filePath);
		if(filePath==null)
			return null;
		try {
			if(filePath.startsWith("/"))
				return new FileInputStream(new File(filePath));
			else
				return Arena.getContainer().getAssets().open(filePath);
		} catch (Exception e) {
			logError("Fail to open file ["+filePath+"].", e);
			return null;
		}
	}
	
	/**
	 * verify the spesify file path
	 * @param path spesify file path
	 * @return the real file path:<br>
	 * start with "/": the file is in the SD card,<br>
	 * not start with "/": the file is in the project folder assets folder,<br>
	 * null: the file not found.
	 */
	public static String verifyFilePath(String path)
	{
		if(path==null)
		{
			return null;
		}
		if (new File(path).exists())
		{
			return path.replace("\\", "/");
		}
		if(path.startsWith("/") || path.startsWith("\\"))
		{
			Util.logError("File path [" + path + "] is absolute and target not found.", null);
			return null;
		}
		String oriPath = path;
		path = Config.iRootPath + path;
		if (new File(path).exists())
		{
			return path.replace("\\", "/");
		}
		try {
			Arena.getContainer().getAssets().open(path).close();
			return path;
		} catch (Exception e) {
			Util.logError("File path ["+oriPath+"] is relative and target ["+path+"] not found.", null);
		}
		return null;
	}
	
	public static String getFilePath(String path){
		if(path==null)
			return null;
		return Config.iRootPath + path;
	}
	
	/**
	 * 创建一个或一层文件夹，然后返回文件夹完整路径，若不成功则返回null
	 * @param folderPath
	 * @return
	 */
	public static String createFolder(String folderPath)
	{
		if (folderPath == null)
		{
			Util.logError("Created folder path is null.", null);
			return null;
		}
		if(!folderPath.startsWith("/"))
		{
			folderPath=Config.iRootPath + folderPath;
		}
		File folder = new File(folderPath);
		if (!folder.exists()&&!folder.mkdirs())
		{
			Util.logError("Fail to create specifies folder.", null);
			return null;
		}
		if(folderPath.endsWith("/")||folderPath.endsWith("\\"))
			return folderPath;
		else
			return folderPath+"/";
	}
	
	/**
	 * to make the thread wait for a while (several times of frame)
	 * @param pauseThreadObject the object which contains the thread need to pause
	 * @param pauseFrameCount how many frames need to wait.
	 */
	public static void pause(Object pauseThreadObject, int pauseFrameCount) {
		synchronized (pauseThreadObject) {
			try {
				Thread.sleep(pauseFrameCount * 1000 / Config.maxFPS);
			} catch (Exception e) {}
		}
	}
	

	public static String getVersion() { return "2.11.0503"; }
	
	public static int getDistance(int x1, int y1, int x2, int y2){
		return (int)Math.sqrt((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2));
	}

	/*************************************************************************************************/
	/******************************** come from Config ********************************/
	/*************************************************************************************************/

	/**
	 * 获取游戏宽度<br>
	 * get game's width
	 */
	public static int getGameWidth()
	{
		return Config.screenWidth;
	}

	/**
	 * 获取游戏高度<br>
	 * get game's height
	 */
	public static int getGameHeight()
	{
		return Config.screenHeight;
	}
	
	public static int getScreenWidth(){
		return Config.realWidth;
	}
	
	public static int getScreenHeight(){
		return Config.realHeight;
	}

	/**
	 * 获取游戏画面最大帧数<br>
	 * to get the max fps
	 */
	public static int getMaxFPS()
	{
		return Config.maxFPS;
	}

	/**
	 * 判断当前是否启用Debug模式<br>
	 * judge whether showing debug message or not
	 */
	public static boolean isDebug()
	{
		return Config.enableDebug;
	}

	/**
	 * 判断当前是否启用视频功能<br>
	 * judge if could play video
	 */
	public static boolean isVideoEnable()
	{
		return !Config.disableVideo;
	}

	/**
	 * 获取屏幕方向<br>
	 * to get the screen direction
	 * 
	 * @return 0: 横屏; 1: 竖屏<br>
	 *         0: LANDSCAPE mode; 1: PORTRAIT mode.
	 */
	public static int getScreenDirection()
	{
		return 0;
	}

	/**
	 * 获取背景颜色<br>
	 * get background color
	 */
	public static int getBackgroundColor()
	{
		return Config.backgroundColor;
	}

	/**
	 * 获取色彩等级
	 */
	public static Bitmap.Config getColorLevel()
	{
		return Config.colorLevel;
	}

	/**
	 * 获取游戏画面的显示模式<br>
	 * get the current display mode
	 */
	public static DisplayMode getDisplayMode()
	{
		return Config.displayMode;
	}

	/**
	 * 判断是否启用抗锯齿功能<br>
	 * to judge if the frame is antialiasing
	 */
	public static boolean isAntialiasing()
	{
		return Config.antialiasing;
	}

	/**
	 * 获取游戏的根目录<br>
	 * to get the game's home path
	 */
	public static String getRootPath()
	{
		return Config.iRootPath;
	}

	/**
	 * 获取游戏中某些切换的速度<br>
	 * to get the switch speed for background and panel
	 */
	public static int getSwitchSpeed()
	{
		return Config.switchSpeed;
	}

	/**
	 * 获取游戏标题
	 * @return
	 */
	public static String getGameTitle()
	{
//		return Config.gameName;
		return null;
	}
	
	/**
	 * 获取全局字体
	 * @return
	 */
	public static Typeface getGlobalFont()
	{
		return Config.sysFont;
	}
	
	/**
	 * 获取全局字体大小
	 * @return
	 */
	public static int getGlobalFontSize()
	{
		return Config.sysFontSize;
	}
	
	public static ArrayList<View> getSysControls()
	{
		return Config.sysControl;
	}
	
	public static String getIconFilePath()
	{
		return null;
	}
}

	
	