package wafly.framework.storyTK.component;

import java.util.ArrayList;

import wafly.framework.storyTK.Arena;
import wafly.framework.storyTK.basic.DrawableHelper;
import wafly.framework.storyTK.basic.StoreMgr;
import wafly.framework.storyTK.basic.Util;
import wafly.framework.storyTK.control.FrameAnimation;
import wafly.framework.storyTK.tmpl.Component;

/**
 * 对话框，在场景中显示角色对话
 * @author Wafly
 */
public class TalkBox extends Component
{
	private DrawableHelper backgroundDraw = null, faceDraw = null;
	private int location_Background_X = 0, location_Background_Y = 0;
	private int location_Text_X = 0, location_Text_Y = 0;
	private int location_Name_X = 0, location_Name_Y = 0;
	private int location_Face_X = 0, location_Face_Y = 0;
	private String talkContent = "", name = null;
	private int lineLength = 20, currentIndex = 0, backgroundColor = 0, needWait = 10, currentWait = 0, skipChrInFrame = 1;
	private int contentColor = 0xffffffff;
	private ArrayList<String> contentLines = new ArrayList<String>();
	private boolean isShowOut = true, destoryed = false;
	private Status status = Status.COMPLETED;
	private FrameAnimation completedPrompt = null;

	/**
	 * set the content to talkbox and show out, the content could be contains "/n" for switch line;<br>
	 * This method will block your thread.
	 * 
	 * @param content
	 *            the content you want to show
	 * @param hasNext
	 *            if has the next, the next content will shown continue the current content.
	 * @param isSkip
	 *            set false if you want to show the content one by one with setting speed, or set
	 *            true to show all the content at once.
	 */
	public void setContent(String content, boolean hasNext, boolean isSkip)
	{
		if (content == null)
			return;
		Arena.refuseInput("StoryScene.TalkBox.setContent", true);
		if (hasNext)
		{
			this.talkContent += content;
		}
		else
		{
			currentIndex = -1;
			this.talkContent = content;
		}
		if (!isShowOut)
		{
			Arena.refuseInput("StoryScene.TalkBox.setContent", false);
			return;
		}
		setShowOut(false);

		while (!isShowOut) {
			synchronized (this) {
				try {
					wait(1000 / Util.getMaxFPS());
				} catch (Exception e) {
					Util.logError("Error in thread.", e);
				}
			}
			if (++currentWait < needWait)
				continue;
			if (talkContent.length() == 0)
				talkContent = " ";
			if (currentIndex >= talkContent.length() - 1) {
				currentIndex = talkContent.length() - 1;
				setShowOut(true);
			} else {
				if (!isSkip)
					currentIndex += skipChrInFrame;
				else {
					currentIndex = talkContent.length() - 1;
					setShowOut(true);
				}
				if (talkContent.charAt(currentIndex) == '/' && currentIndex < talkContent.length() - 1)
					currentIndex++;
				contentLines.clear();
				String[] tmpContents = talkContent.substring(0, currentIndex + 1).split("/n");
				for (int i = 0; i < tmpContents.length; i++) {
					while (true) {
						if (tmpContents[i].length() > lineLength) {
							contentLines.add(tmpContents[i].substring(0, lineLength));
							tmpContents[i] = tmpContents[i].substring(lineLength);
						} else {
							contentLines.add(tmpContents[i]);
							break;
						}
					}
				}
			}
			currentWait = 0;
		}
		Arena.refuseInput("StoryScene.TalkBox.setContent", false);
	}

	/**
	 * 获取当前对话内容
	 * @return content
	 */
	public String getContent()
	{
		return talkContent;
	}

	/**
	 * 设置对话显示速度
	 * @param speed
	 *            shown speed
	 */
	public void setSpeed(int speed)
	{
		if (speed < 1)
		{
			speed = 1;
		}
		else if (speed > Util.getMaxFPS())
		{
			skipChrInFrame = speed / Util.getMaxFPS() + 1;
			speed = speed % Util.getMaxFPS();
			if (speed < 1)
				speed = 1;
		}
		else
		{
			skipChrInFrame = 1;
		}
		this.currentWait = this.needWait = Util.getMaxFPS() / speed;
	}

	/**
	 * 设置对话框背景
	 * @param backgroundPath
	 *            backgroung image file path, set null if you want to disable the background image
	 */
	public void setBackground(DrawableHelper backgroundDraw)
	{
		if(this.backgroundDraw!=null)
			this.backgroundDraw.destroy();
		this.backgroundDraw = backgroundDraw;
	}

	/**
	 * 设置对话框头像
	 * @param role
	 */
	public void setFace(DrawableHelper faceDraw)
	{
		this.faceDraw = faceDraw;
	}

	/**
	 * 获取对话框头像
	 * @return current role
	 */
	public DrawableHelper getFace()
	{
		return this.faceDraw;
	}

	/**
	 * 获取对话框角色名称
	 * @param role
	 */
	public void setName(String name)
	{
		this.name = name;
	}

	/**
	 * 获取对话框角色名称
	 * @return current role
	 */
	public String getName()
	{
		return this.name;
	}

	/**
	 * 设置对话框背景色，仅无背景图片时有效
	 * @param color
	 *            color value
	 */
	public void setBackgroundColor(int color)
	{
		this.backgroundColor = color;
		int[] argb = Util.getARGB(color);
		alpha=argb[3];
	}

	/**
	 * 获取背景色
	 * @return color value
	 */
	public int getBackgroundColor()
	{
		return backgroundColor;
	}

	/**
	 * 设置每一行对话的长度
	 * @param length
	 *            char count in one line
	 */
	public void setLineLength(int length)
	{
		this.lineLength = length;
	}

	/**
	 * 获取每一行文字长度
	 * @return char count in one line
	 */
	public int getLineLength()
	{
		return lineLength;
	}

	/**
	 * 设置文字颜色
	 */
	public void setTextColor(int color)
	{
		contentColor = color;
	}

	/**
	 * 获取文字颜色
	 */
	public int getTextColor()
	{
		return contentColor;
	}

	/**
	 * 设置背景图片位置
	 * @param x
	 * @param y
	 */
	public void setLocation_Background(int x, int y)
	{
		this.location_Background_X = x;
		this.location_Background_Y = y;
	}

	/**
	 * 获取背景图片横坐标位置
	 * @return x location
	 */
	public int left_Background()
	{
		return location_Background_X;
	}

	/**
	 * 获取背景图片纵坐标位置
	 * @return y location
	 */
	public int top_Background()
	{
		return location_Background_Y;
	}

	/**
	 * 设置对话框头像位置
	 * @param x
	 * @param y
	 */
	public void setLocation_Face(int x, int y)
	{
		this.location_Face_X = x;
		this.location_Face_Y = y;
	}

	/**
	 * 获取对话框头像位置
	 * @return x location
	 */
	public int left_Face()
	{
		return location_Face_X;
	}

	/**
	 * 获取对话框头像位置
	 * @return y location
	 */
	public int top_Face()
	{
		return location_Face_Y;
	}

	/**
	 * 设置角色名称位置
	 * @param x
	 * @param y
	 */
	public void setLocation_Name(int x, int y)
	{
		this.location_Name_X = x;
		this.location_Name_Y = y;
	}

	/**
	 * 获取角色名称位置
	 * @return x location
	 */
	public int left_Name()
	{
		return location_Name_X;
	}

	/**
	 * 获取角色名称位置
	 * @return y location
	 */
	public int top_Name()
	{
		return location_Name_Y;
	}

	/**
	 * 设置对话内容显示位置
	 * @param x
	 * @param y
	 */
	public void setLocation_Content(int x, int y)
	{
		this.location_Text_X = x;
		this.location_Text_Y = y;
	}

	/**
	 * 获取对话内容显示位置
	 * @return x location
	 */
	public int left_Content()
	{
		return location_Text_X;
	}

	/**
	 * 获取对话内容显示位置
	 * @return y location
	 */
	public int top_Content()
	{
		return location_Text_Y;
	}

	/**
	 * draw the talkbos to the canvas
	 * 
	 * @param canvas
	 *            usually scene
	 */
	public void onDraw(DrawableHelper draw)
	{
		if (status == Status.HIDE || destoryed)
			return;
        int tAlpha=draw.getDrawAlpha();
        if(tAlpha!=alpha)
            draw.setDrawAlpha(alpha);
		if (backgroundDraw == null)
			draw.drawColor(backgroundColor);
		else
			draw.drawImage(backgroundDraw, location_Background_X, location_Background_Y);
		if (faceDraw != null)
			draw.drawImage(faceDraw, location_Face_X, location_Face_Y);
		
        int tColor=draw.getForeColor();
        if(tColor!=contentColor)
            draw.setForeColor(contentColor);
		if(name!=null)
			draw.drawString(name, location_Name_X, location_Name_Y);
		for (int i = 0; i < contentLines.size(); i++)
			draw.drawString(contentLines.get(i), location_Text_X, (int) (location_Text_Y + i * Util.getGlobalFontSize()));
        if(tColor!=contentColor)
            draw.setForeColor(tColor);
        
		if (isShowOut && completedPrompt != null)
			completedPrompt.onDraw(draw);
		
        if(tAlpha!=alpha)
            draw.setDrawAlpha(tAlpha);
	}

	/**
	 * 对话框状态
	 */
	public enum Status
	{
		/**
		 * 对话内容显示完成
		 */
		COMPLETED, 
		/**
		 * 对话内容正在显示
		 */
		PERFORMING, 
		/**
		 * 对话框隐藏
		 */
		HIDE
	}

	/**
	 * 获取对话框当前状态
	 * @return talkbox current status
	 */
	public Status getStatus()
	{
		return status;
	}

	/**
	 * 隐藏或显示对话框
	 * @param visible
	 *            set false if want to hide the talkbox, otherwise set true
	 */
	public void setVisible(boolean visible)
	{
		if (!visible)
			status = Status.HIDE;
		else if (isShowOut)
			status = Status.COMPLETED;
		else
			status = Status.PERFORMING;
	}

	/**
	 * 判断对话框是否可见
	 */
	public boolean isVisible()
	{
		return status != Status.HIDE;
	}

	/**
	 * 设置对话内容显示完成后的提示符
	 * @param promptAnime
	 */
	public void setCompletedPrompt(FrameAnimation promptAnime)
	{
		if (completedPrompt != null)
			completedPrompt.destroy();
		this.completedPrompt = promptAnime;
		this.completedPrompt.setEndState(wafly.framework.storyTK.control.FrameAnimation.EndState.LOOP);
	}

	/**
	 * 销毁对话框的背景和完成提示符，不会销毁头像
	 */
	public void destroy()
	{
		destoryed = true;
		if (backgroundDraw != null)
		{
			backgroundDraw.destroy();
			backgroundDraw = null;
		}
		if (completedPrompt != null)
		{
			completedPrompt.destroy();
			completedPrompt = null;
		}
		Util.logInfo("TalkBox being destroyed.");
	}

	private void setShowOut(boolean isShowOut)
	{
		if (isShowOut && completedPrompt != null)
			completedPrompt.reset();
		this.isShowOut = isShowOut;
		if (status != Status.HIDE)
		{
			if (isShowOut)
				status = Status.COMPLETED;
			else
				status = Status.PERFORMING;
		}
	}

	@Override
	public void storeValue(StoreMgr storeMgr)
	{
		super.storeValue(storeMgr);
		if(backgroundDraw!=null)
			backgroundDraw.storeValue("TALKBOX."+ID+".BACKGROUNDDRAW", storeMgr);
//		if(faceDraw!=null)
//			faceDraw.storeValue("TALKBOX."+ID+".FACEDRAW", storeMgr);
		storeMgr.setValue("TALKBOX."+ID+".BACKGROUND.LOCATION.X", location_Background_X);
		storeMgr.setValue("TALKBOX."+ID+".BACKGROUND.LOCATION.Y", location_Background_Y);
		storeMgr.setValue("TALKBOX."+ID+".CONTENT.LOCATION.X", location_Text_X);
		storeMgr.setValue("TALKBOX."+ID+".CONTENT.LOCATION.Y", location_Text_Y);
		storeMgr.setValue("TALKBOX."+ID+".NAME.LOCATION.X", location_Name_X);
		storeMgr.setValue("TALKBOX."+ID+".NAME.LOCATION.Y", location_Name_Y);
		storeMgr.setValue("TALKBOX."+ID+".FACE.LOCATION.X", location_Face_X);
		storeMgr.setValue("TALKBOX."+ID+".FACE.LOCATION.Y", location_Face_Y);
		storeMgr.setValue("TALKBOX."+ID+".TALKCONTENT", talkContent);
		storeMgr.setValue("TALKBOX."+ID+".NAME", name);
		storeMgr.setValue("TALKBOX."+ID+".LINELENGTH", lineLength);
		storeMgr.setValue("TALKBOX."+ID+".BACKGROUNDCOLOR", backgroundColor);
		storeMgr.setValue("TALKBOX."+ID+".NEEDWAIT", needWait);
		storeMgr.setValue("TALKBOX."+ID+".SKIPCHRINFRAME", skipChrInFrame);
		storeMgr.setValue("TALKBOX."+ID+".CONTENT.COLOR", contentColor);
		storeMgr.setValue("TALKBOX."+ID+".STATUS", status);
		if(completedPrompt!=null)
		{
			storeMgr.setValue("TALKBOX."+ID+".COMPLETEDPROMPT.ID", completedPrompt.getID());
			completedPrompt.storeValue(storeMgr);
		}
	}
	
	public TalkBox(long id, StoreMgr storeMgr)
	{
		super(id, storeMgr);
		backgroundDraw = DrawableHelper.restoreValue("TALKBOX."+ID+".BACKGROUNDDRAW", storeMgr);
//		faceDraw = DrawableHelper.restoreValue("TALKBOX."+ID+".FACEDRAW", storeMgr);
		location_Background_X = Integer.parseInt(storeMgr.getValue("TALKBOX."+ID+".BACKGROUND.LOCATION.X"));
		location_Background_Y = Integer.parseInt(storeMgr.getValue("TALKBOX."+ID+".BACKGROUND.LOCATION.Y"));
		location_Text_X = Integer.parseInt(storeMgr.getValue("TALKBOX."+ID+".CONTENT.LOCATION.X"));
		location_Text_Y = Integer.parseInt(storeMgr.getValue("TALKBOX."+ID+".CONTENT.LOCATION.Y"));
		location_Name_X = Integer.parseInt(storeMgr.getValue("TALKBOX."+ID+".NAME.LOCATION.X"));
		location_Name_Y = Integer.parseInt(storeMgr.getValue("TALKBOX."+ID+".NAME.LOCATION.Y"));
		location_Face_X = Integer.parseInt(storeMgr.getValue("TALKBOX."+ID+".FACE.LOCATION.X"));
		location_Face_Y = Integer.parseInt(storeMgr.getValue("TALKBOX."+ID+".FACE.LOCATION.Y"));
		talkContent = storeMgr.getValue("TALKBOX."+ID+".TALKCONTENT");
		name = storeMgr.getValue("TALKBOX."+ID+".NAME");
		lineLength = Integer.parseInt(storeMgr.getValue("TALKBOX."+ID+".LINELENGTH"));
		backgroundColor = Integer.parseInt(storeMgr.getValue("TALKBOX."+ID+".BACKGROUNDCOLOR"));
		needWait = Integer.parseInt(storeMgr.getValue("TALKBOX."+ID+".NEEDWAIT"));
		skipChrInFrame = Integer.parseInt(storeMgr.getValue("TALKBOX."+ID+".SKIPCHRINFRAME"));
		contentColor = Integer.parseInt(storeMgr.getValue("TALKBOX."+ID+".CONTENT.COLOR"));
		status = Status.valueOf(storeMgr.getValue("TALKBOX."+ID+".STATUS"));
		try
		{
			completedPrompt = new FrameAnimation(Long.parseLong("TALKBOX."+ID+".COMPLETEDPROMPT.ID"), storeMgr);
		} catch (Exception e) { }
	}
	
	public TalkBox() {}

}
