package wafly.framework.storyTK.component;

import wafly.framework.storyTK.basic.DrawableHelper;
import wafly.framework.storyTK.basic.StoreMgr;
import wafly.framework.storyTK.basic.Util;
import wafly.framework.storyTK.tmpl.Component;

public class TrackAnimation extends Component
{
	private DrawableHelper frameDraw = null;
	private FrameType type = null;

	private int initWidth = 0, initHeight = 0;
	private int frameCount = 1, currentFrameIndex = 0;

	private int startLocation_X = 0, startLocation_Y = 0, endLocation_X = 0, endLocation_Y = 0;
	private int everyX = 0, everyY = 0, currentX = 0, currentY = 0;

	private float startPercentage_Width = 1, startPercentage_Height = 1, endPercentage_Width = 1, endPercentage_Height = 1;
	private float everyPercentage_Width = 0, everyPercentage_Height = 0, currentPercentage_Width = 1, currentPercentage_Height = 1;

	private int startAlpha = 255, endAlpha = 255;
	private int everyAlpha = 0, currentAlpha = 255;

	private EndState endState = EndState.HOLD;
	private Status currentState = Status.INITIALIZED;
	private boolean isStarted = false, fillStyle = true, visible = true;
	private int color = 0xffffffff;

	/**
	 * init an animation via animation profile.
	 * 
	 * @param frame
	 *            the image for frame show
	 */
	public TrackAnimation(DrawableHelper frameDraw, int frameCount)
	{
		this.frameCount = frameCount;
		if (frameDraw == null)
			frameDraw = new DrawableHelper(1, 1);
		this.frameDraw=frameDraw;
		initWidth = frameDraw.getWidth();
		initHeight = frameDraw.getHeight();
	}

	/**
	 * init an animation via FrameType
	 * 
	 * @param type
	 *            OVAL, RECT or LINE
	 * @param originalWidth
	 * @param originalHeight
	 * @param frameCount
	 *            total count of the frame for animation
	 */
	public TrackAnimation(FrameType type, int originalWidth, int originalHeight, int frameCount)
	{
		this.type = type;
		this.initWidth = originalWidth;
		this.initHeight = originalHeight;
		this.frameCount = frameCount;
	}

	/**
	 * get the animation type
	 * 
	 * @return animation type or null if use image path to init.
	 */
	public FrameType getFrameType()
	{
		return type;
	}

	/**
	 * should be setted before animation started
	 * 
	 * @param isFill
	 */
	public void setFillStyle(boolean isFill)
	{
		if (isStarted)
			return;
		fillStyle = isFill;
	}

	/**
	 * should be setted before animation started
	 * 
	 * @param color
	 */
	public void setColor(int color)
	{
		if (isStarted)
			return;
		this.color = color;
	}

	/**
	 * get animation total frame count
	 * @return
	 */
	public int getFrameCount()
	{
		return frameCount;
	}

	/**
	 * to set the start location and end location for the animation
	 * 
	 * @param startX
	 * @param startY
	 * @param endX
	 * @param endY
	 */
	public void setLocationTrack(int startX, int startY, int endX, int endY)
	{
		currentX = this.startLocation_X = startX;
		currentY = this.startLocation_Y = startY;
		this.endLocation_X = endX;
		this.endLocation_Y = endY;
		everyX = (endLocation_X - startLocation_X) / frameCount;
		everyY = (endLocation_Y - startLocation_Y) / frameCount;
	}

	/**
	 * get location of animation start from
	 * @return
	 */
	public int getStartLeft()
	{
		return startLocation_X;
	}

	/**
	 * get location of animation start from
	 * @return
	 */
	public int getStartTop()
	{
		return startLocation_Y;
	}

	/**
	 * get location of animation end to
	 * @return
	 */
	public int getEndLeft()
	{
		return endLocation_X;
	}

	/**
	 * get location of animation end to
	 * @return
	 */
	public int getEndTop()
	{
		return endLocation_Y;
	}

	/**获取正在进行中的当前横轴坐标*/
	public int getCurrentLeft(){ return currentX; }

	/**获取正在进行中的当前纵轴坐标*/
	public int getCurrentTop(){ return currentY; }

	/**
	 * set the start percentage and end percentage for the animation
	 * 
	 * @param startPercentage
	 *            should be >0; 1 as original, >1 means bigger, <1 means smaller
	 * @param endPercentage
	 *            should be >0; 1 as original, >1 means bigger, <1 means smaller
	 */
	public void setPercentageTrack(double startWidthPercentage, double startHeightPercentage, double endWidthPercentage, double endHeightPercentage)
	{
		currentPercentage_Width = this.startPercentage_Width = (float) startWidthPercentage;
		currentPercentage_Height = this.startPercentage_Height = (float) startHeightPercentage;
		this.startPercentage_Width = (float) startWidthPercentage;
		this.startPercentage_Height = (float) startHeightPercentage;
		this.endPercentage_Width = (float) endWidthPercentage;
		this.endPercentage_Height = (float) endHeightPercentage;
		everyPercentage_Width = (endPercentage_Width - startPercentage_Width) / frameCount;
		everyPercentage_Height = (endPercentage_Height - startPercentage_Height) / frameCount;
	}

	/**
	 * get size percentage of animation start from
	 * @return
	 */
	public float getStartWidthPercentage()
	{
		return startPercentage_Width;
	}

	/**
	 * get size percentage of animation start from
	 * @return
	 */
	public float getStartHeightPercentage()
	{
		return startPercentage_Height;
	}

	/**
	 * get size percentage of animation end to
	 * @return
	 */
	public float getEndWidthPercentage()
	{
		return endPercentage_Width;
	}

	/**
	 * get size percentage of animation end to
	 * @return
	 */
	public float getEndHeightPercentage()
	{
		return endPercentage_Height;
	}

	/**获取正在进行中的当前宽度缩放比例*/
	public float getCurrentWidthPercentage(){ return currentPercentage_Width; }

	/**获取正在进行中的当前宽度*/
	public int getCurrentWidth(){ return (int) (frameDraw.getWidth()*currentPercentage_Width); }

	/**获取正在进行中的当前高度缩放比例*/
	public float getCurrentHeightPercentage(){ return currentPercentage_Height; }

	/**获取正在进行中的当前高度*/
	public int getCurrentHeight(){ return (int) (frameDraw.getHeight()*currentPercentage_Height); }

	/**
	 * set the start alpha and end alpha for the animation
	 * 
	 * @param startAlpha
	 *            should be >0 and <255
	 * @param endAlpha
	 *            should be >0 and <255
	 */
	public void setAlphaTrack(int startAlpha, int endAlpha)
	{
		currentAlpha = this.startAlpha = startAlpha;
		this.endAlpha = endAlpha;
		everyAlpha = (this.endAlpha - this.startAlpha) / frameCount;
	}

	/**
	 * get alpha of animation start from
	 * @return
	 */
	public int getStartAlpha()
	{
		return startAlpha;
	}

	/**
	 * get alpha of animation end to
	 * @return
	 */
	public int getEndAlpha()
	{
		return endAlpha;
	}
	
	/**获取正在进行中的当前透明度*/
	public int getCurrentAlpha(){ return currentAlpha; }
	
	@Override
	public void setAlpha(int alpha)
	{
		currentAlpha = startAlpha = endAlpha = alpha;
	}
	
	@Override
	public int getAlpha()
	{
		return currentAlpha;
	}

	/**
	 * set the state for the end of the Animation, it will reset this animation to init status
	 * 
	 * @param endState
	 *            end state
	 */
	public void setEndState(EndState endState)
	{
		reset();
		this.endState = endState;
	}

	/**
	 * get the end state of the animation
	 * 
	 * @return end state
	 */
	public EndState getEndState()
	{
		return endState;
	}

	public int getCurrentFrameIndex()
	{
		return currentFrameIndex;
	}

	/**
	 * to reset the animation to the first frame
	 */
	public void reset()
	{
		currentFrameIndex = 0;
		currentX = startLocation_X;
		currentY = startLocation_Y;
		currentPercentage_Width = startPercentage_Width;
		currentPercentage_Height = startPercentage_Height;
		currentAlpha = startAlpha;
	}
	
	public void stop(){
		currentFrameIndex = frameCount;
	}

	/**
	 * the state for end of animation
	 */
	public enum EndState
	{
		HIDE, HOLD, LOOP
	};

	/**
	 * the current status of the animation
	 */
	public enum Status
	{
		INITIALIZED, HOLD, RUNNING, DESTROYED
	}

	/**
	 * the shape of the animation
	 */
	public enum FrameType
	{
		OVAL, RECT, LINE
	}

	/**
	 * @return the current status, initialized, running, hold or destroyed.
	 */
	public Status getCurrentState()
	{
		return currentState;
	}

	@Override
	public void destroy()
	{
		currentState = Status.DESTROYED;
		if (frameDraw != null)
		{
			frameDraw.destroy();
			frameDraw = null;
		}
		Util.logInfo("TrackAnimation being destroyed.");
	}

	public boolean isVisible()
	{
		return visible;
	}

	public void setVisible(boolean isVisible)
	{
		this.visible = isVisible;
	}

	@Override
	public void onDraw(DrawableHelper draw)
	{
		isStarted = true;
		if (currentFrameIndex < 0 || currentState == Status.DESTROYED || !visible)
			return;
		if (currentFrameIndex >= frameCount)
		{
			if (endState == EndState.HIDE)
			{
				currentFrameIndex = -1;
//				destroy();
				return;
			}
			else if (endState == EndState.HOLD)
			{
				currentFrameIndex = frameCount - 1;
				currentState = Status.HOLD;
			}
			else if (endState == EndState.LOOP)
			{
				reset();
			}
		}
		currentFrameIndex++;

		int tAlpha = draw.getDrawAlpha();
		if (tAlpha != currentAlpha)
			draw.setDrawAlpha(currentAlpha);
		if (frameDraw != null)
		{
			if (everyPercentage_Width != 0 || everyPercentage_Height != 0)
			{
				frameDraw.zoomTo((int) (initWidth * currentPercentage_Width), (int) (initHeight * currentPercentage_Height));
			}
			draw.drawImage(frameDraw, currentX, currentY);
		}
		else if (type != null)
		{
			int tColor=draw.getForeColor();
			if(color!=tColor)
				draw.setForeColor(color);
			switch (type)
			{
				case LINE:
					draw.drawLine(startLocation_X, startLocation_Y,
							currentX, currentY);
					break;
				case OVAL:
					draw.drawOval(currentX, currentY,
							(int)(initWidth*currentPercentage_Width),
							(int)(currentY+initHeight*currentPercentage_Width), fillStyle);
					break;
				case RECT:
					draw.drawRect(currentX, currentY,
							(int)(initWidth*currentPercentage_Width),
							(int)(currentY+initHeight*currentPercentage_Width), fillStyle);
					break;
			}
			if(color!=tColor)
				draw.setForeColor(tColor);
		}
		if (tAlpha != currentAlpha)
			draw.setDrawAlpha(tAlpha);
		if (currentState != Status.HOLD)
		{
			currentState = Status.RUNNING;
			currentX += everyX;
			currentY += everyY;
			currentPercentage_Width += everyPercentage_Width;
			currentPercentage_Height += everyPercentage_Height;
			currentAlpha += everyAlpha;
			if (currentAlpha > 255)
				currentAlpha = 255;
			if (currentAlpha < 0)
				currentAlpha = 0;
		}
	}
	
	public boolean isPlaying(){
		if(endState!=EndState.HOLD && currentFrameIndex>-1)
			return true;
		if(endState==EndState.HOLD && currentFrameIndex<frameCount-1)
			return true;
		return false;
	}
	
	public boolean isHide(){
		return (endState==EndState.HIDE && currentFrameIndex==-1);
	}

	@Override
	public void storeValue(StoreMgr storeMgr)
	{
		super.storeValue(storeMgr);
		storeMgr.setValue("TANIMATION."+ID+".INITWIDTH", initWidth);
		storeMgr.setValue("TANIMATION."+ID+".INITHEIGHT", initHeight);
		storeMgr.setValue("TANIMATION."+ID+".FRAMECOUNT", frameCount);
		storeMgr.setValue("TANIMATION."+ID+".START.LOCATION.X", startLocation_X);
		storeMgr.setValue("TANIMATION."+ID+".START.LOCATION.Y", startLocation_Y);
		storeMgr.setValue("TANIMATION."+ID+".END.LOCATION.X", endLocation_X);
		storeMgr.setValue("TANIMATION."+ID+".END.LOCATION.Y", endLocation_Y);
		storeMgr.setValue("TANIMATION."+ID+".EVERY.LOCATION.X", everyX);
		storeMgr.setValue("TANIMATION."+ID+".EVERY.LOCATION.Y", everyY);
		storeMgr.setValue("TANIMATION."+ID+".START.PERCENTAGE.WIDTH", startPercentage_Width);
		storeMgr.setValue("TANIMATION."+ID+".START.PERCENTAGE.HEIGHT", startPercentage_Height);
		storeMgr.setValue("TANIMATION."+ID+".END.PERCENTAGE.WIDTH", endPercentage_Width);
		storeMgr.setValue("TANIMATION."+ID+".END.PERCENTAGE.HEIGHT", endPercentage_Height);
		storeMgr.setValue("TANIMATION."+ID+".EVERY.PERCENTAGE.WIDTH", everyPercentage_Width);
		storeMgr.setValue("TANIMATION."+ID+".EVERY.PERCENTAGE.HEIGHT", everyPercentage_Height);
		storeMgr.setValue("TANIMATION."+ID+".START.ALPHA", startAlpha);
		storeMgr.setValue("TANIMATION."+ID+".END.ALPHA", endAlpha);
		storeMgr.setValue("TANIMATION."+ID+".EVERY.ALPHA", everyAlpha);
		storeMgr.setValue("TANIMATION."+ID+".ENDSTATE", endState);
		storeMgr.setValue("TANIMATION."+ID+".FILLSTYLE", fillStyle);
		storeMgr.setValue("TANIMATION."+ID+".VISIBLE", visible);
		storeMgr.setValue("TANIMATION."+ID+".COLOR", color);
		storeMgr.setValue("TANIMATION."+ID+".TYPE", type);
		if(frameDraw!=null)
			frameDraw.storeValue("TANIMATION."+ID, storeMgr);
	}
	
	public TrackAnimation(long id, StoreMgr storeMgr)
	{
		super(id, storeMgr);
		initWidth = Integer.parseInt(storeMgr.getValue("TANIMATION."+ID+".INITWIDTH"));
		initHeight = Integer.parseInt(storeMgr.getValue("TANIMATION."+ID+".INITHEIGHT"));
		frameCount = Integer.parseInt(storeMgr.getValue("TANIMATION."+ID+".FRAMECOUNT"));
		type = FrameType.valueOf(storeMgr.getValue("TANIMATION."+ID+".TYPE"));
		color = Integer.parseInt(storeMgr.getValue("TANIMATION."+ID+".COLOR"));
		startLocation_X = Integer.parseInt(storeMgr.getValue("TANIMATION."+ID+".START.LOCATION.X"));
		startLocation_Y = Integer.parseInt(storeMgr.getValue("TANIMATION."+ID+".START.LOCATION.Y"));
		endLocation_X = Integer.parseInt(storeMgr.getValue("TANIMATION."+ID+".END.LOCATION.X"));
		endLocation_Y = Integer.parseInt(storeMgr.getValue("TANIMATION."+ID+".END.LOCATION.Y"));
		everyX = Integer.parseInt(storeMgr.getValue("TANIMATION."+ID+".EVERY.LOCATION.X"));
		everyY = Integer.parseInt(storeMgr.getValue("TANIMATION."+ID+".EVERY.LOCATION.Y"));
		startPercentage_Width = Float.parseFloat(storeMgr.getValue("TANIMATION."+ID+".START.PERCENTAGE.WIDTH"));
		startPercentage_Height = Float.parseFloat(storeMgr.getValue("TANIMATION."+ID+".START.PERCENTAGE.HEIGHT"));
		endPercentage_Width = Float.parseFloat(storeMgr.getValue("TANIMATION."+ID+".END.PERCENTAGE.WIDTH"));
		endPercentage_Height = Float.parseFloat(storeMgr.getValue("TANIMATION."+ID+".END.PERCENTAGE.HEIGHT"));
		everyPercentage_Width = Float.parseFloat(storeMgr.getValue("TANIMATION."+ID+".EVERY.PERCENTAGE.WIDTH"));
		everyPercentage_Height = Float.parseFloat(storeMgr.getValue("TANIMATION."+ID+".EVERY.PERCENTAGE.HEIGHT"));
		startAlpha = Integer.parseInt(storeMgr.getValue("TANIMATION."+ID+".START.ALPHA"));
		endAlpha = Integer.parseInt(storeMgr.getValue("TANIMATION."+ID+".END.ALPHA"));
		everyAlpha = Integer.parseInt(storeMgr.getValue("TANIMATION."+ID+".EVERY.ALPHA"));
		endState = EndState.valueOf(storeMgr.getValue("TANIMATION."+ID+".ENDSTATE"));
		fillStyle = Boolean.getBoolean(storeMgr.getValue("TANIMATION."+ID+".FILLSTYLE"));
		visible = Boolean.parseBoolean(storeMgr.getValue("TANIMATION."+ID+".VISIBLE"));
	}

}