package wafly.framework.storyTK.component;

import java.util.ArrayList;
import java.util.Random;

import wafly.framework.storyTK.Arena;
import wafly.framework.storyTK.basic.DrawableHelper;
import wafly.framework.storyTK.basic.StoreMgr;
import wafly.framework.storyTK.basic.Util;
import wafly.framework.storyTK.tmpl.Component;

/**
 * show weather onto scene
 * 
 * @author Wafly
 */
public class Weather extends Component
{
	private WeatherType type = null;
	private Integer[] weatherLocations_X = null, weatherLocations_Y = null, size = null;
	private int speed = 10, speed_x = 0, speed_y = 10;
	private float angle = 269.9f;
	private int currentWait = 0, flashStatuCount = 4;
	private boolean isFlash = false, visible = true;
	private DrawableHelper customWeatherDraw = null;

	/**
	 * the type of the weather
	 */
	public enum WeatherType
	{
		RAIN, SNOW, FLASH, CUSTOMIZATION, NULL
	}

	/**
	 * init a weather
	 * 
	 * @param type
	 *            weather's type
	 * @param magnitude
	 *            weather's size
	 */
	public Weather(WeatherType type, int count)
	{
		this.type = type;
		if(type==WeatherType.FLASH)
		{
			alpha=0;
			flashStatuCount=count;
		}
		if (count < 0)
			count = 0;
		weatherLocations_X = new Integer[count];
		weatherLocations_Y = new Integer[count];
		size = new Integer[count];
		Random rand = new Random();
		for (int i = 0; i < count; i++)
		{
			weatherLocations_X[i] = rand.nextInt(Arena.getInstance().getWidth() - 1);
			weatherLocations_Y[i] = rand.nextInt(Arena.getInstance().getHeight());
			size[i] = rand.nextInt(5) + 1;
		}
	}
	
	public Weather(long id, StoreMgr storeMgr)
	{
		super(id, storeMgr);
		type = WeatherType.valueOf(storeMgr.getValue("WEATHER."+ID+".TYPE"));
		if(type==WeatherType.FLASH)
		{
			destroy();
			return;
		}
		ArrayList<Integer> sizeArray = new ArrayList<Integer>();
		for(int i=0;;i++)
		{
			try {
				sizeArray.add(Integer.parseInt(storeMgr.getValue("WEATHER."+ID+".SIZE."+i)));
			} catch(Exception e) {
				break;
			}
		}
		size = new Integer[sizeArray.size()];
		sizeArray.toArray(size);
		weatherLocations_X = new Integer[size.length];
		weatherLocations_Y = new Integer[size.length];
		for(int i=0; i<size.length; i++)
		{
			weatherLocations_X[i] = Integer.parseInt(storeMgr.getValue("WEATHER."+ID+".LOCATION."+i+".X"));
			weatherLocations_Y[i] = Integer.parseInt(storeMgr.getValue("WEATHER."+ID+".LOCATION."+i+".Y"));
		}
		setSpeed(Integer.parseInt(storeMgr.getValue("WEATHER."+ID+".SPEED")));
		setAngle(Float.parseFloat(storeMgr.getValue("WEATHER."+ID+".ANGLE")));
		visible=Boolean.parseBoolean(storeMgr.getValue("WEATHER."+ID+".VISIBLE"));
		customWeatherDraw = DrawableHelper.restoreValue("WEATHER."+ID, storeMgr);
	}

	/**
	 * to get the current weather's type
	 * 
	 * @return weather type
	 */
	public WeatherType getType()
	{
		return type;
	}

	/**
	 * to get the weather's size
	 * 
	 * @return weather size
	 */
	public int getCount()
	{
		return size.length;
	}

	/**
	 * 设置天气执行速度
	 * @param speed
	 */
	public void setSpeed(int speed)
	{
		this.speed = speed;
		if(type==WeatherType.FLASH)
		{
			if(this.speed>255)
				this.speed=255;
			else if(this.speed<1)
				this.speed=1;
		}
		speed_x = (int) (speed * Math.cos(Math.toRadians(angle)));
		speed_y = (int) (speed * Math.sin(Math.toRadians(angle)));
	}

	/**
	 * 获取天气演示速度
	 * @return
	 */
	public int getSpeed()
	{
		return speed;
	}

	/**
	 * 设置天气透明度
	 * @param angle
	 */
	public void setAngle(float angle)
	{
		this.angle = angle % 90 == 0 ? angle - 0.1f : angle;
		setSpeed(this.speed);
	}

	/**
	 * 获取天气透明度
	 * @return
	 */
	public float getAngle()
	{
		return this.angle;
	}

	/**
	 * 自定义天气图片
	 * @param weatherDraw
	 */
	public void setCustWeatherUnit(DrawableHelper weatherDraw)
	{
		this.customWeatherDraw = weatherDraw;
	}

	@Override
	public void destroy()
	{
		this.type = WeatherType.NULL;
		if (customWeatherDraw != null)
		{
			customWeatherDraw.destroy();
			customWeatherDraw = null;
		}
		Util.logInfo("Weather being destroyed.");
	}

	@Override
	public boolean isVisible()
	{
		return visible;
	}

	@Override
	public void setVisible(boolean isVisible)
	{
		this.visible = isVisible;
	}

	@Override
	public void onDraw(DrawableHelper draw)
	{
		if (!visible)
			return;
		int tColor=draw.getForeColor();
		if(tColor!=0xffffffff)
			draw.setForeColor(0xffffffff);
		if(type==WeatherType.FLASH)
		{
			if (isFlash)
			{
				alpha-=255*currentWait/speed;
				if(alpha<0)
					alpha=0;
			}
			else
			{
				alpha+=255*currentWait/speed;
				if(alpha>255)
					alpha=255;
			}
			draw.drawColor(alpha*0x1000000+0xffffff);
			if (++currentWait >= speed)
			{
				currentWait = 0;
				if (isFlash&&--flashStatuCount < 0)
					destroy();
				isFlash = !isFlash;
			}
		}
		else
		{
			int tAlpha = draw.getDrawAlpha();
			if(tAlpha!=alpha)
				draw.setDrawAlpha(alpha);
			switch (type)
			{
				case RAIN:
					for (int i = 0; i < size.length; i++) {
						weatherLocations_X[i] += speed_x;
						weatherLocations_Y[i] -= speed_y;
						draw.drawLine(weatherLocations_X[i], weatherLocations_Y[i],
								weatherLocations_X[i] + speed_x * size[i],
								weatherLocations_Y[i] - speed_y * size[i]);
						if (weatherLocations_Y[i] > Arena.getInstance().getHeight())
							weatherLocations_Y[i] = 0;
						else if (weatherLocations_Y[i] < 0)
							weatherLocations_Y[i] = Arena.getInstance().getHeight();
						if (weatherLocations_X[i] > Arena.getInstance().getWidth())
							weatherLocations_X[i] = 0;
						else if (weatherLocations_X[i] < 0)
							weatherLocations_X[i] = Arena.getInstance().getWidth();
					}
					break;
				case SNOW:
					for (int i = 0; i < size.length; i++) {
						weatherLocations_X[i] += speed_x;
						weatherLocations_Y[i] -= speed_y;
						draw.drawOval(weatherLocations_X[i], weatherLocations_Y[i], size[i], size[i], true);
						if (weatherLocations_Y[i] > Arena.getInstance().getHeight())
							weatherLocations_Y[i] = 0;
						else if (weatherLocations_Y[i] < 0)
							weatherLocations_Y[i] = Arena.getInstance().getHeight();
						if (weatherLocations_X[i] > Arena.getInstance().getWidth())
							weatherLocations_X[i] = 0;
						else if (weatherLocations_X[i] < 0)
							weatherLocations_X[i] = Arena.getInstance().getWidth();
					}
					break;
				case CUSTOMIZATION:
					if (customWeatherDraw != null) {
						for (int i = 0; i < size.length; i++) {
							weatherLocations_X[i] += speed_x;
							weatherLocations_Y[i] -= speed_y;

							customWeatherDraw.zoomTo((int) (0.5 * size[i]), (int) (0.5 * size[i]));
							draw.drawImage(customWeatherDraw, weatherLocations_X[i], weatherLocations_Y[i]);
							if (weatherLocations_Y[i] > Arena.getInstance().getHeight())
								weatherLocations_Y[i] = 0;
							else if (weatherLocations_Y[i] < 0)
								weatherLocations_Y[i] = Arena.getInstance().getHeight();
							if (weatherLocations_X[i] > Arena.getInstance().getWidth())
								weatherLocations_X[i] = 0;
							else if (weatherLocations_X[i] < 0)
								weatherLocations_X[i] = Arena.getInstance().getWidth();
						}
					}
					break;
				default:
					break;
			}
			if(tAlpha!=alpha)
				draw.setDrawAlpha(tAlpha);
		}
		if(tColor!=0xffffffff)
			draw.setForeColor(tColor);
	}

	@Override
	public void storeValue(StoreMgr storeMgr)
	{
		super.storeValue(storeMgr);
		storeMgr.setValue("WEATHER."+ID+".TYPE", type);
		for(int i=0; i<size.length; i++)
		{
			storeMgr.setValue("WEATHER."+ID+".LOCATION."+i+".X", weatherLocations_X);
			storeMgr.setValue("WEATHER."+ID+".LOCATION."+i+".Y", weatherLocations_Y);
			storeMgr.setValue("WEATHER."+ID+".SIZE."+i, size);
		}
		storeMgr.setValue("WEATHER."+ID+".SPEED", speed);
		storeMgr.setValue("WEATHER."+ID+".ANGLE", angle);
		storeMgr.setValue("WEATHER."+ID+".VISIBLE", visible);
		if(customWeatherDraw!=null)
			customWeatherDraw.storeValue("WEATHER."+ID, storeMgr);
	}

}
