package wafly.framework.storyTK.control;

import android.annotation.SuppressLint;
import wafly.framework.storyTK.basic.DrawableHelper;
import wafly.framework.storyTK.basic.StoreMgr;
import wafly.framework.storyTK.basic.Util;
import wafly.framework.storyTK.control.FrameAnimation.EndState;
import wafly.framework.storyTK.listener.ActionListener;
import wafly.framework.storyTK.listener.TouchListener;
import wafly.framework.storyTK.listener.TouchReleaseListener;
import wafly.framework.storyTK.tmpl.OperationalControl;

/**
 * 按钮<br>
 * 默认动作：操作按下再弹起（一次完整点击）
 * @author Wafly
 */
public class Button extends OperationalControl
{
//	private Bitmap normalView=null, downView=null, disableView=null;
	private FrameAnimation normalAnime=null, downAnime=null, disableAnime=null, focusAnime=null;
	private DrawableHelper normalDraw = null, focusDraw=null, downDraw = null, disableDraw = null;
	private String text = null;
//	private float textSize=20;
	private int textColor=0;
//	private boolean isPrePerforming=false;
	private int titleLocationX=0, titleLocationY=0;
	private int keyCode=-1;

	/**
	 * 通过外观图片初始化一个按钮
	 */
	public Button(DrawableHelper backImageDrawable)
	{
		this.normalDraw = backImageDrawable;
		width = backImageDrawable.getWidth();
		height = backImageDrawable.getHeight();
		textColor = Util.getInverseColorValue(Util.getBackgroundColor());
		addTouchListener(new TouchListener() { public void onPerform(int x, int y) {
			Button.this.currentStatus = Status.DOWN;
		}});
		addReleaseListener(new TouchReleaseListener() { public void onPerform(int x, int y) {
			if(Button.this.currentStatus == Status.DOWN){
				for(ActionListener listener : actionListenerList){
					listener.onPerform();
				}
				Button.this.currentStatus = Status.FOCUS;
			}	
		}});
	}

	/**
	 * 通过指定的范围初始化一个按钮
	 */
	public Button(int x, int y, int width, int height)
	{
		setLocation(x, y);
		this.width = width;
		this.height = height;
		textColor = Util.getInverseColorValue(Util.getBackgroundColor());
		addTouchListener(new TouchListener() { public void onPerform(int x, int y) {
			Button.this.currentStatus = Status.DOWN;
		}});
		addReleaseListener(new TouchReleaseListener() { public void onPerform(int x, int y) {
			if(Button.this.currentStatus == Status.DOWN){
				for(ActionListener listener : actionListenerList){
					listener.onPerform();
				}
				Button.this.currentStatus = Status.FOCUS;
			}	
		}});
	}
	
	/**
	 * 初始化一个动态按钮
	 */
	public Button(String animeProfile)
	{
		if(animeProfile==null)
			Util.logError("Animation profile could not be null", null);
		normalAnime=new FrameAnimation(animeProfile);
		normalAnime.setEndState(EndState.LOOP);
		width=normalAnime.getWidth();
		height=normalAnime.getHeight();
		textColor = Util.getInverseColorValue(Util.getBackgroundColor());
		addTouchListener(new TouchListener() { public void onPerform(int x, int y) {
			Button.this.currentStatus = Status.DOWN;
		}});
		addReleaseListener(new TouchReleaseListener() { public void onPerform(int x, int y) {
			if(Button.this.currentStatus == Status.DOWN){
				for(ActionListener listener : actionListenerList){
					listener.onPerform();
				}
				Button.this.currentStatus = Status.FOCUS;
			}	
		}});
	}

	/**
	 * set the button's caption
	 * @param text button's caption
	 */
	public void setText(String text)
	{
		this.text=(text==null?"":text.trim());
		titleLocationX = (int) ((width - Util.calStringPixLength(this.text))/2);
		titleLocationY = (int) ((height - Util.getGlobalFontSize())/2);
	}
	
	/**
	 * get the caption of the button
	 * @return the caption of the button
	 */
	public String getText() { return text; }
	
	/**
	 * 设置按钮聚焦时的外观(图片)<br>
	 * set the image of button on the states of "onFocus"
	 */
	public void setFocusView(DrawableHelper focusDraw)
	{
		if (this.focusDraw != null)
		{
			this.focusDraw.destroy();
			this.focusDraw = null;
		}
		if (this.focusAnime != null)
		{
			this.focusAnime.destroy();
			this.focusAnime = null;
		}
		this.focusDraw = focusDraw;
	}

	/**
	 * 设置按钮聚焦时的外观(动画)<br>
	 * set the anime of button on the states of "onFocus"
	 */
	public void setFocusView(FrameAnimation focusAnime)
	{
		if (this.focusAnime != null)
		{
			this.focusAnime.destroy();
			this.focusAnime = null;
		}
		if (this.focusDraw != null)
		{
			this.focusDraw.destroy();
			this.focusDraw = null;
		}
		this.focusAnime = focusAnime;
		if (focusAnime != null)
			this.focusAnime.setEndState(EndState.LOOP);
	}

	/**
	 * 设置按钮按下时的外观(图片)<br>
	 * set the image of button on the states of "onTouch()"
	 */
	public void setDownView(DrawableHelper downDraw)
	{
		if (this.downDraw != null)
		{
			this.downDraw.destroy();
			this.downDraw = null;
		}
		if (this.downAnime != null)
		{
			this.downAnime.destroy();
			this.downAnime = null;
		}
		this.downDraw = downDraw;
	}

	/**
	 * 设置按钮按下时的外观(动画)<br>
	 * set the anime of button on the states of "onTouch()"
	 */
	public void setDownView(FrameAnimation downAnimation)
	{
		if (this.downAnime != null)
		{
			this.downAnime.destroy();
			this.downAnime = null;
		}
		if (this.downDraw != null)
		{
			this.downDraw.destroy();
			this.downDraw = null;
		}
		this.downAnime = downAnimation;
		if (downAnimation != null)
			this.downAnime.setEndState(EndState.LOOP);
	}

	/**
	 * 设置按钮不可用时的外观(图片)<br>
	 * set the image of button on the states of DISABLE
	 * 
	 * @param disableView
	 *            the image of button for DISABLE status
	 */
	public void setDisableView(DrawableHelper disableDraw)
	{
		if (this.disableDraw != null)
		{
			this.disableDraw.destroy();
			this.disableDraw = null;
		}
		if (this.disableAnime != null)
		{
			this.disableAnime.destroy();
			this.disableAnime = null;
		}
		this.disableDraw = disableDraw;
	}

	/**
	 * 设置按钮不可用时的外观(动画)<br>
	 * set the anime of button on the states of DISABLE
	 */
	public void setDisableView(FrameAnimation disableAnimation)
	{
		if (this.disableAnime != null)
		{
			this.disableAnime.destroy();
			this.disableAnime = null;
		}
		if (this.disableDraw != null)
		{
			this.disableDraw.destroy();
			this.disableDraw = null;
		}
		this.disableAnime = disableAnimation;
		if (disableAnimation != null)
			this.disableAnime.setEndState(EndState.LOOP);
	}
	
	/**
	 * set the font color of button caption
	 * @param color font color
	 */
	public void setColor(int color) { textColor=color; }
	
	/**
	 * @return the font color of button caption
	 */
	public int getColor() { return textColor; }

	@Override
	public void setAlpha(int alpha)
	{
		super.setAlpha(alpha);
		if (normalAnime != null)
			normalAnime.setAlpha(alpha);
		if (focusAnime != null)
			focusAnime.setAlpha(alpha);
		if (downAnime != null)
			downAnime.setAlpha(alpha);
		if (disableAnime != null)
			disableAnime.setAlpha(alpha);
	}
	
	/**
	 * 设置按钮的键值，设置之后对此按钮的Touch和LongTouch操作将映射到指定按键的KeyDown和LongPress操作
	 * @param keyCode
	 */
	public void setKeyCode(int keyCode)
	{
		this.keyCode = keyCode;
	}
	
	/**
	 * 获取此按钮对应的键值
	 * @return 没有对应键值则返回-1
	 */
	public int getKeyCode()
	{
		return keyCode;
	}

	/**
	 * 在画布上绘出按钮<br>
	 * to draw the button to canvas
	 */
	@SuppressLint("WrongCall")
	public void onDraw(DrawableHelper draw)
	{
		if (destoryed || currentStatus==Status.HIDE)
			return;
		int tAlpha = draw.getDrawAlpha();
		if (tAlpha != alpha)
			draw.setDrawAlpha(alpha);
		switch (currentStatus)
		{
			case NORMAL:
				if (normalDraw != null)
					draw.drawImage(normalDraw, relaX + location_X, relaY + location_Y);
				else if (normalAnime != null)
					normalAnime.onDraw(draw, relaX + location_X, relaY + location_Y);
				break;
			case FOCUS:
				if (focusDraw != null)
					draw.drawImage(focusDraw, relaX + location_X, relaY + location_Y);
				else if (focusAnime != null)
					focusAnime.onDraw(draw, relaX + location_X, relaY + location_Y);
				else if (normalDraw != null)
					draw.drawImage(normalDraw, relaX + location_X, relaY + location_Y);
				else if (normalAnime != null)
					normalAnime.onDraw(draw, relaX + location_X, relaY + location_Y);
				break;
			case DOWN:
				if (downDraw != null)
					draw.drawImage(downDraw, relaX + location_X, relaY + location_Y);
				else if (downAnime != null)
					downAnime.onDraw(draw, relaX + location_X, relaY + location_Y);
				else if (normalDraw != null)
					draw.drawImage(normalDraw, relaX + location_X, relaY + location_Y);		
				else if (normalAnime != null)
					normalAnime.onDraw(draw, relaX + location_X, relaY + location_Y);
				break;
			case DISABLE:
				if (disableDraw != null)
					draw.drawImage(disableDraw, relaX + location_X, relaY + location_Y);
				else if (disableAnime != null)
					disableAnime.onDraw(draw, relaX + location_X, relaY + location_Y);
				else
					if (normalDraw != null)
						draw.drawImage(normalDraw, relaX + location_X, relaY + location_Y);
					else if (normalAnime != null)
						normalAnime.onDraw(draw, relaX + location_X, relaY + location_Y);
				break;
			default:
				break;
		}

		if (this.text != null)
		{
			int tColor = draw.getForeColor();
			if (tColor != textColor)
				draw.setForeColor(textColor);

			draw.drawString(this.text, relaX + location_X + titleLocationX, relaY + location_Y + titleLocationY);

			if (tColor != textColor)
				draw.setForeColor(tColor);
		}

		if (tAlpha != alpha)
			draw.setDrawAlpha(tAlpha);
	}
	
	@Override
	public void destroy()
	{
		if (normalDraw != null) {
			normalDraw.destroy();
			normalDraw = null;
		}
		if (focusDraw != null) {
			focusDraw.destroy();
			focusDraw = null;
		}
		if (downDraw != null) {
			downDraw.destroy();
			downDraw = null;
		}
		if (disableDraw != null) {
			disableDraw.destroy();
			disableDraw = null;
		}
		if (normalAnime != null) {
			normalAnime.destroy();
			normalAnime = null;
		}
		if (focusAnime != null) {
			focusAnime.destroy();
			focusAnime = null;
		}
		if (downAnime != null) {
			downAnime.destroy();
			downAnime = null;
		}
		if (disableAnime != null) {
			disableAnime.destroy();
			disableAnime = null;
		}
		Util.logInfo("Button being destroyed.");
	}
	@Override
	public void storeValue(StoreMgr storeMgr)
	{
		super.storeValue(storeMgr);
		if(normalAnime!=null)
		{
			storeMgr.setValue("BUTTON."+ID+".NORMALANIME.ID", normalAnime.getID());
			normalAnime.storeValue(storeMgr);
		}
		if(focusAnime!=null)
		{
			storeMgr.setValue("BUTTON."+ID+".FOCUSANIME.ID", focusAnime.getID());
			focusAnime.storeValue(storeMgr);
		}
		if(downAnime!=null)
		{
			storeMgr.setValue("BUTTON."+ID+".DOWNANIME.ID", downAnime.getID());
			downAnime.storeValue(storeMgr);
		}
		if(disableAnime!=null)
		{
			storeMgr.setValue("BUTTON."+ID+".DISABLEANIME.ID", disableAnime.getID());
			disableAnime.storeValue(storeMgr);
		}
		if(normalDraw!=null)
		{
			normalDraw.storeValue("BUTTON."+ID+".NORMALDRAW", storeMgr);
		}
		if(focusDraw!=null)
		{
			focusDraw.storeValue("BUTTON."+ID+".FOCUSDRAW", storeMgr);
		}
		if(downDraw!=null)
		{
			downDraw.storeValue("BUTTON."+ID+".DOWNDRAW", storeMgr);
		}
		if(disableDraw!=null)
		{
			disableDraw.storeValue("BUTTON."+ID+".DISABLEDRAW", storeMgr);
		}
		storeMgr.setValue("BUTTON."+ID+".TEXT", text);
		storeMgr.setValue("BUTTON."+ID+".TEXTCOLOR", textColor);
	}

	public Button(long id, StoreMgr storeMgr)
	{
		super(id, storeMgr);
		text = storeMgr.getValue("BUTTON."+ID+".TEXT");
		textColor=Integer.parseInt(storeMgr.getValue("BUTTON."+ID+".TEXTCOLOR"));
		normalDraw=DrawableHelper.restoreValue("BUTTON."+ID+".NORMALDRAW", storeMgr);
		if(normalDraw==null)
		{
			normalAnime = new FrameAnimation(Long.parseLong(storeMgr.getValue("BUTTON."+ID+".NORMALANIME.ID")), storeMgr);
		}
		focusDraw=DrawableHelper.restoreValue("BUTTON."+ID+".FOCUSDRAW", storeMgr);
		if(focusDraw==null)
		{
			focusAnime = new FrameAnimation(Long.parseLong(storeMgr.getValue("BUTTON."+ID+".FOCUSANIME.ID")), storeMgr);
		}
		downDraw=DrawableHelper.restoreValue("BUTTON."+ID+".DOWNDRAW", storeMgr);
		if(downDraw==null)
		{
			downAnime = new FrameAnimation(Long.parseLong(storeMgr.getValue("BUTTON."+ID+".DOWNANIME.ID")), storeMgr);
		}
		disableDraw=DrawableHelper.restoreValue("BUTTON."+ID+".DISABLEDRAW", storeMgr);
		if(disableDraw==null)
		{
			disableAnime = new FrameAnimation(Long.parseLong(storeMgr.getValue("BUTTON."+ID+".DISABLEANIME.ID")), storeMgr);
		}
	}
	
}
