package wafly.framework.storyTK.control;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Properties;

import wafly.framework.storyTK.basic.DrawableHelper;
import wafly.framework.storyTK.basic.StoreMgr;
import wafly.framework.storyTK.basic.Util;
import wafly.framework.storyTK.tmpl.Control;

/**
 * 帧动画<br>
 * animation with frame by frame<br>
 * <br>
 * 如果通过属性文件来初始化一个动画,则属性文件的内容通常是这样的:<br>
 * <br>
 * <strong>#The first line should be blank as this properties file should be UTF-8#</strong><br>
 * <strong>FRAME_PATH=/sdcard/StoryToolKit/1.png[,/sdcard/StoryToolKit/2.png]</strong><br>
 * <strong>FRAME_COUNT=5</strong><br>
 * <strong>FRAME_DELAY=3</strong><br>
 * <strong>LOCATION=18,33</strong>(可选)<br>
 * <strong>END=HOLD/HIDE/LOOP</strong>(可选)<br>
 * <br>
 * 其中"#"开头的行表示注释. 文件中共有三个属性: FRAME_PATH,FRAME_COUNT和FRAME_DELAY<br>
 * FRAME_PATH所指向的值表示动画图像所在的路径,文件内容是按<strong>横向</strong>排列的动画帧序列。
 * 如果动画包含帧数太多，可以将其拆分为多个图片文件，每个图片包含的帧数一致，并在FRAME_COUNT参数中指明每个图片具体包含多少帧；
 * 多个图片文件的情况下，最后一组图片的帧数可以少于FRAME_COUNT指定的帧幅，但每一帧的宽度要和前面的一致，也就是最后一组的图片总长度可以小于前面的
 * <br>
 * FRAME_COUNT所指向的值表示一个动画图像中包含了多少帧 (这个值可以留空,默认是1)<br>
 * FRAME_DELAY所指向的值表示每一帧幅的动画画面会在游戏中停留多少帧 (这个值可以留空,默认是1)<br>
 * LOCATION指的是动画出现位置，可以不设置，设置了也可以通过代码再次更改<br>
 * END指的是动画完成后的动作，可以不设置，设置了也可以通过代码再次更改<br>
 * <br>
 * 需要注意的是<br>
 * 1. 参数名称均为大写<br>
 * 2. 文件编码格式建议是UTF-8<br>
 * 3. 文件第一行不要有内容(注释或留空)<br>
 * <br>
 * 默认动作：无
 * 
 * @author Wafly
 */
public class FrameAnimation extends Control
{
	private EndState endState;
	private int currentFrameIndex = 0, frameCount = 1, delay = 1, currentDelay=0, currentFrameArrayIndex=0;
//	private String animePath=null, fileName=null;
//	private Bitmap[] imageArray = null;
	private DrawableHelper[] imageArrayDraw = null;
	private String[] framePaths = null;

	/**
	 * 通过一个动画属性文件来初始化一个动画
	 */
	public FrameAnimation(String profilePath)
	{
		this.endState = EndState.HOLD;
		Properties prop = new Properties();
//		DrawableHelper oriImage = null;
		try {
			InputStream is = Util.openFile(profilePath);
			prop.load(is);
			is.close();
			framePaths = prop.getProperty("FRAME_PATH").split(",");
//			oriImage = new DrawableHelper(framePaths[0]);
			try {
				frameCount = Integer.parseInt(prop.getProperty("FRAME_COUNT").trim());
				if (frameCount < 1)
					frameCount = 1;
			} catch (Exception e) {
				frameCount = 1;
			}
			try {
				delay = Integer.parseInt(prop.getProperty("FRAME_DELAY").trim());
				if (delay < 1)
					delay = 1;
			} catch (Exception e) {
				delay = 1;
			}
			try {
				String sl = prop.getProperty("LOCATION");
				if(sl!=null){
					String[] locals = sl.split(",");
					location_X=Integer.parseInt(locals[0].trim());
					location_Y=Integer.parseInt(locals[1].trim());
				}
			} catch (Exception e) { }
			try {
				String strEnd = prop.getProperty("END");
				if(strEnd!=null){
					endState=EndState.valueOf(strEnd.trim());
				}
			} catch (Exception e) { }
			this.imageArrayDraw = DrawableHelper.getDrawableArray(framePaths[0], frameCount);
			width = this.imageArrayDraw[0].getWidth();
			height = this.imageArrayDraw[0].getHeight();
		} catch (Exception e) {
			Util.logError("Fail to load the animation profile.", e);
		} finally {
			if(prop!=null)
				prop.clear();
		}
	}
	
	@Override
	public FrameAnimation clone() {
		FrameAnimation fa = new FrameAnimation(imageArrayDraw, delay);
		fa.setEndState(endState);
		return fa;
	}

	/** 获取指定序号的帧图像，如果非法的序号则返回null */
	public DrawableHelper getFrame(int frameIndex){
		if(frameIndex<0 || frameIndex>=imageArrayDraw.length)
			return null;
		return imageArrayDraw[frameIndex];
	}

	private void fetchNextFrameArray(){
		DrawableHelper oriImage = new DrawableHelper(framePaths[currentFrameArrayIndex]);
		for (int i = 0; i < frameCount; i++) {
			this.imageArrayDraw[i].destroy();
			this.imageArrayDraw[i] = oriImage.getPart(width * i, 0, width, height);
		}
		oriImage.destroy();
		oriImage = null;
	}
	
	public FrameAnimation(DrawableHelper arrayImageDraw, int frameCount, int frameDelay) {
		framePaths = new String[0];
		this.endState = EndState.LOOP;
		try {
			if (arrayImageDraw == null)
				arrayImageDraw = new DrawableHelper(1, 1);
			if (frameCount < 1)
				this.frameCount = 1;
			else
				this.frameCount = frameCount;
			if (delay < 1)
				delay = 1;
			else
				delay = frameDelay;
			this.imageArrayDraw = new DrawableHelper[frameCount];
			width = arrayImageDraw.getWidth() / frameCount;
			height = arrayImageDraw.getHeight();
			for (int i = 0; i < this.imageArrayDraw.length; i++)
				this.imageArrayDraw[i] = arrayImageDraw.getPart(width * i, 0, width, height);
		} catch (Exception e) {
			Util.logError("Fail to load the animation profile.", e);
		} finally {
			arrayImageDraw.destroy();
			arrayImageDraw = null;
		}

	}

	/**
	 * 通过一组图片来初始化一个动画
	 * @param imageArray 动画帧序列
	 * @param frameDelay 每一帧幅的动画画面会在游戏中停留多少帧, 不得小于1
	 */
	public FrameAnimation(DrawableHelper[] imageArrayDraw, int frameDelay)
	{
		if (imageArrayDraw == null || imageArrayDraw.length == 0)
		{
			this.imageArrayDraw = new DrawableHelper[]
			{ new DrawableHelper(1, 1) };
		}
		else
		{
			this.imageArrayDraw = imageArrayDraw;
		}
		width = this.imageArrayDraw[0].getWidth();
		height = this.imageArrayDraw[0].getHeight();
		this.delay = frameDelay < 1 ? 1 : frameDelay;
	}
	
	public FrameAnimation(long id, StoreMgr storeMgr)
	{
		super(id, storeMgr);
		endState=EndState.valueOf(storeMgr.getValue("FRAMEANIMATION."+ID+".ENDSTATE"));
		currentFrameIndex=Integer.parseInt(storeMgr.getValue("FRAMEANIMATION."+ID+".CURRENTFRAMEINDEX"));
		frameCount=Integer.parseInt(storeMgr.getValue("FRAMEANIMATION."+ID+".FRAMECOUNT"));
		delay=Integer.parseInt(storeMgr.getValue("FRAMEANIMATION."+ID+".DELAY"));
		currentDelay=Integer.parseInt(storeMgr.getValue("FRAMEANIMATION."+ID+".CURRENTDELAY"));
		ArrayList<DrawableHelper> imgArray = new ArrayList<DrawableHelper>();
		for(int i=0;;i++)
		{
			DrawableHelper arrayImg = DrawableHelper.restoreValue("FRAMEANIMATION."+ID+".IMAGEARRAY."+i, storeMgr);
			if(arrayImg==null)
				break;
			imgArray.add(arrayImg);
		}
		imageArrayDraw = new DrawableHelper[imgArray.size()];
		imgArray.toArray(imageArrayDraw);
		imgArray=null;
	}

	/**
	 * 设置每一帧幅的动画画面会在游戏中停留多少帧, 不得小于1
	 */
	public void setDelay(int delay) { this.delay=delay<1?1:delay; }
	
	/**
	 * the state for end of animation
	 */
	public enum EndState { HIDE, HOLD, LOOP };
	
	/**
	 * set the state for the end of the Animation
	 * @param endState end state
	 */
	public void setEndState(EndState endState) { this.endState = endState; }
	
	/**
	 * get the current index of the frame of animation<br>
	 * when animation start, the current index is 0<br>
	 * when animation complete, the index is -1 as EndState is HIDE, or is total animation frame count -1 as EndState id HOLD
	 * @return current index if the frame of animation
	 */
	public int getCurrentFrameIndex() { return currentFrameIndex; }
	
	/**
	 * get total frame count of animation
	 * @return total frame count
	 */
	public int getFrameCount() { return frameCount; }
	
	/**
	 * reset the animation to frist frame
	 */
	public void reset() { currentFrameIndex = 0; }
	
	/**
	 * get the end state of the animation
	 * @return end state
	 */
	public EndState getEndState() { return endState; }
	
	@Override
	public void destroy()
	{
		for (int i = 0; i < imageArrayDraw.length; i++)
			imageArrayDraw[i].destroy();
		Util.logInfo("FrameAnimation being destroyed.");
	}

	@Override
	public void onDraw(DrawableHelper draw)
	{
		if (currentFrameIndex < 0 || destoryed)
			return;
		if (currentFrameIndex >= frameCount) {
			if(framePaths.length>1){
				currentFrameArrayIndex++;
				if(currentFrameArrayIndex>=framePaths.length){
					currentFrameArrayIndex=0;
					if (endState == EndState.HIDE) {
						fetchNextFrameArray();
						currentFrameIndex = -1;
						return;
					} else if (endState == EndState.HOLD) {
						currentFrameIndex = frameCount - 1;
					} else if (endState == EndState.LOOP) {
						fetchNextFrameArray();
						currentFrameIndex = 0;
					}
				}else{
					fetchNextFrameArray();
					currentFrameIndex = 0;
				}
			}else{
				if (endState == EndState.HIDE) {
					currentFrameIndex = -1;
					return;
				} else if (endState == EndState.HOLD) {
					currentFrameIndex = frameCount - 1;
				} else if (endState == EndState.LOOP) {
					currentFrameIndex = 0;
				}
			}
		}
		int tAlpha = draw.getDrawAlpha();
		if (tAlpha != alpha)
			draw.setDrawAlpha(alpha);
		if(imageArrayDraw!=null && imageArrayDraw.length>currentFrameIndex)
			draw.drawImage(imageArrayDraw[currentFrameIndex], relaX + location_X, relaY + location_Y);
		if (tAlpha != alpha)
			draw.setDrawAlpha(tAlpha);
		if (++currentDelay >= delay)
		{
			currentFrameIndex++;
			currentDelay = 0;
		}
	}
	
	public boolean isPlaying(){
		if(endState!=EndState.HOLD && currentFrameIndex>-1)
			return true;
		if(endState==EndState.HOLD && currentFrameIndex<frameCount-1)
			return true;
		return false;
	}
	
	public boolean isHide(){
		return (endState==EndState.HIDE && currentFrameIndex==-1);
	}

	/**
	 * 在指定位置绘制动画
	 * @param draw
	 * @param x
	 * @param y
	 */
	public void onDraw(DrawableHelper draw, int x, int y)
	{
		if (currentFrameIndex < 0 || destoryed || !visible)
			return;
		if (currentFrameIndex >= frameCount) {
			if(framePaths.length>1){
				currentFrameArrayIndex++;
				if(currentFrameArrayIndex>=framePaths.length){
					currentFrameArrayIndex=0;
					if (endState == EndState.HIDE) {
						fetchNextFrameArray();
						currentFrameIndex = -1;
						return;
					} else if (endState == EndState.HOLD) {
						currentFrameIndex = frameCount - 1;
					} else if (endState == EndState.LOOP) {
						fetchNextFrameArray();
						currentFrameIndex = 0;
					}
				}else{
					fetchNextFrameArray();
					currentFrameIndex = 0;
				}
			}else{
				if (endState == EndState.HIDE) {
					currentFrameIndex = -1;
					return;
				} else if (endState == EndState.HOLD) {
					currentFrameIndex = frameCount - 1;
				} else if (endState == EndState.LOOP) {
					currentFrameIndex = 0;
				}
			}
		}
		int tAlpha = draw.getDrawAlpha();
		if (tAlpha != alpha)
			draw.setDrawAlpha(alpha);
		draw.drawImage(imageArrayDraw[currentFrameIndex], x, y);
		if (tAlpha != alpha)
			draw.setDrawAlpha(tAlpha);
		if (++currentDelay >= delay)
		{
			currentFrameIndex++;
			currentDelay = 0;
		}
	}

	@Override
	public void storeValue(StoreMgr storeMgr)
	{
		super.storeValue(storeMgr);
		storeMgr.setValue("FRAMEANIMATION."+ID+".ENDSTATE", endState);
		storeMgr.setValue("FRAMEANIMATION."+ID+".CURRENTFRAMEINDEX", currentFrameIndex);
		storeMgr.setValue("FRAMEANIMATION."+ID+".FRAMECOUNT", frameCount);
		storeMgr.setValue("FRAMEANIMATION."+ID+".DELAY", delay);
		storeMgr.setValue("FRAMEANIMATION."+ID+".CURRENTDELAY", currentDelay);
		for(int i=0; i<imageArrayDraw.length; i++)
			imageArrayDraw[i].storeValue("FRAMEANIMATION."+ID+".IMAGEARRAY."+i, storeMgr);
	}

}
