package wafly.framework.storyTK.control;

import java.util.ArrayList;

import wafly.framework.storyTK.basic.DrawableHelper;
import wafly.framework.storyTK.basic.StoreMgr;
import wafly.framework.storyTK.basic.Util;
import wafly.framework.storyTK.tmpl.Control;

/**
 * 用于显示一段文字的容器<br>
 * 默认动作：无
 * @author Wafly
 *
 */
public class Label extends Control
{

	private String text = null;
	private ArrayList<String> lineText = new ArrayList<String>();
	private boolean multipleLine = false;
	private int textColor=0;
	private int textSize=Util.getGlobalFontSize();
	
	/**
	 * init a label to show specified text to canvas
	 * @param text
	 */
	public Label(String text)
	{
		this.text = text == null ? "" : text.trim();
		textColor = Util.getInverseColorValue(Util.getBackgroundColor());
		width = Util.calStringPixLength(this.text);
		height = textSize;
	}
	
	public Label()
	{
		this.text = "";
		textColor = Util.getInverseColorValue(Util.getBackgroundColor());
		width = 1;
		height = textSize;
	}
	
	public Label(long id, StoreMgr storeMgr)
	{
		super(id, storeMgr);
		text = storeMgr.getValue("LABEL."+ID+".TEXT");
		multipleLine = Boolean.parseBoolean(storeMgr.getValue("LABEL."+ID+".MULTIPLELINE"));
		textColor = Integer.parseInt(storeMgr.getValue("LABEL."+ID+".TEXTCOLOR"));
	}
	
	private void refreshText()
	{
		StringBuffer tmpStr = new StringBuffer("");
		if (multipleLine)
		{
			lineText.clear();
			int minWidth = Util.calStringPixLength("abc");
			if (width < minWidth)
				width = minWidth;
			for (int i = 0; i < text.length(); i++)
			{
				tmpStr.append(text.charAt(i));
				if (Util.calStringPixLength(tmpStr.toString()) > this.width)
				{
					tmpStr.delete(tmpStr.length() - 1, tmpStr.length());
					lineText.add(tmpStr.toString());
					tmpStr.delete(0, tmpStr.length());
					i--;
				}
				else if (text.charAt(i) == '/' && i < text.length() - 1 && text.charAt(i + 1) == 'n')
				{
					tmpStr.delete(tmpStr.length() - 1, tmpStr.length());
					lineText.add(tmpStr.toString());
					tmpStr.delete(0, tmpStr.length());
					i++;
				}
				if ((lineText.size() + 1) * textSize > height)
					break;
			}
			lineText.add(tmpStr.toString());
		}
		else
		{
			for (int i = 0; i < this.text.length(); i++)
			{
				tmpStr.append(this.text.charAt(i));
				if (Util.calStringPixLength(tmpStr.toString()) > width && tmpStr.length()>1)
				{
					tmpStr.delete(tmpStr.length() - 1, tmpStr.length());
					break;
				}
			}
			this.text = tmpStr.toString();
		}
	}
	
	/**
	 * to set the label text's font color
	 * @param color font color
	 */
	public void setTextColor(int color) { textColor=color; }
	
	/**
	 * @return the label text's font color
	 */
	public int getTextColor() { return textColor; }
	
	/**
	 * set the label's text
	 * @param text
	 */
	public void setText(String text)
	{
		this.text = text == null ? "" : text;
//		if(multipleLine)
			refreshText();
	}
	
	/**
	 * @return the label's text
	 */
	public String getText() { return this.text; }
	
	/**
	 * set the label as multiple line mode or single line mode
	 * @param multipleLine set true as multiple line mode, or set false as single line mode
	 */
	public void setMultipleLine(boolean multipleLine)
	{
		this.multipleLine = multipleLine;
//		if(multipleLine)
			refreshText();
	}
	
	/**
	 * 设置Label控件宽度<br>
	 * set the width of label, only effective on multiple line mode
	 */
	public void setSize(int width, int height)
	{
		this.width = width;
		this.height = height;
		refreshText();
	}
	
	@Override
	public void onDraw(DrawableHelper draw)
	{
		if(!visible)
			return;
		int tAlpha = draw.getDrawAlpha();
		if (tAlpha != alpha)
			draw.setDrawAlpha(alpha);

		int tColor = draw.getForeColor();
		if (tColor != textColor)
			draw.setForeColor(textColor);

		if (multipleLine) {
			for (int i = 0, lineTop = 0; i < lineText.size(); i++, lineTop += textSize)
				draw.drawString(lineText.get(i), relaX + location_X,
						relaY + location_Y + lineTop);
		} else {
			draw.drawString(text, relaX + location_X, relaY + location_Y);
		}

		if (tAlpha != alpha)
			draw.setDrawAlpha(tAlpha);

		if (tColor != textColor)
			draw.setForeColor(tColor);
	}

	@Override
	public void storeValue(StoreMgr storeMgr)
	{
		super.storeValue(storeMgr);
		storeMgr.setValue("LABEL."+ID+".TEXT", text);
		storeMgr.setValue("LABEL."+ID+".MULTIPLELINE", multipleLine);
		storeMgr.setValue("LABEL."+ID+".TEXTCOLOR", textColor);
	}

	@Override
	public void destroy() { }
	
}
