package wafly.framework.storyTK.control;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Properties;
import wafly.framework.storyTK.Arena;
import wafly.framework.storyTK.basic.DrawableHelper;
import wafly.framework.storyTK.basic.StoreMgr;
import wafly.framework.storyTK.basic.Util;
import wafly.framework.storyTK.component.TrackAnimation;
import wafly.framework.storyTK.listener.CollideListener;
import wafly.framework.storyTK.listener.HitListener;
import wafly.framework.storyTK.scene.war.WarScene;
import wafly.framework.storyTK.tmpl.Control;
import wafly.framework.storyTK.tmpl.IaiAction;
import wafly.framework.storyTK.tmpl.IaiJudge;

public class Sprite extends Control {
	private FrameAnimation[] spriteFigures = null;
	private int[] directions = null;
	private int currentDirectionIdx = 0; // 不是具体方向值，而是已存储的方向数组的值的索引
	private FrameAnimation[] staticFigure = null; // 精灵静立时的形象图片，未设置时默认使用行走图第一帧
	private ArrayList<int[]> moveCmdList = new ArrayList<int[]>();
	private Thread movingThread = null;
	private boolean isStopped = true;
	private int standAreaLeft=0, standAreaTop=0; // 碰撞检测区域
	private int speed = 0,  // 移动速度，一个大概值，没有参考意义
			frameMultiple = 0; // 每一精灵行走帧延迟多少游戏帧
	private FrameAnimation performFrameAnimation = null;
	private TrackAnimation performTrackAnimation = null;
	private ArrayList<FrameAnimation> showupFrameAnimationList = new ArrayList<FrameAnimation>();
	private ArrayList<TrackAnimation> showupTrackAnimationList = new ArrayList<TrackAnimation>();
	private WarScene container = new WarScene(0);
	private ArrayList<CollideListener> collideListenerList = new ArrayList<CollideListener>();
	private ArrayList<HitListener> hitListenerList = new ArrayList<HitListener>();
	private String ID = null;
	private HashMap<String, String> paramMap = new HashMap<String, String>();

	/** 精灵所有支持的前进方向 */
	public class Directions {
		/** 正面（朝南） */
		public static final int DOWN = 1;
		/** 左侧（朝左） */
		public static final int LEFT = 2;
		/** 右侧（朝右） */
		public static final int RIGHT = 3;
		/** 背面（朝北） */
		public static final int UP = 4;
		/** 朝向西南（面向左下） */
		public static final int DOWN_LEFT = 5;
		/** 朝向东南（面向右下） */
		public static final int DOWN_RIGHT = 6;
		/** 朝向西北（面向左上） */
		public static final int UP_LEFT = 7;
		/** 朝向东北（面向右上） */
		public static final int UP_RIGHT = 8;
	}

	/**
	 * 初始化一个精灵对象，参数directions的个数以及frameCount的数值决定了image对象中包含了多少个单位
	 * 
	 * @param image
	 *            精灵行走图，图中必须是每一行一个方向
	 * @param directions
	 *            精灵行走图中包含了哪些方向（方向数组排列按行走图中从上到下的顺序）<br>
	 *            请传入Directions类的属性作为参数
	 * @param frameCount
	 *            每一个方向包含多少幅帧
	 */
	public Sprite(DrawableHelper image, int[] directions, int frameCount) {
		if (directions == null || directions.length == 0 || frameCount < 1) {
			Util.logError("Invalide parameter.", null);
		} else {
			for (int i = 0; i < directions.length; i++) {
				if (directions[i] < 1 || directions[i] > 8)
					Util.logError("Invalide parameter.", null);
			}
			frameMultiple = Util.getMaxFPS() / frameCount;
			width = image.getWidth() / frameCount;
			height = image.getHeight() / directions.length;
			if (width == 0 || height == 0) {
				Util.logError("Invalide parameter.", null);
			} else {
				this.directions = directions;
				spriteFigures = new FrameAnimation[directions.length];
				staticFigure = new FrameAnimation[directions.length];
				for (int i = 0; i < directions.length; i++) {
					DrawableHelper directionFrames = image.getPart(0, i * height, image.getWidth(), height);
					spriteFigures[i] = new FrameAnimation(directionFrames, frameCount, frameMultiple);
					staticFigure[i] = new FrameAnimation(directionFrames.getPart(0, 0, width, height), 1, frameMultiple);
				}
				image.destroy();
				image = null;
			}
		}
	}

	/**
	 * 初始化一个精灵对象
	 * @param figures 行走动画，此数组长度及顺序与directions参数一致
	 * @param directions 方向数组，决定了此精灵能像哪些方向移动；具体值请参考Directions类
	 * @param stands 静止时的精灵表现，此数组长度及顺序与directions参数一致
	 */
	public Sprite(FrameAnimation[] figures, int[] directions, FrameAnimation[] stands) {
		spriteFigures = new FrameAnimation[figures.length];
		for (int i = 0; i < spriteFigures.length; i++) {
			spriteFigures[i] = figures[i];
		}
//		this.directions = new int[directions.length];
//		for (int i = 0; i < this.directions.length; i++)
//			this.directions[i] = directions[i];
		this.directions=directions;
		if(stands!=null){
			staticFigure = new FrameAnimation[stands.length];
			for (int i = 0; i < staticFigure.length; i++)
				staticFigure[i] = stands[i];
		}else{
			staticFigure = new FrameAnimation[directions.length];
			for (int i = 0; i < directions.length; i++)
				staticFigure[i] = new FrameAnimation(spriteFigures[i].getFrame(0), 1, frameMultiple);
		}
		width = spriteFigures[0].getWidth();
		height = spriteFigures[0].getHeight();
	}

	/** 通过已有的精灵复制一个实例 */
	@Override
	public Sprite clone() {
		return new Sprite(spriteFigures, directions, staticFigure);
	}

	/**
	 * 设置当精灵静止不动时的形象
	 * @param figure 静止动画
	 * @param directions 形象所属方向
	 */
	public void setStaticFigure(FrameAnimation figure, int direction) {
		int d = verifyDirection(direction);
		if (d == -1) {
			Util.logError("Direction [" + direction + "] not defined.", null);
			return;
		}
		figure.setEndState(FrameAnimation.EndState.LOOP);
		this.staticFigure[d].destroy();
		this.staticFigure[d] = figure;
	}
	
	/**
	 * 设置精灵静止不动时的形象（全方向）
	 * @param figures 总数要与设定的方向数机顺序一致
	 */
	public void setStaticFigure(FrameAnimation[] figures){
		if(figures.length!=directions.length){
			Util.logError("figures count not match the dtrections count.", null);
			return;
		}
		for(int i=0; i<figures.length; i++){
			figures[i].setEndState(FrameAnimation.EndState.LOOP);
			this.staticFigure[i].destroy();
			this.staticFigure[i] = figures[i];
		}
	}
	
	/**在精灵所在方位增加显示一段动画*/
	public void showupAnimation(FrameAnimation show){
		if(show==null)
			return;
		this.showupFrameAnimationList.add(show);
	}

	/**在精灵所在方位增加显示一段动画*/
	public void showupAnimation(TrackAnimation show){
		if(show==null)
			return;
		this.showupTrackAnimationList.add(show);
	}
	
	/**将精灵形象切换为一段动画*/
	public void performAnimation(FrameAnimation perform){
		if(perform==null)
			return;
		if(this.performFrameAnimation==perform){
			this.performFrameAnimation.reset();
		}else if(this.performFrameAnimation!=null){
			this.performFrameAnimation.destroy();
			this.performFrameAnimation=null;
			this.performFrameAnimation = perform;
		}
	}

	/**将精灵形象切换为一段动画*/
	public void performAnimation(TrackAnimation perform){
		if(perform==null)
			return;
		if(this.performTrackAnimation==perform){
			this.performTrackAnimation.reset();
		}else if(this.performTrackAnimation!=null){
			this.performTrackAnimation.destroy();
			this.performTrackAnimation=null;
			this.performTrackAnimation = perform;
		}
	}
	
	public void setRoleID(String id){
		this.ID = id;
	}
	
	public String getRoleID(){
		return ID;
	}
	
	public void setParam(String key, Object value){
		this.paramMap.put(key, value.toString());
	}
	
	public String getParam(String key){
		return paramMap.get(key);
	}
	
	/**停止精灵移动*/
	public void stopMoving(){
		isStopped=true;
		moveCmdList.clear();
	}

	/**暂停精灵移动，可以用move()方法恢复移动*/
	public void pauseMoving(){
		isStopped=true;
	}
	
	public boolean isMoving(){ return !isStopped; }
	
	/**
	 * 指定移动命令，使精灵移动
	 * @param moveCmd 长度为2的数组，第一个表示方向，第二个表示行走距离
	 * @param isImmediately 是否立即执行移动命令，是的话则中断当前现有的移动命令，立刻执行新的移动命令；
	 * 不是的话，会在所有已有的移动命令执行完毕后才执行
	 * @see #move()
	 */
	public void move(int[] moveCmd, boolean isImmediately){
		if(isImmediately)
			stopMoving();
		if(verifyDirection(moveCmd[0])==-1){
			Util.logError("Invalid move direction ["+moveCmd[0]+"]", null);
			return;
		}
		this.moveCmdList.add(moveCmd);
		move();
	}
	
	/**
	 * 移动精灵，精灵会按照已有的移动命令去移动，如果没有移动命令则不移动；
	 * 放与其他精灵发生碰撞时，则会取消整条移动命令。
	 */
	public synchronized void move(){
		if(!isStopped)
			return;
		try{
			//确保上一次的移动命令已经完成
			Thread.sleep(1000/Util.getMaxFPS());
		}catch(Exception e){}
		isStopped=false;
		movingThread = new Thread(){public void run(){
			moveMethod();
		}};
		movingThread.start();
	}
	
	private void moveMethod(){
		while(!isStopped){
			if(moveCmdList.size()==0){
				isStopped=true;
				break;
			}
			int[] movCmd = moveCmdList.get(0);
			currentDirectionIdx = verifyDirection(movCmd[0]);
			ArrayList<Sprite> colliSPs = null;
			switch (movCmd[0]) {
			case 1:
				location_Y += speed;
				colliSPs = container.getCollideSprites(location_X+standAreaLeft, location_Y+standAreaTop, Sprite.this);
				if(colliSPs.size()>0){
					location_Y -= speed;
					moveCmdList.remove(0);
					for(CollideListener l : collideListenerList)
						l.onCollideTo(colliSPs);
					for(Sprite colliSP : colliSPs){
						for(CollideListener col : colliSP.getCollideListeners())
							col.onBeingCollided(Sprite.this);
					}
				}else if(container.isCollidedByObstacle(location_X+standAreaLeft, location_Y+standAreaTop)){
					location_Y -= speed;
					moveCmdList.remove(0);
				}
				movCmd[1]-=speed;
				container.reOrderZ(Sprite.this);
				if(movCmd[1]<=0)
					moveCmdList.remove(0);
				break;
			case 2:
				location_X -= speed;
				colliSPs = container.getCollideSprites(location_X+standAreaLeft, location_Y+standAreaTop, Sprite.this);
				if(colliSPs.size()>0){
					location_X += speed;
					moveCmdList.remove(0);
					for(CollideListener l : collideListenerList)
						l.onCollideTo(colliSPs);
					for(Sprite colliSP : colliSPs){
						for(CollideListener col : colliSP.getCollideListeners())
							col.onBeingCollided(Sprite.this);
					}
				}else if(container.isCollidedByObstacle(location_X+standAreaLeft, location_Y+standAreaTop)){
					location_X += speed;
					moveCmdList.remove(0);
				}
				movCmd[1]-=speed;
				container.reOrderZ(Sprite.this);
				if(movCmd[1]<=0)
					moveCmdList.remove(0);
				break;
			case 3:
				location_X += speed;
				colliSPs = container.getCollideSprites(location_X+standAreaLeft, location_Y+standAreaTop, Sprite.this);
				if(colliSPs.size()>0){
					location_X -= speed;
					moveCmdList.remove(0);
					for(CollideListener l : collideListenerList)
						l.onCollideTo(colliSPs);
					for(Sprite colliSP : colliSPs){
						for(CollideListener col : colliSP.getCollideListeners())
							col.onBeingCollided(Sprite.this);
					}
				}else if(container.isCollidedByObstacle(location_X+standAreaLeft, location_Y+standAreaTop)){
					location_X -= speed;
					moveCmdList.remove(0);
				}
				movCmd[1]-=speed;
				container.reOrderZ(Sprite.this);
				if(movCmd[1]<=0)
					moveCmdList.remove(0);
				break;
			case 4:
				location_Y -= speed;
				colliSPs = container.getCollideSprites(location_X+standAreaLeft, location_Y+standAreaTop, Sprite.this);
				if(colliSPs.size()>0){
					location_Y += speed;
					moveCmdList.remove(0);
					for(CollideListener l : collideListenerList)
						l.onCollideTo(colliSPs);
					for(Sprite colliSP : colliSPs){
						for(CollideListener col : colliSP.getCollideListeners())
							col.onBeingCollided(Sprite.this);
					}
				}else if(container.isCollidedByObstacle(location_X+standAreaLeft, location_Y+standAreaTop)){
					location_Y += speed;
					moveCmdList.remove(0);
				}
				movCmd[1]-=speed;
				container.reOrderZ(Sprite.this);
				if(movCmd[1]<=0)
					moveCmdList.remove(0);
				break;
			case 5:
				location_X -= speed;
				location_Y += speed;
				colliSPs = container.getCollideSprites(location_X+standAreaLeft, location_Y+standAreaTop, Sprite.this);
				if(colliSPs.size()>0){
					location_X += speed;
					location_Y -= speed;
					moveCmdList.remove(0);
					for(CollideListener l : collideListenerList)
						l.onCollideTo(colliSPs);
					for(Sprite colliSP : colliSPs){
						for(CollideListener col : colliSP.getCollideListeners())
							col.onBeingCollided(Sprite.this);
					}
				}else if(container.isCollidedByObstacle(location_X+standAreaLeft, location_Y+standAreaTop)){
					location_X += speed;
					location_Y -= speed;
					moveCmdList.remove(0);
				}
				movCmd[1]-=speed;
				container.reOrderZ(Sprite.this);
				if(movCmd[1]<=0)
					moveCmdList.remove(0);
				break;
			case 6:
				location_X += speed;
				location_Y += speed;
				colliSPs = container.getCollideSprites(location_X+standAreaLeft, location_Y+standAreaTop, Sprite.this);
				if(colliSPs.size()>0){
					location_X -= speed;
					location_Y -= speed;
					moveCmdList.remove(0);
					for(CollideListener l : collideListenerList)
						l.onCollideTo(colliSPs);
					for(Sprite colliSP : colliSPs){
						for(CollideListener col : colliSP.getCollideListeners())
							col.onBeingCollided(Sprite.this);
					}
				}else if(container.isCollidedByObstacle(location_X+standAreaLeft, location_Y+standAreaTop)){
					location_X -= speed;
					location_Y -= speed;
					moveCmdList.remove(0);
				}
				movCmd[1]-=speed;
				container.reOrderZ(Sprite.this);
				if(movCmd[1]<=0)
					moveCmdList.remove(0);
				break;
			case 7:
				location_X -= speed;
				location_Y -= speed;
				colliSPs = container.getCollideSprites(location_X+standAreaLeft, location_Y+standAreaTop, Sprite.this);
				if(colliSPs.size()>0){
					location_X += speed;
					location_Y += speed;
					moveCmdList.remove(0);
					for(CollideListener l : collideListenerList)
						l.onCollideTo(colliSPs);
					for(Sprite colliSP : colliSPs){
						for(CollideListener col : colliSP.getCollideListeners())
							col.onBeingCollided(Sprite.this);
					}
				}else if(container.isCollidedByObstacle(location_X+standAreaLeft, location_Y+standAreaTop)){
					location_X += speed;
					location_Y += speed;
					moveCmdList.remove(0);
				}
				movCmd[1]-=speed;
				container.reOrderZ(Sprite.this);
				if(movCmd[1]<=0)
					moveCmdList.remove(0);
				break;
			case 8:
				location_X += speed;
				location_Y -= speed;
				colliSPs = container.getCollideSprites(location_X+standAreaLeft, location_Y+standAreaTop, Sprite.this);
				if(colliSPs.size()>0){
					location_X -= speed;
					location_Y += speed;
					moveCmdList.remove(0);
					for(CollideListener l : collideListenerList)
						l.onCollideTo(colliSPs);
					for(Sprite colliSP : colliSPs){
						for(CollideListener col : colliSP.getCollideListeners())
							col.onBeingCollided(Sprite.this);
					}
				}else if(container.isCollidedByObstacle(location_X+standAreaLeft, location_Y+standAreaTop)){
					location_X -= speed;
					location_Y += speed;
					moveCmdList.remove(0);
				}
				movCmd[1]-=speed;
				container.reOrderZ(Sprite.this);
				if(movCmd[1]<=0)
					moveCmdList.remove(0);
				break;
			}
			try{
				Thread.sleep(1000/Util.getMaxFPS());
			}catch(Exception e){}
		}
	}
	
	public void setStandAreaPosition(int x, int y){
		this.standAreaLeft=x;
		this.standAreaTop=y;
	}
	
	public int getStandAreaPositionX(){
		return this.standAreaLeft;
	}
	
	public int getStandAreaPositionY(){
		return this.standAreaTop;
	}
	
	public void addCollideListener(CollideListener l){
		this.collideListenerList.add(l);
	}
	
	public ArrayList<CollideListener> getCollideListeners(){
		return this.collideListenerList;
	}
	
	public void addHitListener(HitListener l){
		this.hitListenerList.add(l);
	}
	
	public ArrayList<HitListener> getHitListeners(){
		return this.hitListenerList;
	}
	
	public void setScene(WarScene scene){
		this.container=scene;
	}
	
	public WarScene getScene(){
		return this.container;
	}
	
	public boolean isInAction(){
		return isStopped &&
				(performFrameAnimation==null || performFrameAnimation.isHide()) &&
				(performTrackAnimation==null || performTrackAnimation.isHide());
	}

	/**
	 * 设置精灵位置
	 * 
	 * @param x
	 *            精灵相对于地图的横坐标
	 * @param y
	 *            精灵相对于地图的纵坐标
	 * @return 是否成功设置了新位置，若放置在障碍物上时返回false
	 */
	@Override
	public void setLocation(int x, int y) {
		location_X = Arena.getScene().getBackgroundLocation_X()+x;
		location_Y = Arena.getScene().getBackgroundLocation_Y()+y;
		container.reOrderZ(this);
	}

	@Override
	public void onDraw(DrawableHelper draw) {
		if (!visible || destoryed)
			return;
		int tAlpha = draw.getDrawAlpha();
		if (tAlpha != alpha)
			draw.setDrawAlpha(alpha);
		if (isStopped) {
			if(performFrameAnimation!=null && !performTrackAnimation.isHide()){
				performFrameAnimation.onDraw(draw, location_X+relaX, location_Y+relaY);
			}else if(performTrackAnimation!=null && !performTrackAnimation.isHide()){
				performTrackAnimation.onDraw(draw);
			}else{
				staticFigure[currentDirectionIdx].onDraw(draw, location_X+relaX, location_Y+relaY);
			}
			for(int i=0; i<showupFrameAnimationList.size(); i++){
				showupFrameAnimationList.get(i).onDraw(draw, location_X+relaX, location_Y+relaY);
				if(showupFrameAnimationList.get(i).getCurrentFrameIndex()==-1){
					FrameAnimation fa = showupFrameAnimationList.remove(i);
					fa.destroy();
					i--;
				}
			}
			for(int i=0; i<showupTrackAnimationList.size(); i++){
				showupTrackAnimationList.get(i).onDraw(draw);
				if(showupTrackAnimationList.get(i).getCurrentFrameIndex()==-1){
					TrackAnimation fa = showupTrackAnimationList.remove(i);
					fa.destroy();
					i--;
				}
			}
		} else {

			if(performFrameAnimation!=null && !performTrackAnimation.isHide()){
				performFrameAnimation.onDraw(draw, location_X+relaX, location_Y+relaY);
			}else if(performTrackAnimation!=null && !performTrackAnimation.isHide()){
				performTrackAnimation.onDraw(draw);
			}else{
				spriteFigures[currentDirectionIdx].onDraw(draw, location_X+relaX, location_Y+relaY);
			}
			for(int i=0; i<showupFrameAnimationList.size(); i++){
				showupFrameAnimationList.get(i).onDraw(draw, location_X+relaX, location_Y+relaY);
				if(showupFrameAnimationList.get(i).getCurrentFrameIndex()==-1){
					FrameAnimation fa = showupFrameAnimationList.remove(i);
					fa.destroy();
					i--;
				}
			}
			for(int i=0; i<showupTrackAnimationList.size(); i++){
				showupTrackAnimationList.get(i).onDraw(draw);
				if(showupTrackAnimationList.get(i).getCurrentFrameIndex()==-1){
					TrackAnimation fa = showupTrackAnimationList.remove(i);
					fa.destroy();
					i--;
				}
			}
		}
		if (tAlpha != alpha)
			draw.setDrawAlpha(tAlpha);

	}

	/**
	 * 销毁一个精灵的资源
	 */
	public void destroy() {
		destoryed = true;
		for(int i=0; i<spriteFigures.length; i++)
			spriteFigures[i].destroy();
		for(int i=0; i<staticFigure.length; i++)
			staticFigure[i].destroy();
		if(performFrameAnimation!=null)
			performFrameAnimation.destroy();
		if(performTrackAnimation!=null)
			performTrackAnimation.destroy();
		for(FrameAnimation showupFrame : showupFrameAnimationList)
			showupFrame.destroy();
		for(TrackAnimation showupTrack : showupTrackAnimationList)
			showupTrack.destroy();
	}

	@Override
	public void storeValue(StoreMgr storeMgr) {}

	/**
	 * 设置精灵前进速度
	 * 
	 * @param speed
	 */
	public void setSpeed(int speed) {
		this.speed = speed<1?1:speed;
	}

	/**
	 * 获取精灵前进速度
	 * 
	 * @return
	 */
	public double getspeed() {
		return this.speed;
	}

	/**
	 * 获取当前行进的方向
	 * 
	 * @return 1.正面（朝南），2.左侧（朝左），3.右侧（朝右），4.背面（朝北），5.朝向西南（面向左下），6.朝向东南（面向右下），
	 *         7.朝向西北（面向左上），8.朝向东北（面向右上）
	 */
	public int getCurrentDirection() {
		return directions[currentDirectionIdx];
	}

	/**
	 * 获取此精灵所有有效的行进方向
	 * 
	 * @return
	 */
	public int[] getValidDirections() {
		return directions;
	}

	/**获取指定的方向的队列索引值，若为无效方向则返回-1*/
	public int verifyDirection(int direction) {
		for (int i = 0; i < directions.length; i++) {
			if (directions[i] == direction)
				return i;
		}
		return -1;
	}

	
	/************************************************************************************************/
	/************************************************************************************************/
	
	
	/**
	 * AI配置文件，通常内容格式是：
	 * 
	 * AI.x=ai判断 [参数1] [参数2] ...
	 * AI.ACT.x=ai行为 [参数1] [参数2] ...
	 * 
	 * x表示int型的序号，必须按顺序，否则遇到不存在的序号则此AI判断停止；序号从1开始
	 * ai命令需是实现wafly.framework.storyTK.tmpl.IaiJudge的类名，或内置类
	 * ai行为需是实现wafly.framework.storyTK.tmpl.IaiAction的类名
	 */
	private Properties aiProp = new Properties();
	private Thread aiExcutingThread = null;
	private boolean isAIing = false;
	private HashMap<String, Object> cachedAIprogram = new HashMap<String, Object>();
	/**为精灵设置一个AI配置文件，但不会立即执行。注意：必须是停止AI行为时才可以设置，AI执行期间不能设置*/
	public void setAI(String aiFile){
		if(isAIing)
			return;
		aiProp.clear();
		try{
			InputStream is = Util.openFile(aiFile);
			aiProp.load(is);
			is.close();
		}catch(Exception e){
			Util.logError("Fail to load the AI file.", null);
			return;
		}
	}
	
	/**如果AI行为已经停止，则立刻开始AI行为*/
	public void executeAI(){
		if(aiExcutingThread!=null)
			return;
		aiExcutingThread = new Thread(){public void run(){
			while(isAIing){
				for(int i=1;;i++){
					String aiJudge = aiProp.getProperty("AI."+i);
					if(aiJudge==null)
						break;
					String[] aiJudgeCmd = aiJudge.trim().replaceAll(" +", " ").split(" ");
					boolean isAction = false;
					try {
						if(cachedAIprogram.get(aiJudgeCmd[0])==null)
							cachedAIprogram.put(aiJudgeCmd[0], Class.forName(aiJudgeCmd[0]).newInstance());
						isAction = ((IaiJudge) cachedAIprogram.get(aiJudgeCmd[0])).onJudge(Sprite.this, aiJudgeCmd);
					} catch (Exception e) {
						Util.logError("Fail to perform AI judge [" + aiJudgeCmd[0] + "]", e);
						continue;
					}
					if(!isAction)
						continue;
					Util.logInfo("Perform AI ["+i+"]");
					String aiAction = aiProp.getProperty("AI.ACT."+i);
					if(aiAction==null)
						break;
					String[] aiActCmd = aiAction.trim().replaceAll(" +", " ").split(" ");
					try {
						if(cachedAIprogram.get(aiActCmd[0])==null)
							cachedAIprogram.put(aiActCmd[0], Class.forName(aiActCmd[0]).newInstance());
						((IaiAction) cachedAIprogram.get(aiActCmd[0])).onAction(Sprite.this, aiActCmd);
						while(isInAction());
						break;
					} catch (Exception e) {
						Util.logError("Fail to perform AI action [" + aiJudgeCmd[0] + "]", e);
						continue;
					}
				}
			}
			aiExcutingThread=null;
		}};
		isAIing=true;
		aiExcutingThread.start();
	}
	
	/**指示AI停止，但不会真的立刻停止，会等待最后一轮执行完毕*/
	public void stopAI(){
		isAIing=false;
	}
}
