package wafly.framework.storyTK.control;

import java.util.ArrayList;
import wafly.framework.storyTK.basic.DrawableHelper;
import wafly.framework.storyTK.basic.Util;
import wafly.framework.storyTK.listener.ActionListener;
import wafly.framework.storyTK.listener.TouchListener;
import wafly.framework.storyTK.tmpl.OperationalControl;

/**
 * 状态指示器，通过不同的图示表示不同的状态值，不同的值即为不同图示在数组中的索引<br>
 * 默认动作：状态值更改后
 * @author Wafly
 *
 */
public class StatusBox extends OperationalControl
{
	private ArrayList<DrawableHelper> statusDraws = new ArrayList<DrawableHelper>();
	private int statusIndex = 0;

	/**
	 * 通过一个指示“通常状态”的图示初始化一个状态指示器，其值为0
	 * @param normalDraw
	 */
	public StatusBox(DrawableHelper normalDraw)
	{
		if(normalDraw==null)
			normalDraw=new DrawableHelper(1, 1);
		statusDraws.add(normalDraw);
		this.width=normalDraw.getWidth();
		this.height=normalDraw.getHeight();
		addTouchListener(new TouchListener() { public void onPerform(int x, int y) {
			if(statusIndex==statusDraws.size()-1)
				statusIndex=0;
			else
				statusIndex++;
			onStatusChanged(statusIndex);
		}});
	}

	/**
	 * 通过一组图示初始化一个状态指示器，第一个图示值为0
	 * @param statusDraws
	 */
	public StatusBox(DrawableHelper... statusDraws)
	{
		if(statusDraws.length==0)
			this.statusDraws.add(new DrawableHelper(1, 1));
		else for(int i=0; i<statusDraws.length; i++)
			this.statusDraws.add(statusDraws[i]);
		this.width=this.statusDraws.get(0).getWidth();
		this.height=this.statusDraws.get(0).getHeight();
		addTouchListener(new TouchListener() { public void onPerform(int x, int y) {
			if(statusIndex==StatusBox.this.statusDraws.size()-1)
				statusIndex=0;
			else
				statusIndex++;
			onStatusChanged(statusIndex);
		}});
	}
	
	/**
	 * 添加一个状态图示
	 * @param statusDraw
	 */
	public void addStatus(DrawableHelper statusDraw)
	{
		if(statusDraw==null)
			statusDraw=new DrawableHelper(1, 1);
		statusDraws.add(statusDraw);
	}
	
	/**
	 * 获取当前状态值
	 * @return
	 */
	public int getValue()
	{
		return statusIndex;
	}
	
	/**
	 * 设置一个状态值，这个值必须在状态图示数组长度范围内
	 * @param statusValue
	 * @return
	 */
	public boolean setValue(int statusValue)
	{
		if(statusValue>-1&&statusValue<statusDraws.size())
		{
			statusIndex=statusValue;
			onStatusChanged(statusIndex);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * 当状态值更改时发生事件
	 * @param statusValue
	 */
	private void onStatusChanged(int statusValue) {
		for(ActionListener listener:actionListenerList){
			listener.onPerform();
		}
	}
	
	@Override
	public void onDraw(DrawableHelper draw)
	{
		if (destoryed || currentStatus==Status.HIDE)
			return;
		int tAlpha = draw.getDrawAlpha();
		if (tAlpha != alpha)
			draw.setDrawAlpha(alpha);
		draw.drawImage(statusDraws.get(statusIndex),
				relaX + location_X, relaY + location_Y);
		if (tAlpha != alpha)
			draw.setDrawAlpha(tAlpha);
	}

	@Override
	public void destroy() {
		for(DrawableHelper draw: statusDraws){
			draw.destroy();
		}
		Util.logInfo("StatusBox being destroyed.");
	}

}
