package wafly.framework.storyTK.control;

import wafly.framework.storyTK.basic.DrawableHelper;
import wafly.framework.storyTK.basic.StoreMgr;
import wafly.framework.storyTK.basic.Util;
import wafly.framework.storyTK.listener.ActionListener;
import wafly.framework.storyTK.listener.DragListener;
import wafly.framework.storyTK.listener.TouchListener;
import wafly.framework.storyTK.tmpl.OperationalControl;

/**
 * 数值栏，通过点击或拖拉可以更改其数值或获得其所示数值<br>
 * 默认动作：数值变动
 * @author Wafly
 */
public class ValueBar extends OperationalControl
{
	private DrawableHelper background=null, foreground=null, sweeper=null;
	private int offset_X=0, offset_Y=0;
	private int maxValue=100, currentValue=0, moveDistance_Start=0, moveDistance_End=0;
	private double perValueDistance=0;
	private DisplayDirection direction=null;
	
	/**
	 * 通过背景图片, 前景图片, 滑动方向来初始化一个ValueBar<br>
	 * init ValueBar
	 */
	public ValueBar(DrawableHelper background, DrawableHelper foreground, DisplayDirection direction)
	{
		if(background==null)
			Util.logError("ValueBar background could not be null.", null);
		this.background = background;
		if(foreground!=null)
			this.foreground = foreground;
		width=background.getWidth();
		height=background.getHeight();
		this.direction=direction;
		if(direction==DisplayDirection.HORIZONTAL)
			perValueDistance=width/100.0;
		else if(direction==DisplayDirection.VERTICAL)
			perValueDistance=height/100.0;
		moveDistance_End=width;
		addTouchListener(new TouchListener() { public void onPerform(int x, int y) {
			if(ValueBar.this.direction==DisplayDirection.HORIZONTAL)
				setValue((int)((x-location_X-moveDistance_Start)/perValueDistance));
			else if(ValueBar.this.direction==DisplayDirection.VERTICAL)
				setValue((int)((y-location_Y-moveDistance_Start)/perValueDistance));
		}});
		addDragListener(new DragListener() { public void onPerform(int x, int y) {
			if(ValueBar.this.direction==DisplayDirection.HORIZONTAL)
				setValue((int)((x-location_X-moveDistance_Start)/perValueDistance));
			else if(ValueBar.this.direction==DisplayDirection.VERTICAL)
				setValue((int)((y-location_Y-moveDistance_Start)/perValueDistance));
		}});
	}

	public ValueBar(long id, StoreMgr storeMgr)
	{
		super(id, storeMgr);
		background = DrawableHelper.restoreValue("VALUEBAR."+ID+".BACKGROUNDDRAW", storeMgr);
		foreground = DrawableHelper.restoreValue("VALUEBAR."+ID+".FOREGROUNDDRAW", storeMgr);
		sweeper = DrawableHelper.restoreValue("VALUEBAR."+ID+".SWEEPERDRAW", storeMgr);
		offset_X = Integer.parseInt(storeMgr.getValue("VALUEBAR."+ID+".OFFSET.X"));
		offset_Y = Integer.parseInt(storeMgr.getValue("VALUEBAR."+ID+".OFFSET.Y"));
		direction = DisplayDirection.valueOf(storeMgr.getValue("VALUEBAR."+ID+".DIRECTION"));
		moveDistance_Start = Integer.parseInt(storeMgr.getValue("VALUEBAR."+ID+".MOVEDISTANCE.START"));
		moveDistance_End = Integer.parseInt(storeMgr.getValue("VALUEBAR."+ID+".MOVEDISTANCE.END"));
		setMaxValue(Integer.parseInt(storeMgr.getValue("VALUEBAR."+ID+".MAXVALUE")));
		currentValue = Integer.parseInt(storeMgr.getValue("VALUEBAR."+ID+".CURRENTVALUE"));
		addTouchListener(new TouchListener() { public void onPerform(int x, int y) {
			if(ValueBar.this.direction==DisplayDirection.HORIZONTAL)
				setValue((int)((x-location_X-moveDistance_Start)/perValueDistance));
			else if(ValueBar.this.direction==DisplayDirection.VERTICAL)
				setValue((int)((y-location_Y-moveDistance_Start)/perValueDistance));
		}});
		addDragListener(new DragListener() { public void onPerform(int x, int y) {
			if(ValueBar.this.direction==DisplayDirection.HORIZONTAL)
				setValue((int)((x-location_X-moveDistance_Start)/perValueDistance));
			else if(ValueBar.this.direction==DisplayDirection.VERTICAL)
				setValue((int)((y-location_Y-moveDistance_Start)/perValueDistance));
		}});
	}
	
	/**
	 * 滑动方向<br>
	 * slide direction on this valuebar
	 */
	public enum DisplayDirection {
		/**
		 * 横向
		 */
		HORIZONTAL,
		/**
		 * 纵向
		 */
		VERTICAL }
	
	/**
	 * 设置滑块图片
	 * @param image 滑块图片
	 * @param offset_X 绘制滑块的横轴将基于背景图原点位移量
	 * @param offset_Y 绘制滑块的纵轴将基于背景图原点位移量
	 */
	public void setSweepImage(DrawableHelper image, int offset_X, int offset_Y)
	{
		if(image!=null)
			this.sweeper = image;
		this.offset_X = offset_X;
		this.offset_Y = offset_Y;
	}
	
	/**
	 * 设置最大值
	 */
	public void setMaxValue(int value)
	{
		this.maxValue=value;
		perValueDistance=(moveDistance_End-moveDistance_Start)/maxValue;
		if(foreground!=null)
		{
			if(direction==DisplayDirection.HORIZONTAL)
				foreground.zoomTo((moveDistance_End-moveDistance_Start)*currentValue/maxValue, foreground.getHeight());
			else if(direction==DisplayDirection.VERTICAL)
				foreground.zoomTo(foreground.getWidth(), (moveDistance_End-moveDistance_Start)*currentValue/maxValue);
		}
	}
	
	/**
	 * 设置当前值
	 */
	public void setValue(int value)
	{
		if(value<0)
			value=0;
		if(value>maxValue)
			value=maxValue;
		currentValue=value;
		onValueChanged(currentValue);
		if(foreground!=null)
		{
			if(direction==DisplayDirection.HORIZONTAL)
				foreground.zoomTo((moveDistance_End-moveDistance_Start)*currentValue/maxValue, foreground.getHeight());
			else if(direction==DisplayDirection.VERTICAL)
				foreground.zoomTo(foreground.getWidth(), (moveDistance_End-moveDistance_Start)*currentValue/maxValue);
		}
	}
	
	/**
	 * 从背景图基点开始计算,设置滑块的移动范围, 起始点总是小于终点
	 * @param start
	 * @param end
	 */
	public void setMoveDistance(int start, int end)
	{
		if(start>end)
		{
			Util.logError("Start should be smaller than end, switched.", null);
			int tmp=start;
			start=end;
			end=tmp;
		}
		moveDistance_Start=start;
		moveDistance_End=end;
		if(foreground!=null)
		{
			if(direction==DisplayDirection.HORIZONTAL)
				foreground.zoomTo((moveDistance_End-moveDistance_Start)*currentValue/maxValue, foreground.getHeight());
			else if(direction==DisplayDirection.VERTICAL)
				foreground.zoomTo(foreground.getWidth(), (moveDistance_End-moveDistance_Start)*currentValue/maxValue);
		}
	}
	
	/**
	 * 当前值发生变化时会发生此事件
	 * @param currentValue 值发生变化后的新值
	 */
	private void onValueChanged(int currentValue) {
		for(ActionListener listener:actionListenerList){
			listener.onPerform();
		}
	}
	
	@Override
	public void onDraw(DrawableHelper draw)
	{
		if(currentStatus==Status.HIDE)
			return;
		draw.drawImage(background, relaX + location_X, relaY + location_Y);
		if(foreground!=null)
			draw.drawImage(foreground, relaX + location_X, relaY + location_Y);
		if(sweeper!=null)
		{
			if(direction==DisplayDirection.HORIZONTAL)
				draw.drawImage(sweeper,
						relaX + (int)(location_X+offset_X+moveDistance_Start+currentValue*perValueDistance),
						relaY + location_Y+offset_Y);
			else if(direction==DisplayDirection.VERTICAL)
				draw.drawImage(sweeper,
						relaX + location_X+offset_X,
						relaY + (int)(location_Y+offset_Y+moveDistance_Start+currentValue*perValueDistance));
		}
	}

	@Override
	public void storeValue(StoreMgr storeMgr)
	{
		super.storeValue(storeMgr);
		background.storeValue("VALUEBAR."+ID+".BACKGROUNDDRAW", storeMgr);
		if(foreground!=null)
			foreground.storeValue("VALUEBAR."+ID+".FOREGROUNDDRAW", storeMgr);
		if(sweeper!=null)
			sweeper.storeValue("VALUEBAR."+ID+".SWEEPERDRAW", storeMgr);
		storeMgr.setValue("VALUEBAR."+ID+".OFFSET.X", offset_X);
		storeMgr.setValue("VALUEBAR."+ID+".OFFSET.Y", offset_Y);
		storeMgr.setValue("VALUEBAR."+ID+".MAXVALUE", maxValue);
		storeMgr.setValue("VALUEBAR."+ID+".CURRENTVALUE", currentValue);
		storeMgr.setValue("VALUEBAR."+ID+".MOVEDISTANCE.START", moveDistance_Start);
		storeMgr.setValue("VALUEBAR."+ID+".MOVEDISTANCE.END", moveDistance_End);
		storeMgr.setValue("VALUEBAR."+ID+".DIRECTION", direction);
	}

	@Override
	public void destroy() {
		if(background!=null)
			background.destroy();
		if(foreground!=null)
			foreground.destroy();
		if(sweeper!=null)
			sweeper.destroy();
		Util.logInfo("ValueBar being destroyed.");
	}

}
