package wafly.framework.storyTK.scene.story;

import java.io.BufferedReader;
import java.io.FileReader;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.HashMap;
import wafly.framework.storyTK.Arena;
import wafly.framework.storyTK.Panel;
import wafly.framework.storyTK.Scene;
import wafly.framework.storyTK.basic.DrawableHelper;
import wafly.framework.storyTK.basic.StoreMgr;
import wafly.framework.storyTK.basic.Util;
import wafly.framework.storyTK.component.TalkBox;
import wafly.framework.storyTK.component.TrackAnimation;
import wafly.framework.storyTK.component.Weather;
import wafly.framework.storyTK.control.Button;
import wafly.framework.storyTK.control.FrameAnimation;
import wafly.framework.storyTK.control.Label;
import wafly.framework.storyTK.control.PictureBox;
import wafly.framework.storyTK.listener.ActionListener;
import wafly.framework.storyTK.tmpl.Component;
import wafly.framework.storyTK.tmpl.Control;

/**
 * 读入的剧本中可以使用以下命令：
 * <br><font color=red>不支持带空格的文件路径！</font><br><br>
 * background [-m 遮罩层图片路径] [-x 背景图片X轴偏移] [-y 背景图片Y轴偏移] 背景图片路径
 * <br><br>
 * backswitchtype 背景切换类型
 * （具体类型参照wafly.framework.storyTK.Scene.BackgroundSwitchType）
 * <br><br>
 * talk [-b 对话框ID] [-f 表情ID] [-n 名字] [-c] 说话内容，内容中不要带空格
 * （ID表示已通过ready命令存储的资源ID，-c表示此次内容是否接着上一次继续显示的开关）
 * <br><br>
 * ready ID talkbox [-l 对话框显示层(整数型)] [-cl 对话内容最大行长度] [-b 对话框背景图片路径] [-bc 对话内容背景色（16进制整数，不含“0x”前缀）] 
 * [-bx 背景图片横坐标] [-by 背景图片纵坐标] [-tc 显示文字颜色（16进制整数，不含“0x”前缀）] [-nx 名字显示横坐标] [-ny 名字显示纵坐标] 
 * [-fx 表情图片横坐标] [-fy 表情图片纵坐标] [-cx 内容文字显示横坐标] [-cy 内容文字显示纵坐标] 
 * [-cp 对话显示完成提示符（Frame动画的属性文件路径）] [-cpx 对话显示完成提示符的横坐标位置] 
 * [-cpy 对话显示完成提示符的纵坐标位置] [-sp 对话内容显示速度]
 * <br><br>
 * ready ID picture [-x 图片显示横坐标位置] [-y 图片显示纵坐标位置] 图片文件路径
 * <br><br>
 * ready ID weather [-l 天气显示图层] [-c 天气浓度] 天气类型
 * （天气类型参考wafly.framework.storyTK.component.Weather.WeatherType）
 * <br><br>
 * ready ID frame [-e 动画播放类型] [-x 动画显示横坐标位置] [-y 动画显示纵坐标位置] 动画属性文件路径
 * （动画播放类型参考wafly.framework.storyTK.control.FrameAnimation.EndState；
 * 属性文件内容格式参考wafly.framework.storyTK.control.FrameAnimation的类注释）
 * <br><br>
 * ready ID text [-x 字符串显示横坐标位置] [-y 字符串显示纵坐标位置] [-c 字符串颜色（16进制整数，不含“0x”前缀）] 字符串内容
 * <br><br>
 * ready ID track [-l 轨迹动画显示图层] [-sx 起点横坐标] [-sy 起点纵坐标] [-ex 终点横坐标] [-ey 终点纵坐标] 
 * [-sw 起始横向缩放比例] [-sh 起始纵向缩放比例] [-ew 最终横向缩放比例] [-eh 最终纵向缩放比例] [-sa 初始透明度] 
 * [-ea 最终透明度] [-e 动画播放类型] 轨迹动画图片文件路径
 * （动画播放类型参考wafly.framework.storyTK.component.TrackAnimation.EndState）
 * <br><br>
 * ready ID select [-p 选项框背景图片路径] [-w 选项对话框宽度，如果设置了背景图片则无效] [-h 选项对话框高度，如果设置了背景图片则无效] [-c 背景色，包括透明度的16进制整数，不含“0x”前缀]
 * <br><br>
 * hide 资源ID（表示已通过ready命令存储的资源）
 * <br><br>
 * show 资源ID（表示已通过ready命令存储的资源）
 * <br><br>
 * destroy 资源ID（表示已通过ready命令存储的资源）
 * <br><br>
 * movie 视频文件路径
 * <br><br>
 * sound 声音文件路径
 * <br><br>
 * music 音乐文件路径
 * <br><br>
 * execute [-p] 类名 [参数值1] [参数值2] ...
 * （类名指的是已实现了wafly.framework.storyTK.tmpl.Ievent接口的类全名；-p是指示等待此事件执行完毕或立刻返回的开关）
 * <br><br>
 * goto [-l 行号] 剧本文件路径（如果没有设置行号，则默认从第一行开始）
 * <br><br>
 * wait [begin/等待的帧数]<br>
 * 等待响应用户操作或等待一段时间后，才继续执行下一行剧本。
 * （等待直到用户点击后才继续下一行剧本；如果后面带了数字（帧数），则等待指定帧数后自动继续下一行剧本；
 * 如果后面附带的是标识符“begin”，则仅仅会开始统计已过期的等待时间（不会立刻停止），直到下次指定了等待帧数后，
 * 将指定的等待帧数减去已过期的等待时间；但如果下次未指定等待时间，则本次计时失效）
 * <br><br>
 * pend &lt;movie/frame/track&gt; [ID]<br>
 * 等待某个指定的资源执行完毕后执行下一行剧本
 * （pend命令后面必须带识别命令之一，即movie或frame或track，如果是frame或track，则需要多一个ID参数）
 * <br><br>
 * select [-b 选项对话框ID] [选项1|对应剧本1] [选项2|对应剧本2] ...
 * 
 * @author Wafly
 */
public class DramaScene extends Scene {
	private Button screenTouch = null;
	private HashMap<String, Button> selectButtons = new HashMap<String, Button>();
	private String[] dramas = null;
	private TalkBox talkBox = null;
	private Panel selectPanel = null;
	private int lineNo = -1;
	private boolean isProcessing = false;
	private HashMap<String, Object> cacheResourceMap = new HashMap<String, Object>();
	private boolean touchHanging = false, isThreadWaiting = false;
	private String dramaFilePath=null;

	public DramaScene()
	{
		screenTouch = new Button(0, 0, Arena.getInstance().getWidth(), Arena.getInstance().getHeight());
		screenTouch.addActionListener(new ActionListener() { public void onPerform() {
				if (getPanelCount() == 0 && !isProcessing){
					processNextLine();
				}else{
					touchHanging=true;
				}
		}});
		addControl(screenTouch);
	}

	/** 执行下一行剧本 */
	public void processNextLine()
	{
		if (dramas == null) {
			Util.logError("No drama loaded.", null);
			return;
		}
		if (lineNo > dramas.length-2) {
			Util.logError("Drama loaded with file end but do nothing.", null);
			return;
		}
		if(isThreadWaiting)
			return;
		isProcessing = true;
		String dramaLine = dramas[++lineNo].trim().replaceAll(" +", " ");
		Util.logInfo("Read line [" + (lineNo+1) + "]");
		try
		{
			if (dramaLine.startsWith("background "))
				BackgroundLine(dramaLine.substring(11).trim().split(" "));
			else if (dramaLine.startsWith("backswitchtype "))
				BackgroundSwitchLine(dramaLine.substring(15));
			else if (dramaLine.startsWith("show "))
				ShowLine(dramaLine.substring(5));
			else if (dramaLine.startsWith("hide "))
				HideLine(dramaLine.substring(5));
			else if (dramaLine.startsWith("talk "))
				TalkLine(dramaLine.substring(5).trim().split(" "));
			else if (dramaLine.startsWith("ready "))
				ReadyLine(dramaLine.substring(6).trim().split(" "));
			else if (dramaLine.startsWith("movie "))
				MovieLine(dramaLine.substring(6));
			else if (dramaLine.startsWith("music "))
				MusicLine(dramaLine.substring(6));
			else if (dramaLine.startsWith("sound "))
				SoundLine(dramaLine.substring(6));
			else if (dramaLine.startsWith("destroy "))
				DestroyLine(dramaLine.substring(8));
			else if (dramaLine.startsWith("execute "))
				EventLine(dramaLine.substring(8).trim().split(" "));
			else if (dramaLine.startsWith("goto "))
				GotoLine(dramaLine.substring(5).trim().split(" "));
			else if (dramaLine.startsWith("wait"))
				WaitLine(dramaLine.trim().split(" "));
			else if (dramaLine.startsWith("pend "))
				PendLine(dramaLine.substring(5).trim().split(" "));
			else if (dramaLine.startsWith("select "))
				SelectLine(dramaLine.substring(7).trim().split(" "));
			else
				Util.logError("Invalid command: " + dramaLine, null);
		} catch (Exception e)
		{
			Util.logError("Command error: " + dramaLine, e);
		}
		onPerformLineDone(dramaLine.trim(), lineNo+1);
		if(isProcessing){
			processNextLine();
		}
	}
	
	/**
	 * 当一行剧本执行完毕后调用此方法
	 * @param lineNo 剧本有效的已执行行数，与日志中显示的一致
	 */
	protected void onPerformLineDone(String lineContent, int lineNo){ }

	public void loadAllSelects(HashMap<String, Button> selects)
	{
		selectButtons.putAll(selects);
	}


	public void setSelectPanel(Panel panel)
	{
		if (selectPanel != null)
			selectPanel.destroy();
		selectPanel = panel;
	}

	/** (重新)读入Drama文件的文本内容，需要UTF-8编码；剧本内容格式参照本类注释 */
	public void loadDrama(String dramaFilePath)
	{
		this.dramaFilePath = Util.verifyFilePath(dramaFilePath);
		if(this.dramaFilePath==null)
		{
			Util.logError("Drama not found.", null);
			return;
		}
		try
		{
			BufferedReader reader = new BufferedReader(new FileReader(this.dramaFilePath));
			ArrayList<String> dramaStrings = new ArrayList<String>();
			String line = null;
			while ((line = reader.readLine()) != null){
				line = line.trim();
				if(line.length()==0 || line.startsWith("#"))
					continue;
				dramaStrings.add(line);
			}
			dramas = new String[dramaStrings.size()];
			dramaStrings.toArray(dramas);
			lineNo = -1;
			dramaStrings.clear();
			reader.close();
		} catch (Exception e)
		{
			Util.logError("Fail to load drama [" + this.dramaFilePath + "].", e);
		}
	}
	
	public String getDramaFilepath(){
		return dramaFilePath;
	}

	/** 设置下一次要执行的剧本的行号，从1开始；注意不是真实剧本文件，而是有效剧本内容（即不包含注释和空行）的行号*/
	public void setDramaLineIndex(int lineNo) { this.lineNo = lineNo - 1; }
	
	public int getLineIndex(){ return lineNo; }

	@Override
	public void storeValue(StoreMgr storeMgr)
	{
		super.storeValue(storeMgr);
		storeMgr.setValue("TALKSCENE.DRAMA.PATH", dramaFilePath);
		storeMgr.setValue("TALKSCENE.DRAMA.LINE", lineNo);
	}

	@Override
	public void restoreValue(StoreMgr storeMgr)
	{
		super.restoreValue(storeMgr);
		dramaFilePath = storeMgr.getValue("TALKSCENE.DRAMA.PATH");
		loadDrama(dramaFilePath);
		setDramaLineIndex(Integer.parseInt(storeMgr.getValue("TALKSCENE.DRAMA.LINE")));
	}

	/*************************************************************************************************************************************/
	/*************************************************************************************************************************************/
	/*************************************************************************************************************************************/

	private void BackgroundLine(String[] args) {
		int x = 0, y = 0;
		DrawableHelper maskLayerDraw = null;
		DrawableHelper newBackground = null;
		for (int i = 0; i < args.length; i++) {
			if (args[i].equals("-m")) {
				i++;
				maskLayerDraw = new DrawableHelper(args[i]);
			} else if (args[i].equals("-x")) {
				i++;
				x = Integer.parseInt(args[i]);
			} else if (args[i].equals("-y")) {
				i++;
				y = Integer.parseInt(args[i]);
			} else if(!args[i].startsWith("-")) {
				newBackground = new DrawableHelper(args[i]);
			}
		}
		setBackground(newBackground, Util.getLeftOnScreen(x), Util.getTopOnScreen(y), maskLayerDraw);
	}

	private void BackgroundSwitchLine(String type) { setBackgroundSwitchType(BackgroundSwitchType.valueOf(type)); }

	private void TalkLine(String[] args)
	{
		if(args.length==0)
			return;
		String faceID = null, name = null;
		boolean isContinue = false;
		removeComponent(talkBox);
		for (int i = 0; i < args.length; i++) {
			if (args[i].equals("-b")) {
				i++;
				talkBox = (TalkBox)cacheResourceMap.get(args[i]);
				if(talkBox==null){
					Util.logError("TalkBox with ID ["+args[i]+"] not ready yet.", null);
					return;
				}
			} else if (args[i].equals("-f")) {
				i++;
				faceID = args[i];
			} else if (args[i].equals("-n")) {
				i++;
				name = args[i];
			} else if (args[i].equals("-c")) {
				isContinue = true;
			}
		}
		if(talkBox==null){
			Util.logError("Talkbox not initialized yet.", null);
			return;
		}
		if(faceID!=null){
			talkBox.setFace(((PictureBox)cacheResourceMap.get(faceID)).getImage());
		}else{
//			if(talkBox.getFace()!=null)
//				talkBox.getFace().destroy();
			talkBox.setFace(null);
		}
		talkBox.setName(name);
		addComponent(talkBox);
		String content = args[args.length-1];
		if(content.equals("null"))
			content=" ";
		talkBox.setContent(content, isContinue, false);
		isProcessing=false;
	}

	private void ReadyLine(String[] args)
	{
		String id = args[0];
		if(id==null) {
			Util.logError("ready command must be contains id", null);
			return;
		}
		if(cacheResourceMap.get(id)!=null){
			Util.logError("Resource with the same ID ["+id+"] already exist.", null);
			return;
		}
		String readyType = args[1];
		if(readyType.equals("talkbox")){
			TalkBox talkBox = new TalkBox();
			int backX=0, backY=0, nameX=0, nameY=0, faceX=0, faceY=0, contX=0, contY=0;
			FrameAnimation ComPrompt = null;
			int cpx = 0, cpy = 0;
			for (int i = 2; i < args.length; i++) {
				if (args[i].equals("-cl")) {
					i++;
					talkBox.setLineLength(Integer.parseInt(args[i]));
				} else if (args[i].equals("-l")) {
					i++;
					talkBox.setLayerIndex(Integer.parseInt(args[i]));
				} else if (args[i].equals("-b")) {
					i++;
					talkBox.setBackground(new DrawableHelper(args[i]));
				} else if (args[i].equals("-bc")) {
					i++;
					talkBox.setBackgroundColor(new BigInteger(args[i],16).intValue());
				} else if (args[i].equals("-bx")) {
					i++;
					backX = Integer.parseInt(args[i]);
				} else if (args[i].equals("-by")) {
					i++;
					backY = Integer.parseInt(args[i]);
				} else if (args[i].equals("-tc")) {
					i++;
					talkBox.setTextColor(new BigInteger(args[i],16).intValue());
				} else if (args[i].equals("-nx")) {
					i++;
					nameX = Integer.parseInt(args[i]);
				} else if (args[i].equals("-ny")) {
					i++;
					nameY = Integer.parseInt(args[i]);
				} else if (args[i].equals("-cx")) {
					i++;
					contX = Integer.parseInt(args[i]);
				} else if (args[i].equals("-cy")) {
					i++;
					contY = Integer.parseInt(args[i]);
				} else if (args[i].equals("-fx")) {
					i++;
					faceX = Integer.parseInt(args[i]);
				} else if (args[i].equals("-fy")) {
					i++;
					faceY = Integer.parseInt(args[i]);
				} else if (args[i].equals("-sp")) {
					i++;
					talkBox.setSpeed(Integer.parseInt(args[i]));
				} else if (args[i].equals("-cp")) {
					i++;
					ComPrompt = new FrameAnimation(args[i]);
				} else if (args[i].equals("-cpx")) {
					i++;
					cpx = Integer.parseInt(args[i]);
				} else if (args[i].equals("-cpy")) {
					i++;
					cpy = Integer.parseInt(args[i]);
				}
			}
			if(ComPrompt!=null){
				ComPrompt.setLocation(cpx, cpy);
				talkBox.setCompletedPrompt(ComPrompt);
			}
			talkBox.setLocation_Background(Util.getLeftOnScreen(backX), Util.getTopOnScreen(backY));
			talkBox.setLocation_Face(Util.getLeftOnScreen(faceX), Util.getTopOnScreen(faceY));
			talkBox.setLocation_Name(Util.getLeftOnScreen(nameX), Util.getTopOnScreen(nameY));
			talkBox.setLocation_Content(Util.getLeftOnScreen(contX), Util.getTopOnScreen(contY));
			talkBox.setName(id);
			cacheResourceMap.put(id, talkBox);
		} else if(readyType.equals("picture")) {
			int x = 0, y = 0;
			String imgPath = null;
			for (int i = 2; i < args.length; i++) {
				if (args[i].equals("-x")) {
					i++;
					x = Integer.parseInt(args[i]);
				} else if (args[i].equals("-y")) {
					i++;
					y = Integer.parseInt(args[i]);
				} else if(!args[i].startsWith("-")) {
					imgPath = args[i];
				}
			}
			imgPath=Util.getFilePath(imgPath);
			if(imgPath==null){
				Util.logError("Image file path is null.", null);
				return;
			}
			PictureBox picBox = new PictureBox(new DrawableHelper(imgPath));
			picBox.setLocation(Util.getLeftOnScreen(x), Util.getTopOnScreen(y));
			picBox.setName(id);
			cacheResourceMap.put(id, picBox);
		} else if(readyType.equals("weather")) {
			Weather.WeatherType type = null;
			int count = 10, layer = 9;
			for (int i = 2; i < args.length; i++) {
				if (args[i].equals("-l")) {
					i++;
					layer = Integer.parseInt(args[i]);
				} else if (args[i].equals("-c")) {
					i++;
					count = Integer.parseInt(args[i]);
				} else if(!args[i].startsWith("-")) {
					type = Weather.WeatherType.valueOf(args[i]);
				}
			}
			if(type==null){
				Util.logError("Can not parse the weather type", null);
				return;
			}
			Weather w = new Weather(type, count);
			w.setLayerIndex(layer);
			w.setName(id);
			cacheResourceMap.put(id, w);
		} else if(readyType.equals("frame")) {
			FrameAnimation.EndState state = null;
			String profilePath = null;
			String strx=null, stry=null;
			for (int i = 2; i < args.length; i++) {
				if (args[i].equals("-e")) {
					i++;
					state = FrameAnimation.EndState.valueOf(args[i]);
				} else if (args[i].equals("-x")) {
					i++;
					strx=args[i];
				} else if (args[i].equals("-y")) {
					i++;
					stry=args[i];
				} else if(!args[i].startsWith("-")) {
					profilePath = args[i];
				}
			}
			if(profilePath==null){
				Util.logError("Has no profile file path.", null);
				return;
			}
			FrameAnimation anime = new FrameAnimation(profilePath);
			if(state!=null)
				anime.setEndState(state);
			if(strx!=null || stry!=null){
				try {
					int x=anime.getX();
					if(strx!=null)
						x=Integer.parseInt(strx);
					int y=anime.getY();
					if(stry!=null)
						y=Integer.parseInt(stry);
					anime.setLocation(x, y);
				} catch (Exception e) {
					Util.logError("Fail to parse the animation's location.", e);
				}
			}
			anime.setName(id);
			cacheResourceMap.put(id, anime);
		} else if(readyType.equals("text")) {
			String text = null;
			int x=0, y=0, color=0;
			for (int i = 2; i < args.length; i++) {
				if (args[i].equals("-x")) {
					i++;
					x = Integer.parseInt(args[i]);
				} else if (args[i].equals("-y")) {
					i++;
					y = Integer.parseInt(args[i]);
				} else if (args[i].equals("-c")) {
					i++;
					color = new BigInteger(args[i],16).intValue();
				} else if(!args[i].startsWith("-")) {
					text = args[i];
				}
			}
			if(text==null){
				Util.logError("Text can not be empty.", null);
				return;
			}
			Label l = new Label(text);
			l.setLocation(Util.getLeftOnScreen(x), Util.getTopOnScreen(y));
			l.setTextColor(color);
			l.setName(id);
			cacheResourceMap.put(id, l);
		} else if(readyType.equals("track")) {
			TrackAnimation.EndState state = null;
			int frameCount = 1, layer = 8;
			String framePath = null;
			int sx=0, sy=0, ex=0, ey=0;
			double sw=1, sh=1, ew=1, eh=1;
			int sa=255, ea=255;
			for (int i = 2; i < args.length; i++) {
				if (args[i].equals("-sx")) {
					i++;
					sx = Integer.parseInt(args[i]);
				} else if (args[i].equals("-sy")) {
					i++;
					sy = Integer.parseInt(args[i]);
				} else if (args[i].equals("-ex")) {
					i++;
					ex = Integer.parseInt(args[i]);
				} else if (args[i].equals("-ey")) {
					i++;
					ey = Integer.parseInt(args[i]);
				} else if (args[i].equals("-sw")) {
					i++;
					sw = Double.parseDouble(args[i]);
				} else if (args[i].equals("-sh")) {
					i++;
					sh = Double.parseDouble(args[i]);
				} else if (args[i].equals("-ew")) {
					i++;
					ew = Double.parseDouble(args[i]);
				} else if (args[i].equals("-eh")) {
					i++;
					eh = Double.parseDouble(args[i]);
				} else if (args[i].equals("-sa")) {
					i++;
					sa = Integer.parseInt(args[i]);
				} else if (args[i].equals("-ea")) {
					i++;
					ea = Integer.parseInt(args[i]);
				} else if (args[i].equals("-l")) {
					i++;
					layer = Integer.parseInt(args[i]);
				} else if (args[i].equals("-e")) {
					i++;
					state = TrackAnimation.EndState.valueOf(args[i]);
				} else if(!args[i].startsWith("-")) {
					framePath = args[i];
				}
			}
			if(framePath==null){
				Util.logError("Frame image file path can not be null.", null);
				return;
			}
			TrackAnimation anime = new TrackAnimation(new DrawableHelper(framePath), frameCount);
			anime.setLocationTrack(Util.getLeftOnScreen(sx), Util.getTopOnScreen(sy), Util.getLeftOnScreen(ex), Util.getTopOnScreen(ey));
			anime.setPercentageTrack(sw, sh, ew, eh);
			anime.setAlphaTrack(sa, ea);
			anime.setLayerIndex(layer);
			if(state!=null)
				anime.setEndState(state);
			anime.setName(id);
			cacheResourceMap.put(id, anime);
		} else if(readyType.equals("select")){
			SelectBoxBean sbb = new SelectBoxBean();
			for (int i = 2; i < args.length; i++) {
				if (args[i].equals("-p")) {
					i++;
					sbb.setBgPic(args[i]);
				} else if (args[i].equals("-w")) {
					i++;
					sbb.setWidth(Integer.parseInt(args[i]));
				} else if (args[i].equals("-h")) {
					i++;
					sbb.setHeight(Integer.parseInt(args[i]));
				} else if (args[i].equals("-c")) {
					i++;
					sbb.setColor(new BigInteger(args[i],16).intValue());
				}
				cacheResourceMap.put(id, sbb);
			}
		} else {
			Util.logError("The resource type [" + readyType + "] is unknown", null);
		}
		Util.logInfo("Resource ["+readyType+"] with ID ["+id+"] already cached.");
	}
	
	private void HideLine(String id){
		Object cachedResource = cacheResourceMap.get(id);
		if(cachedResource==null){
			Util.logError("Resource with ID [" + id + "] not found.", null);
			return;
		}
		boolean result = false;
		if(cachedResource instanceof Component) {
			result = removeComponent((Component)cachedResource);
		} else if(cachedResource instanceof Control) {
			result = removeControl((Control)cachedResource);
		} else {
			Util.logError("the resource type with ID ["+id+"] is unknown", null);
			return;
		}
		if(result){
			Util.logInfo("Resource with ID ["+id+"] being hided.");
		}else{
			Util.logError("The resource with ID ["+id+"] fail be hidden.", null);
		}
	}
	
	private void ShowLine(String id){
		Object cachedResource = cacheResourceMap.get(id);
		if(cachedResource==null){
			Util.logError("The resource with id ["+id+"] not ready yet.", null);
			return;
		}
		if(cachedResource instanceof Component) {
			addComponent((Component)cachedResource);
			if(cachedResource instanceof TalkBox)
				this.talkBox = (TalkBox)cachedResource;
		} else if(cachedResource instanceof Control) {
			addControl((Control)cachedResource);
		} else {
			Util.logError("the resource with id ["+id+"] s type is unknown", null);
			return;
		}
		Util.logInfo("The resource with id ["+id+"] be shown.");
	}
	
	private void DestroyLine(String id){
		Object cachedResource = cacheResourceMap.get(id);
		if(cachedResource==null){
			Util.logError("The resource with id ["+id+"] not ready yet.", null);
			return;
		}
		if(cachedResource instanceof Component) {
			removeComponent((Component)cachedResource);
			((Component)cachedResource).destroy();
		} else if(cachedResource instanceof Control) {
			removeControl((Control)cachedResource);
			((Control)cachedResource).destroy();
		} else {
			Util.logError("the resource with id ["+id+"] s type is unknown", null);
			return;
		}
		cacheResourceMap.remove(id);
		Util.logInfo("The resource with id ["+id+"] being destroyed.");
	}

	private void MovieLine(String filePath) {
		Util.logInfo("Play movie ["+filePath+"]");
		Arena.playMovie(filePath);
	}

	private void SoundLine(String filePath) {
		Util.logInfo("Play sound ["+filePath+"]");
		playSound(filePath);
	}

	private void MusicLine(String filePath) {
		Arena.playMusic(filePath);
		Util.logInfo("Play music ["+filePath+"]");
	}

	private void EventLine(String[] args)
	{
		ArrayList<String> argsArray = new ArrayList<String>();
		boolean isWaiting = args[0].equals("-p");
		for(int i=isWaiting?2:1; i<args.length; i++)
			argsArray.add(args[i]);
		String[] newArgs = new String[argsArray.size()];
		argsArray.toArray(newArgs);
		
		Util.logInfo("Execute event: " + args[isWaiting?1:0]);
		executeEvent(args[isWaiting?1:0], isWaiting, newArgs);
	}
	
	private void SelectLine(String[] args){
		SelectBoxBean abb = null;
		ArrayList<SelectBean> selections = new ArrayList<SelectBean>();
		for (int i = 0; i < args.length; i++) {
			if (args[i].equals("-b")) {
				i++;
				abb=(SelectBoxBean) cacheResourceMap.get(args[i]);
			}else{
				String[] selectInfo = args[i].split("\\|");
				SelectBean sb = new SelectBean();
				sb.setTitle(selectInfo[0]);
				sb.setDramaPath(selectInfo[1]);
				selections.add(sb);
			}
		}
		SelectBox sBox = null;
		if(abb.getBgPic()!=null){
			sBox = new SelectBox(abb.getBgPic());
		}else{
			sBox = new SelectBox(abb.getWidth(), abb.getHeight(), abb.getColor());
		}
		sBox.setContainer(this);
		sBox.setSelections(selections.toArray(new SelectBean[0]));
		sBox.show();
	}


//	private void SelectLine(final String[] args)
//	{
//		if (args[0].equals("-b"))
//		{
//			int selectHeight = selectPanel.getHeight() / (args.length - 1);
//			for (int i = 1; i < args.length - 1; i++)
//			{
//				Button select = selectButtons.get(args[i]);
//				if (select == null)
//				{
//					Util.logError("The option [" + args[i] + "] could not found.", null);
//					continue;
//				}
//				if (selectPanel == null)
//				{
//					select.setLocation(talkBox.left_Content() + selectHeight * i, 0);
//					addControl(select);
//				}
//				else
//				{
//					select.setLocation((Arena.getInstance().getWidth() - select.getWidth()) / 2, (Arena.getInstance().getHeight() - selectPanel.getHeight()) / 2 + selectHeight * i);
//					selectPanel.addControl(select);
//					selectPanel.show();
////					setPanel(selectPanel);
//				}
//			}
//		}
//		else if (args[0].equals("-t"))
//		{
//			Button[] selects = new Button[args.length - 2];
//			int selectHeight = selectPanel.getHeight() / selects.length;
//			for (int i = 2; i < selects.length; i++)
//			{
//				final String drama = args[i].substring(args[i].indexOf("|") + 1);
//				final String caption = args[i].substring(0, args[i].indexOf("|"));
//				selects[i] = new Button(new DrawableHelper(
//						(Arena.getInstance().getWidth() - Util.calStringPixLength(caption)) / 2,
//						(Arena.getInstance().getHeight() - selectPanel.getHeight()) / 2 + selectHeight * i));
//				selects[i].addActionListener(new ActionListener() { public void onPerform() {
//					selectPanel.close();
//					loadDrama(drama);
//				}});
////				selects[i] = new Button(new DrawableHelper((Arena.getInstance().getWidth() - Util.calStringPixLength(caption)) / 2, (Arena.getInstance().getHeight() - selectPanel.getHeight()) / 2 + selectHeight * i))
////				{
////					public void onClick(int x, int y)
////					// selects[i] = new Button((Arena.getInstance().getWidth()-Util.calStringPixLength(caption, textSize))/2,
////					// (Arena.getInstance().getHeight()-selectPanel.getHeight())/2+selectHeight*i,
////					// selectPanel.getWidth(), selectHeight){ public void onClick(int x, int y)
////					{
//////						setPanel(null);
////						selectPanel.close();
////						loadDrama(drama);
////					}
////				};
//				selects[i].setText(caption);
//				if (selectPanel == null)
//				{
//					selects[i].setLocation(talkBox.left_Content() + selectHeight * i, 0);
//					addControl(selects[i]);
//				}
//				else
//				{
//					selects[i].setLocation((Arena.getInstance().getWidth() - selects[i].getWidth()) / 2, (Arena.getInstance().getHeight() - selectPanel.getHeight()) / 2 + selectHeight * i);
//					selectPanel.addControl(selects[i]);
////					setPanel(selectPanel);
//					selectPanel.show();
//				}
//			}
//		}
//		else
//		{
//			Util.logError("Miss some param, please use -b(use button) or -t(set text size).", null);
//		}
//	}

	private void GotoLine(String[] args) {
		if (args[0].equals("-l")) {
			loadDrama(args[2]);
			setDramaLineIndex(Integer.parseInt(args[1]));
			Util.logInfo("Goto drama ["+args[1]+"]");
		} else {
			loadDrama(args[0]);
			if(args[1].equals("-l"))
				setDramaLineIndex(Integer.parseInt(args[2]));
			Util.logInfo("Goto drama ["+args[0]+"]");
		}
		processNextLine();
	}
	
	private long waitingStartTime = 0;
	private void WaitLine(final String[] args) {
		if(args.length==1){
			Util.logInfo("Waiting for user input.");
			isProcessing = touchHanging;
			waitingStartTime=0;
			touchHanging = false;
		} else {
			if(args[1].equals("begin")){
				Util.logInfo("Begin for the next wait.");
				waitingStartTime = System.currentTimeMillis();
			}else{
				Util.logInfo("Waiting for the time.");
				isThreadWaiting=true;
				new Thread(){public void run(){
					synchronized (this) {
						try{
							long sleepTime = 0;
							if(waitingStartTime>0)
								sleepTime=Integer.parseInt(args[1])*1000/Util.getMaxFPS()-System.currentTimeMillis()+waitingStartTime;
							else
								sleepTime = Integer.parseInt(args[1])*1000/Util.getMaxFPS();
							Thread.sleep(sleepTime<1?1:sleepTime);
						}catch(Exception e){
							Util.logError("Waiting time ["+args[1]+"] is not number format", null);
						}
					}
					isThreadWaiting=false;
					Util.logInfo("Waiting done and process the next drama line.");
					processNextLine();
				}}.start();
				waitingStartTime=0;
			}
		}
	}
	
	private void PendLine(final String[] args) {
		isThreadWaiting=true;
		new Thread(){public void run(){
			if(args[0].equals("movie")){
				Util.logInfo("Waiting for the movie complete.");
				while(Arena.isMoviePlaying()){
					synchronized (this) {
						try {
							Thread.sleep(100);
						} catch (Exception e) { }
					}
				}
			}else if(args[0].equals("frame")){
				if(args.length==1){
					Util.logError("Pending frame with unknown ID", null);
				}else{
					FrameAnimation fa = (FrameAnimation) cacheResourceMap.get(args[1]);
					if(fa.getEndState()==FrameAnimation.EndState.LOOP){
						Util.logError("Frame ["+args[1]+"] is loop and would no end.", null);
					}else{
						Util.logInfo("Waiting for the frame ["+args[1]+"] complete.");
						while(fa.isPlaying()){
							synchronized (this) {
								try {
									Thread.sleep(100);
								} catch (Exception e) { }
							}
						}
					}
				}
			}else if(args[0].equals("track")){
				if(args.length==1){
					Util.logError("Pending track with unknown ID", null);
				}else{
					TrackAnimation ta = (TrackAnimation) cacheResourceMap.get(args[1]);
					if(ta.getEndState()==TrackAnimation.EndState.LOOP){
						Util.logError("Track ["+args[1]+"] is loop and would no end.", null);
					}else{
						Util.logInfo("Waiting for the track ["+args[1]+"] complete.");
						while(ta.isPlaying()){
							synchronized (this) {
								try {
									Thread.sleep(100);
								} catch (Exception e) { }
							}
						}
					}
				}
			}
			isThreadWaiting=false;
			Util.logInfo("Pending done and process the next drama line.");
			processNextLine();
		}}.start();
	}
	
	public void cacheResource(String id, Object res){
		cacheResourceMap.put(id, res);
	}
	
	public HashMap<String, Object> getAllCachedResources(){
		return cacheResourceMap;
	}

	@Override
	public void destroy() {
		super.destroy();
		for(int i=0; i<cacheResourceMap.size(); i++){
			Object cachedResource = cacheResourceMap.get(i);
			if(cachedResource instanceof Component) {
				((Component)cachedResource).destroy();
			} else if(cachedResource instanceof Control) {
				((Control)cachedResource).destroy();
			} else {
				Util.logError("Cached resource ["+cachedResource+"] is unknown type", null);
				return;
			}
		}
	}

}
