package wafly.framework.storyTK.tmpl;

import wafly.framework.storyTK.Arena;
import wafly.framework.storyTK.basic.DrawableHelper;
import wafly.framework.storyTK.basic.StoreMgr;
import wafly.framework.storyTK.basic.Util;

public abstract class Component
{
	protected long ID = (long) (System.currentTimeMillis()*1000+Math.random()*1000);
	private String name=null;

	/**
	 * 当按帧显示组件时发生事件
	 * @param draw
	 */
	abstract public void onDraw(DrawableHelper draw);

	/**
	 * 销毁一个组件
	 */
	abstract public void destroy();

	/**
	 * 组件是否可见
	 * @return
	 */
	abstract public boolean isVisible();

	/**
	 * 设置组件是否可见
	 * @param isVisible
	 */
	abstract public void setVisible(boolean isVisible);
	
	/**
	 * 获取组件ID
	 * @return
	 */
	public long getID(){ return ID; }
	
	public void setName(String name){ this.name=name; }
	
	public String getName(){ return name; }

	private int layerIndex = 9;

	/**
	 * 设置组件所在层
	 * @param layer 组件所在图层, 数值为0到9，数值越小的在下方
	 */
	public void setLayerIndex(int layer)
	{
		if(layer<0)
			layer=0;
		else if(layer>9)
			layer=9;
		this.layerIndex = layer;
		if(Arena.getScene().removeComponent(this)){
			Util.logInfo("Component's layer being changed to "+this.layerIndex);
			Arena.getScene().addComponent(this);
		}
//		ArrayList<Component> es = Arena.getScene().getComponents(index);
//		if(es!=null && !es.contains(this))
//			Arena.getScene().setComponentLayer(this, index);
	}

	/**
	 * 获取组件所在层
	 * @return
	 */
	public int getLayerIndex()
	{
		return this.layerIndex;
	}
	
	/**组件透明度*/
	protected int alpha=255;
	
	/**
	 * 设置组件透明度
	 * @param alpha
	 */
	public void setAlpha(int alpha)
	{
		this.alpha=alpha;
	}
	
	/**
	 * 获取组件透明度
	 * @return
	 */
	public int getAlpha()
	{
		return alpha;
	}

	/**
	 * 存储组件相关数据到磁盘文件
	 * @param storeMgr
	 */
	public void storeValue(StoreMgr storeMgr)
	{
		storeMgr.setValue("COMPONENT."+ID+".LAYERINDEX", layerIndex);
		storeMgr.setValue("COMPONENT."+ID+".ALPHA", alpha);
	}
	
	/**
	 * 通过存储器从一个磁盘文件中恢复一个组件（通常不需要手动调用）
	 * @param id
	 * @param storeMgr
	 */
	public Component(long id, StoreMgr storeMgr)
	{
		ID=id;
		layerIndex = Integer.parseInt(storeMgr.getValue("COMPONENT."+ID+".LAYERINDEX"));
		alpha = Integer.parseInt(storeMgr.getValue("COMPONENT."+ID+".ALPHA"));
		Arena.getScene().addComponent(this);//.setComponentLayer(this, layerIndex);
	}
	
	protected Component() {}
}