package wafly.framework.storyTK.tmpl;

import wafly.framework.storyTK.basic.DrawableHelper;
import wafly.framework.storyTK.basic.StoreMgr;

/**
 * The basic of all the control
 * @author Wafly
 */
public abstract class Control
{
	/**
	 * the real size/position on screen
	 */
	protected int width = 1, height = 1;
	protected int location_X = 0, location_Y = 0;
	protected int relaX = 0, relaY = 0;
	
	protected long ID = 0;
	private String name=null;
	
	protected Control(){};
	
	public Control(long id, StoreMgr storeMgr)
	{
		ID=id;
		width=Integer.parseInt(storeMgr.getValue("CONTROL."+ID+".WIDTH"));
		height=Integer.parseInt(storeMgr.getValue("CONTROL."+ID+".HEIGHT"));
		location_X=Integer.parseInt(storeMgr.getValue("CONTROL."+ID+".LOCATION.X"));
		location_Y=Integer.parseInt(storeMgr.getValue("CONTROL."+ID+".LOCATION.Y"));
		alpha=Integer.parseInt(storeMgr.getValue("CONTROL."+ID+".ALPHA"));
	}
	
	/**
	 * 从指定存储器中恢复一个控件
	 * @param storeMgr
	 */
	public void storeValue(StoreMgr storeMgr)
	{
		storeMgr.setValue("CONTROL."+ID+".WIDTH", width);
		storeMgr.setValue("CONTROL."+ID+".HEIGHT", height);
		storeMgr.setValue("CONTROL."+ID+".LOCATION.X", location_X);
		storeMgr.setValue("CONTROL."+ID+".LOCATION.Y", location_Y);
		storeMgr.setValue("CONTROL."+ID+".ALPHA", alpha);
	}

	/**
	 * 设置此控件的相对参考点坐标，此数值影响Location，默认为原点(0,0)
	 * @param x
	 * @param y
	 */
	public void setRelative(int x, int y){
		relaX=x;
		relaY=y;
	}
	
	public void setName(String name){ this.name=name; }
	
	public String getName(){ return name; }
	
	/**
	 * when destory() it will become true
	 */
	protected boolean destoryed=false;
	protected int alpha=255;
	protected boolean visible = true;
	
	public long getID() { return ID; }

	/**
	 * 将控件画到画板上<br>
	 * to draw the control to scene or panel
	 */
	public abstract void onDraw(DrawableHelper draw);
	
	/**
	 * invoke this method to destory all the bitmap resource
	 */
	public abstract void destroy();
	
	/**
	 * to get the location x
	 * @return
	 */
	public int left() { return location_X; }
	
	/**
	 * to get the location y
	 * @return
	 */
	public int top() { return location_Y; }
	
	/**
	 * 获取此控件的横坐标位置
	 * @return
	 */
	public int getX() { return location_X; }
	
	/**
	 * 获取此控件纵坐标位置
	 * @return
	 */
	public int getY() { return location_Y; }
	
	/**
	 * to get the width of the control
	 * @return the width of the control
	 */
	public int getWidth() { return width; }
	
	/**
	 * to get the height of the control
	 * @return the height of the control
	 */
	public int getHeight() { return height; }
	
	/**
	 * to set the location of the control
	 * @param x left from game's left
	 * @param y top from game's top
	 */
	public void setLocation(int x, int y)
	{
		this.location_X = x;
		this.location_Y = y;
	}

	/**
	 * to set the transparency of the control
	 * @param alpha transparency
	 */
	public void setAlpha(int alpha) { this.alpha=alpha; }
	
	/**
	 * @return the transparency of the control
	 */
	public int getAlpha() { return alpha; }
	
	/**
	 * 设置控件是否可见
	 * @param visible
	 */
	public void setVisible(boolean visible) { this.visible=visible; }
	
	/**
	 * 判断控件当前是否可见
	 * @return
	 */
	public boolean isVisible() { return visible; }
}
