package wafly.framework.storyTK.tmpl;

import java.util.ArrayList;
import wafly.framework.storyTK.basic.DrawableHelper;
import wafly.framework.storyTK.basic.Util;
import wafly.framework.storyTK.component.TrackAnimation;
import wafly.framework.storyTK.control.FrameAnimation;
import wafly.framework.storyTK.control.Sprite;
import wafly.framework.storyTK.listener.HitListener;

public abstract class Hit extends Component{
	private Sprite owner = null;
	private long actDelay = 0;
	private TrackAnimation trackAction = null;
	private FrameAnimation frameAction = null;
	private boolean isOnAction = false;
	private boolean destroyed = false;
	
	public Hit(Sprite actOwner){
		this.owner=actOwner;
	}
	
	public void takeAction(){
		if(isOnAction)
			return;
		int hx=owner.location_X,hy=owner.location_Y,hw=0,hh=0;
		if(frameAction!=null){
			frameAction.reset();
			frameAction.setLocation(hx, hy);
			hw=frameAction.width;
			hh=frameAction.height;
		}
		if(trackAction!=null){
			TargetPoint tp = getHitTargetPoint(owner);
			trackAction.reset();
			if(tp!=null){
				trackAction.setLocationTrack(owner.location_X, owner.location_Y, tp.x, tp.y);
			}else{
				Util.logError("Error hit target because of HitTargetPoint unset", null);
			}
		}
		final Sprite[] targets = getHitTargetSprites(owner, hx, hy, hw, hh);
		if(targets!=null && targets.length>0){
			new Thread(){public void run(){
				synchronized (this) { try {
					Thread.sleep(actDelay);
				} catch (Exception e) {}}
				for(int i=0; i<targets.length; i++){
					ArrayList<HitListener> ls = targets[i].getHitListeners();
					for(HitListener l:ls)
						l.onBeingHit(Hit.this, owner);
				}
			}}.start();
		}
		owner.performAnimation(getOwnerPerformentsFrame());
		ArrayList<HitListener> ls = owner.getHitListeners();
		for(HitListener l:ls)
			l.onHit(this);
		isOnAction=true;
	}
	
	protected abstract Sprite[] getHitTargetSprites(Sprite actionOwner, int hitAreaLeft, int hitAreaTop, int hitAreaWidth, int hitAreaHeight);
	/**只有在设置了轨迹打击动画后才需要在这里判断打击坐标*/
	protected abstract TargetPoint getHitTargetPoint(Sprite actionOwner);
	protected abstract FrameAnimation getOwnerPerformentsFrame();
	
	public Sprite getActionOwner(){ return owner; }
	
	/**设置区域打击动画*/
	public void setMapHit(FrameAnimation hitPerformFrame, long effectiveDelay){
		if(frameAction!=null)
			frameAction.destroy();
		this.frameAction=hitPerformFrame;
		this.frameAction.setEndState(FrameAnimation.EndState.HIDE);
		this.actDelay=effectiveDelay;
	}
	
	public FrameAnimation getMapHit(){ return frameAction; }

	/**设置轨迹打击动画*/
	public void setTrackHit(TrackAnimation hitPerformTrack){
		if(trackAction!=null)
			trackAction.destroy();
		this.trackAction=hitPerformTrack;
		this.trackAction.setEndState(TrackAnimation.EndState.HIDE);
	}

	@Override
	public void onDraw(DrawableHelper draw){
		if(!isOnAction || destroyed)
			return;
		if(frameAction!=null){
			frameAction.onDraw(draw);
		}
		if(trackAction!=null){
			trackAction.onDraw(draw);
			ArrayList<Sprite> beHitSprites = owner.getScene().getCollideSprites(
					trackAction.getCurrentLeft()+trackAction.getCurrentWidth()/2,
					trackAction.getCurrentTop()+trackAction.getCurrentHeight()/2, owner);
			if(beHitSprites.size()>0){
				for(Sprite hs:beHitSprites){
					ArrayList<HitListener> ls = hs.getHitListeners();
					for(HitListener l:ls)
						l.onBeingHit(Hit.this, owner);
				}
				trackAction.stop();
			}
		}
		if(!frameAction.isPlaying() && !trackAction.isPlaying())
			isOnAction=false;
	}
	
	public void destroy(){
		destroyed=true;
		if(frameAction!=null)
			frameAction.destroy();
		if(trackAction!=null)
			trackAction.destroy();
	}
	
	public class TargetPoint{
		public int x,y;
		public TargetPoint(int x, int y){
			this.x=x; this.y=y;
		}
	}

	@Override
	public boolean isVisible() { return true; }

	@Override
	public void setVisible(boolean isVisible) { }
}
