package wafly.framework.storyTK.tmpl;

import java.util.ArrayList;
import wafly.framework.storyTK.basic.StoreMgr;
import wafly.framework.storyTK.basic.Util;
import wafly.framework.storyTK.listener.ActionListener;
import wafly.framework.storyTK.listener.BlurListener;
import wafly.framework.storyTK.listener.DragListener;
import wafly.framework.storyTK.listener.FocusListener;
import wafly.framework.storyTK.listener.TouchListener;
import wafly.framework.storyTK.listener.TouchReleaseListener;

/**
 * if you want to take the control could be operated, shound be extends this class
 * @author Wafly
 */
public abstract class OperationalControl extends Control
{
	private ArrayList<TouchListener> touchListenerList = new ArrayList<TouchListener>();
	private ArrayList<TouchReleaseListener> releaseListenerList = new ArrayList<TouchReleaseListener>();
	private ArrayList<DragListener> dragListenerList = new ArrayList<DragListener>();
	private ArrayList<FocusListener> focusListenerList = new ArrayList<FocusListener>();
	private ArrayList<BlurListener> blurListenerList = new ArrayList<BlurListener>();
	
	/**
	 * when touch the control, will take affect this event
	 * @param x touch point x
	 * @param y touch point y
	 */
	public final boolean onTouch(final int x, final int y){
		if(x<relaX+this.location_X
				|| y<relaY+this.location_Y
				|| x>relaX+(this.location_X+this.width)
				|| y>relaY+(this.location_Y+this.height)
				|| currentStatus == Status.DISABLE
				|| currentStatus == Status.HIDE){
			if(currentStatus==Status.DOWN || currentStatus==Status.FOCUS)
				currentStatus=Status.NORMAL;
			return false;
		}
		new Thread(){public void run(){
			setStatus(Status.DOWN);
			Util.logInfo("OperationalControl being touched.");
			for(TouchListener listener : touchListenerList){
				listener.onPerform(x, y);
			}
		}}.start();
		return true;
	}
	
	public void addTouchListener(TouchListener listener){
		this.touchListenerList.add(listener);
	}
	public ArrayList<TouchListener> getTouchListeners(){
	    return touchListenerList;
	}
	
	public void addReleaseListener(TouchReleaseListener listener){
		this.releaseListenerList.add(listener);
	}
	public ArrayList<TouchReleaseListener> getReleaseListeners(){
	    return releaseListenerList;
	}
	
	public void addDragListener(DragListener listener){
		this.dragListenerList.add(listener);
	}
	public ArrayList<DragListener> getDragListeners(){
	    return dragListenerList;
	}
	
	public void addFocusListener(FocusListener listener){
		this.focusListenerList.add(listener);
	}
	public ArrayList<FocusListener> getFocusListeners(){
	    return focusListenerList;
	}
	
	public void addBlurListener(BlurListener listener){
		this.blurListenerList.add(listener);
	}
	public ArrayList<BlurListener> getBlurListeners(){
	    return blurListenerList;
	}
	
	/**
	 * when leave the control, will take affect this event
	 * @param x release point x
	 * @param y release point y
	 */
	public final boolean onRelease(final int x, final int y){
		if(x<relaX+this.location_X
				|| y<relaY+this.location_Y
				|| x>relaX+(this.location_X+this.width)
				|| y>relaY+(this.location_Y+this.height)
				|| getStatus() == Status.DISABLE
				|| getStatus() == Status.HIDE){
			if(currentStatus==Status.DOWN || currentStatus==Status.FOCUS)
				currentStatus=Status.NORMAL;
			return false;
		}
		new Thread(){public void run(){
		setStatus(Status.DOWN);
		for(TouchReleaseListener listener : releaseListenerList){
			listener.onPerform(x, y);
		}
		}}.start();
		return true;
	}
	
	/**
	 * when move on the control, will take affect this event
	 * @param x current point x
	 * @param y current point y
	 */
	public final void onDrag(int x, int y){
		if(getStatus() == Status.DISABLE
				|| getStatus() == Status.HIDE)
			return;
		for(DragListener listener : dragListenerList){
			listener.onPerform(x, y);
		}
		return;
	}
	
	protected ArrayList<ActionListener> actionListenerList = new ArrayList<ActionListener>();
	/**
	 * 添加一个控件默认动作的事件监听器，如Button是OnClick，ValueBar是OnValueChanged
	 * @param listener
	 */
	public final void addActionListener(ActionListener listener) {
	    if(listener==null)
	        return;
	    actionListenerList.add(listener);
	}
	public final ArrayList<ActionListener> getActionListeners(){
	    return actionListenerList;
	}

	/**
	 * the status for operational control
	 */
	public enum Status
	{

		/**
		 * 通常状态
		 */
		NORMAL,
		/**
		 * 按住控件时的状态
		 */
		DOWN,
		/**
		 * 控件处于选中状态
		 */
		FOCUS,
		/**
		 * 控件不可用
		 */
		DISABLE,
		/**
		 * 控件隐藏起来了
		 */
		HIDE
	}
	
	/**
	 * the current status of operational control
	 */
	protected Status currentStatus = Status.NORMAL;
	
	/**
	 * to get the current status of control
	 * @return current status
	 */
	public Status getStatus() { return currentStatus; }
	
	/**
	 * to get the scope which could be perform event in it
	 * @return touch scope rect
	 */
//	public Rect getTouchScope()
//	{
//		int tlx=relaX+location_X+Arena.getInstance().getLeft(), tly=relaY+location_Y+Arena.getInstance().getTop();
//		return new Rect(tlx, tly, tlx+width, tly+height);
//	}
	
	/**
	 * to disable or enable the control
	 * @param isDisabled set true if you want to disable the control, otherwise set false
	 */
	public void setDisabled(boolean isDisabled)
	{
		if(isDisabled)
			currentStatus=Status.DISABLE;
		else
			currentStatus=Status.NORMAL;
	}
	
	/**
	 * to get the disabled state
	 * @return true if disable, or return false
	 */
	public boolean isDisabled() { return currentStatus==Status.DISABLE; }
	
	/**
	 * to hide or show the control
	 * @param isVisible set false if you want to hide the control, otherwise set true
	 */
	public void setVisible(boolean isVisible)
	{
		if(isVisible)
			currentStatus=Status.NORMAL;
		else
			currentStatus=Status.HIDE;
	}
	
	/**
	 * to get the visible state
	 * @return false if hide, or return true
	 */
	public boolean isVisible() { return currentStatus==Status.HIDE; }
	
	/**
	 * set the status for the control
	 * @param status control's status
	 */
	public void setStatus(Status status) { this.currentStatus=status; }

	/**
	 * 设置当前控件获取输入焦点或失去输入焦点
	 */
	public void setFocus(boolean focus)
	{
		if(focus)
		{
			this.currentStatus=Status.FOCUS;
			for(FocusListener l:focusListenerList){
				l.onPerform();
			}
		}
		else
		{
			this.currentStatus=Status.NORMAL;
			for(BlurListener l:blurListenerList){
				l.onPerform();
			}
		}
	}

	/**
	 * 判断当前控件是否获取输入焦点
	 */
	public boolean isFocus()
	{
		return this.currentStatus == Status.FOCUS;
	}

	@Override
	public void storeValue(StoreMgr storeMgr)
	{
		super.storeValue(storeMgr);
		storeMgr.setValue("OPERATIONALCONTROL."+ID+".STATUS", currentStatus);
	}
	
	/**
	 * 指定ID在存储器中恢复一个控件，通常不需要手动调用
	 * @param id
	 * @param storeMgr
	 */
	public OperationalControl(long id, StoreMgr storeMgr)
	{
		super(id, storeMgr);
		currentStatus=Status.valueOf(storeMgr.getValue("OPERATIONALCONTROL."+ID+".STATUS"));
	}
	
	protected OperationalControl(){}

}
